'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-cd75521b.js');

const sasPaginationCss = ":host{display:block}:host ul{display:inline-flex;gap:5px;padding:0}:host li{list-style:none}:host sas-button{--button-border-radius:50%;--button-border:1px solid var(--s-200);--button-color:var(--pagination-text);--button-width:29px;--button-height:29px;--button-padding:0;--button-font-size:12px}:host sas-button[disabled]:not([disabled=false]){--button-color:var(--button-disabled-text)}:host sas-button.pagination-active{--button-background-color:#ECEFF1}:host #prev-icon,:host #next-icon{--icon-size:10px}";

let SasPagination = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.paginationUpdate = index.createEvent(this, "paginationUpdate", 7);
    this.paginationupdate = index.createEvent(this, "paginationupdate", 7);
    this.validTotalIndex = this.getValidTotalIndex();
    /**
     * Total Pages
     */
    this.totalPage = 100;
    /**
     * Total Visible Indexes, odd number only
     */
    this.totalIndex = 9;
    /**
     * Current Page Index
     */
    this.currentPage = 0;
    this.updateCurrentPage = (event) => {
      this.currentPage = event.target.dataset.index;
    };
    this.goToPrevPage = () => {
      this.currentPage -= 1;
    };
    this.goToNextPage = () => {
      this.currentPage += 1;
    };
  }
  watchHandler(newValue) {
    this.addActiveClass(newValue);
    this.updateControlButtonStatus(newValue);
    this.paginationUpdate.emit(newValue);
    this.paginationupdate.emit(newValue);
  }
  componentDidLoad() {
    this.addActiveClass(this.currentPage);
    this.updateControlButtonStatus(this.currentPage);
  }
  getValidTotalIndex() {
    if (this.totalIndex % 2 === 0) {
      return this.totalIndex + 1;
    }
    return this.totalIndex;
  }
  updateControlButtonStatus(currentPage) {
    const prevButton = this.el.shadowRoot.querySelector('#prev-icon');
    const nextButton = this.el.shadowRoot.querySelector('#next-icon');
    prevButton.disabled = false;
    nextButton.disabled = false;
    if (currentPage === 0) {
      prevButton.disabled = true;
    }
    if (currentPage === this.totalPage - 1) {
      nextButton.disabled = true;
    }
  }
  generatePageIndexButtons() {
    const pageIndexButtons = [];
    let listItem;
    if (this.validTotalIndex >= this.totalPage) {
      for (let i = 0; i < this.totalPage; i += 1) {
        listItem = index.h("li", null, index.h("sas-button", { type: "text", text: `${i + 1}`, "data-index": i, onClick: this.updateCurrentPage }));
        pageIndexButtons.push(listItem);
      }
    }
    else {
      const divider = Math.floor(this.validTotalIndex / 2);
      if (this.currentPage < divider) { // show right ellipsis
        for (let i = 0; i < this.validTotalIndex; i += 1) {
          listItem = index.h("li", null, index.h("sas-button", { type: "text", text: `${i + 1}`, "data-index": i, onClick: this.updateCurrentPage }));
          if (i === this.validTotalIndex - 2) {
            listItem = index.h("li", null, index.h("sas-button", { type: "text", text: '...', disabled: true }));
          }
          else if (i === this.validTotalIndex - 1) {
            listItem = index.h("li", null, index.h("sas-button", { type: "text", text: `${this.totalPage}`, "data-index": this.totalPage - 1, onClick: this.updateCurrentPage }));
          }
          pageIndexButtons.push(listItem);
        }
      }
      else if (this.currentPage < this.totalPage - divider) { // show left and right ellipsis
        for (let i = 0; i < this.validTotalIndex; i += 1) {
          listItem = index.h("li", null, index.h("sas-button", { type: "text", text: `${this.currentPage - divider + i + 1}`, "data-index": this.currentPage - divider + i, onClick: this.updateCurrentPage }));
          if (i === 0) {
            listItem = index.h("li", null, index.h("sas-button", { type: "text", text: '1', "data-index": 0, onClick: this.updateCurrentPage }));
          }
          else if (i === 1) {
            listItem = index.h("li", null, index.h("sas-button", { type: "text", text: '...', disabled: true }));
          }
          else if (i === this.validTotalIndex - 2) {
            listItem = index.h("li", null, index.h("sas-button", { type: "text", text: '...', disabled: true }));
          }
          else if (i === this.validTotalIndex - 1) {
            listItem = index.h("li", null, index.h("sas-button", { type: "text", text: `${this.totalPage}`, "data-index": this.totalPage - 1, onClick: this.updateCurrentPage }));
          }
          pageIndexButtons.push(listItem);
        }
      }
      else if (this.currentPage < this.totalPage) { // show left ellipsis
        for (let i = 0; i < this.validTotalIndex; i += 1) {
          listItem = index.h("li", null, index.h("sas-button", { type: "text", text: `${this.totalPage - this.validTotalIndex + i + 1}`, "data-index": this.totalPage - this.validTotalIndex + i, onClick: this.updateCurrentPage }));
          if (i === 0) {
            listItem = index.h("li", null, index.h("sas-button", { type: "text", text: '1', "data-index": 0, onClick: this.updateCurrentPage }));
          }
          else if (i === 1) {
            listItem = index.h("li", null, index.h("sas-button", { type: "text", text: '...', disabled: true }));
          }
          pageIndexButtons.push(listItem);
        }
      }
    }
    return pageIndexButtons;
  }
  addActiveClass(index) {
    this.removeActiveClass();
    const clickedButton = this.getButton(index);
    if (Boolean(clickedButton)) {
      clickedButton.classList.add('pagination-active');
    }
  }
  removeActiveClass() {
    const buttons = this.el.shadowRoot.querySelectorAll('sas-button');
    buttons.forEach((item) => {
      item.classList.remove('pagination-active');
    });
  }
  getButton(index) {
    const divider = Math.floor(this.validTotalIndex / 2);
    // in two ellipsis condition, force to select the middle button
    if (this.currentPage >= divider && this.currentPage < this.totalPage - divider) {
      const buttons = this.el.shadowRoot.querySelectorAll('sas-button');
      return buttons[divider + 1];
    }
    return this.el.shadowRoot.querySelector(`sas-button[data-index='${index}']`);
  }
  render() {
    return (index.h(index.Host, null, index.h("ul", null, index.h("li", null, index.h("sas-button", { type: "text", id: "prev-icon", text: "", icon: "fas fa-chevron-left", onClick: this.goToPrevPage })), this.generatePageIndexButtons(), index.h("li", null, index.h("sas-button", { type: "text", id: "next-icon", text: "", icon: "fas fa-chevron-right", onClick: this.goToNextPage })))));
  }
  get el() { return index.getElement(this); }
  static get watchers() { return {
    "currentPage": ["watchHandler"]
  }; }
};
SasPagination.style = sasPaginationCss;

exports.sas_pagination = SasPagination;
