'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-cd75521b.js');

const sasRadioGroupCss = ":host .container{display:grid;row-gap:var(--radio-group-row-gap, 5px);position:relative}:host .label{color:var(--secondary);font-size:14px;position:absolute;left:0;bottom:100%;transform:translateY(-10px)}:host .required-symbol{margin-left:5px;color:var(--primary)}:host(:not([label=\"\"])) .container{margin-top:30px}";

let SasRadioGroup = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.radioGroupUpdate = index.createEvent(this, "radioGroupUpdate", 7);
    this.radiogroupupdate = index.createEvent(this, "radiogroupupdate", 7);
    /**
     * name
     */
    this.name = null;
    /**
     * Label
     */
    this.label = '';
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * Checked value
     */
    this.value = null;
    /**
     * Default value
     */
    this.defaultValue = null;
    /**
     * Checked text
     */
    this.text = null;
    /**
     * Complete checked data
     */
    this.rawData = null;
    /**
     * Options
     */
    this.options = [];
  }
  watchValueHandler() {
    const radioElements = this.getRadioElements();
    // not validate when no options and list items, as in this case, options are not loaded
    if (this.options.length !== 0 || radioElements.length !== 0) {
      const isValid = this.validateValue();
      if (!isValid) {
        this.value = this.defaultValue;
      }
    }
    const hasShadowDom = this.checkShadowDom();
    if (!hasShadowDom) {
      // update check status in light dom
      this.updateCheckStatus();
    }
    this.text = this.getText();
    this.rawData = this.updateRawData();
  }
  watchOptionsHandler() {
    if (this.options.length > 0) {
      const isValid = this.validateValue();
      if (!isValid) {
        this.value = this.defaultValue;
        this.text = this.getText();
      }
    }
    else {
      this.value = null;
      this.text = this.getText();
    }
  }
  componentWillLoad() {
    // if in light dom and has pre select element
    const hasShadowDom = this.checkShadowDom();
    if (!hasShadowDom) {
      const checkedElement = this.getCheckedElement();
      if (checkedElement) {
        this.value = checkedElement.value;
        this.text = checkedElement.text;
        this.rawData = this.updateRawData();
      }
    }
    else { // if in shadow dom, validate value
      if (this.value !== null) {
        const isValid = this.validateValue();
        if (!isValid) {
          this.value = this.defaultValue;
          this.text = this.getText();
          this.rawData = this.updateRawData();
        }
      }
    }
  }
  radioUpdateHandler(event) {
    if (this.value !== event.detail.value) {
      const hasShadowDom = this.checkShadowDom();
      if (!hasShadowDom) {
        // update check status in light dom
        this.updateCheckStatus();
      }
      // update value
      this.value = event.detail.value;
      // update text
      this.text = event.detail.text;
      // update raw data
      this.rawData = this.updateRawData();
      // emit event
      this.radioGroupUpdate.emit(event.detail);
      this.radiogroupupdate.emit(event.detail);
    }
  }
  /**
   * Trigger radiogroupupdate event programmatically
   */
  async update() {
    const text = this.getText();
    const obj = {
      value: this.value,
      text: text,
    };
    this.radioGroupUpdate.emit(obj);
    this.radiogroupupdate.emit(obj);
  }
  getText() {
    const hasShadowDom = this.checkShadowDom();
    if (this.value === null) {
      return null;
    }
    else {
      if (hasShadowDom) {
        let result;
        this.options.forEach((option) => {
          if ('sub' in option) {
            option.sub.forEach((sub) => {
              if (this.value === sub.value) {
                result = sub;
              }
            });
          }
          else {
            if (this.value === option.value) {
              result = option;
            }
          }
        });
        return result.text;
      }
      else {
        const checkedElement = this.getCheckedElement();
        return checkedElement.text;
      }
    }
  }
  validateValue() {
    const hasShadowDom = this.checkShadowDom();
    let isFound = false;
    if (hasShadowDom) {
      this.options.forEach((option) => {
        if ('sub' in option) {
          option.sub.forEach((sub) => {
            if (sub.value === this.value) {
              isFound = true;
            }
          });
        }
        else {
          if (option.value === this.value) {
            isFound = true;
          }
        }
      });
    }
    else {
      const radioElements = this.getRadioElements();
      radioElements.forEach((radioElement) => {
        if (radioElement.value === this.value) {
          isFound = true;
        }
      });
    }
    return isFound;
  }
  checkShadowDom() {
    if (this.options.length > 0) {
      return true;
    }
    return false;
  }
  updateCheckStatus() {
    const radioElements = this.getRadioElements();
    radioElements.forEach((radioElement) => {
      if (radioElement.value === this.value) {
        radioElement.checked = true;
      }
      else {
        radioElement.checked = false;
      }
    });
  }
  updateRawData() {
    const value = this.value;
    const text = this.getText();
    let rawData;
    const hasShadowDom = this.checkShadowDom();
    if (hasShadowDom) {
      this.options.forEach((option) => {
        if ('sub' in option) {
          option.sub.forEach((sub) => {
            if (this.value === sub.value) {
              rawData = {
                value: option.value,
                text: option.text,
                sub: {
                  value: value,
                  text: text,
                },
              };
            }
          });
        }
        else {
          if (this.value === option.value) {
            rawData = {
              value: value,
              text: text,
            };
          }
        }
      });
    }
    else {
      const checkedElement = this.getCheckedElement();
      if (checkedElement) {
        rawData = {
          value: value,
          text: text,
        };
      }
    }
    return rawData;
  }
  getRadioElements() {
    return this.el.querySelectorAll('sas-radio');
  }
  getCheckedElement() {
    const radioElements = this.getRadioElements();
    let checkedElement = null;
    radioElements.forEach((radioElement) => {
      if (radioElement.value === this.value || radioElement.checked) {
        checkedElement = radioElement;
      }
    });
    return checkedElement;
  }
  getRadioGroupElements() {
    let elements = [];
    this.options.forEach((option) => {
      let isChecked = false;
      if (this.value !== null && option.value === this.value) {
        isChecked = true;
      }
      let element = index.h("sas-radio", { value: option.value, text: option.text, checked: isChecked, disabled: option.disabled });
      elements.push(element);
    });
    return elements;
  }
  render() {
    return (index.h(index.Host, null, index.h("div", { class: "container" }, Boolean(this.label)
      ? index.h("label", { class: "label" }, this.label, this.required
        ? index.h("span", { class: "required-symbol" }, "*")
        : null)
      : null, this.getRadioGroupElements(), index.h("slot", null))));
  }
  get el() { return index.getElement(this); }
  static get watchers() { return {
    "value": ["watchValueHandler"],
    "options": ["watchOptionsHandler"]
  }; }
};
SasRadioGroup.style = sasRadioGroupCss;

exports.sas_radio_group = SasRadioGroup;
