'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-cd75521b.js');

const sasTimeFieldCss = ":host{display:block;}:host sas-form-input-template{--form-input-template-container-min-width:none;--form-input-template-container-max-width:380px;--input-height:var(--time-field-height, 40px)}:host .time-field{padding:20px;display:grid;grid-template-columns:repeat(3, 1fr);column-gap:10px}:host .time-field:focus{outline:none}:host .part::-webkit-scrollbar{display:none}:host .part{-ms-overflow-style:none;scrollbar-width:none;}:host sas-list{max-height:200px;overflow-y:auto;position:relative}:host sas-list-item{margin-bottom:6px;--list-item-border-radius:5px;--list-item-padding:12px 40px;--list-item-text-align:center}:host sas-list-item[selected]:not([selected=false]){--list-item-selected-background-color:white;--list-item-selected-border:1px solid var(--s-300);--list-item-selected-border-left:1px solid var(--s-300)}";

let SasTimeField = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.timeFieldUpdate = index.createEvent(this, "timeFieldUpdate", 7);
    this.timefieldupdate = index.createEvent(this, "timefieldupdate", 7);
    this.isUpdateTimeFieldList = true;
    /**
     * Label
     */
    this.label = '';
    /**
     * Label position
     */
    this.outside = false;
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * name
     */
    this.name = '';
    /**
     * Value
     */
    this.value = '';
    /**
     * Max value
     */
    this.max = '';
    /**
     * Min value
     */
    this.min = '';
    /**
     * disabled
     */
    this.disabled = false;
    /**
     * Clear
     */
    this.clear = false;
    /**
     * Default container position
     */
    this.position = null;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
      rangeUnderflow: false,
      rangeOverflow: false,
    };
    /**
     * Only shows when prop error is true, auto generated
     */
    this.message = '';
  }
  watchHandler() {
    if (this.isUpdateTimeFieldList) {
      this.updateTimeFieldList();
    }
    else {
      this.isUpdateTimeFieldList = true;
    }
    this.resetErrorStatus();
  }
  componentDidLoad() {
    this.infiniteScroll();
  }
  inputUpdateHandler(event) {
    this.value = event.detail;
    this.timeFieldUpdate.emit(event.detail);
    this.timefieldupdate.emit(event.detail);
  }
  inputContainerClickHandler() {
    this.updateTimeFieldList();
  }
  listItemClickedHandler() {
    const hourList = this.el.shadowRoot.querySelector('#hour');
    const minuteList = this.el.shadowRoot.querySelector('#minute');
    const dayPartList = this.el.shadowRoot.querySelector('#day-part');
    if (Boolean(hourList.value) && Boolean(minuteList.value) && Boolean(dayPartList.value)) {
      const currentValue = this.convertTo24Hour(this.formatValue(hourList.value), this.formatValue(minuteList.value), dayPartList.value);
      if (this.value !== currentValue) {
        this.isUpdateTimeFieldList = false;
        this.value = currentValue;
        this.timeFieldUpdate.emit(currentValue);
        this.timefieldupdate.emit(currentValue);
      }
    }
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async validate() {
    let valueMissing = false;
    if (this.required && !Boolean(this.value)) {
      valueMissing = true;
    }
    let rangeUnderflow = false;
    if (Boolean(this.min) && Boolean(this.value)) {
      if (this.value < this.min) {
        rangeUnderflow = true;
      }
    }
    let rangeOverflow = false;
    if (Boolean(this.max) && Boolean(this.value)) {
      if (this.value > this.max) {
        rangeOverflow = true;
      }
    }
    this.validity = {
      valid: !valueMissing && !rangeUnderflow && !rangeOverflow,
      valueMissing: valueMissing,
      rangeUnderflow: rangeUnderflow,
      rangeOverflow: rangeOverflow,
    };
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    await this.validate();
    const formInputTemplate = this.el.shadowRoot.querySelector('sas-form-input-template');
    if (Boolean(formInputTemplate)) {
      formInputTemplate.error = !this.validity.valid;
    }
    if (!this.validity.valid) {
      this.message = this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    const formInputTemplate = this.el.shadowRoot.querySelector('sas-form-input-template');
    if (Boolean(formInputTemplate)) {
      formInputTemplate.error = false;
    }
    this.message = '';
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      return `${this.label} is required`;
    }
    if (validity.rangeOverflow) {
      return `${this.label} must be less or equal to ${this.max}`;
    }
    if (validity.rangeUnderflow) {
      return `${this.label} must be larger or equal to ${this.min}`;
    }
  }
  infiniteScroll() {
    const observer = new IntersectionObserver((entries) => {
      entries.forEach((entry) => {
        if (entry.intersectionRatio > 0) {
          const topCurrent = entry.boundingClientRect.top;
          const parent = entry.target.parentElement;
          const value = entry.target.value;
          parent.scrollTop = 0;
          const targetElement = parent.querySelector(`sas-list-item.middle[text="${value}"]`);
          const topAll = targetElement.getBoundingClientRect().top;
          const scrollTop = topAll - topCurrent;
          parent.scrollTop = scrollTop;
        }
      });
    });
    const hourStartElement = this.el.shadowRoot.querySelector('sas-list-item.top[text="01"]');
    observer.observe(hourStartElement);
    const hourEndElement = this.el.shadowRoot.querySelector('sas-list-item.bottom[text="12"]');
    observer.observe(hourEndElement);
    const minuteStartElement = this.el.shadowRoot.querySelector('sas-list-item.top[text="00"]');
    observer.observe(minuteStartElement);
    const minuteEndElement = this.el.shadowRoot.querySelector('sas-list-item.bottom[text="59"]');
    observer.observe(minuteEndElement);
  }
  updateTimeFieldList() {
    let time;
    if (Boolean(this.value)) {
      time = this.value;
    }
    else {
      const now = new Date();
      const hour = `${now.getHours()}`;
      const minute = `${now.getMinutes()}`;
      time = `${hour}:${minute}`;
    }
    const formatTimeObj = this.convertTo12Hour(time);
    const hourList = this.el.shadowRoot.querySelector('#hour');
    hourList.value = formatTimeObj.hour;
    const selectedHour = hourList.querySelector(`sas-list-item.middle[text="${formatTimeObj.hour}"]`);
    selectedHour.parentElement.scrollTop = selectedHour.offsetTop;
    const minuteList = this.el.shadowRoot.querySelector('#minute');
    minuteList.value = formatTimeObj.minute;
    const selectedMinute = minuteList.querySelector(`sas-list-item.middle[text="${formatTimeObj.minute}"]`);
    selectedMinute.parentElement.scrollTop = selectedMinute.offsetTop;
    const dayPartList = this.el.shadowRoot.querySelector('#day-part');
    dayPartList.value = formatTimeObj.dayPart;
  }
  convertTo12Hour(time) {
    const timeArray = time.split(':');
    const hour = this.getFormatHour(Number(timeArray[0]));
    const minute = this.getFormatMinute(Number(timeArray[1]));
    const dayPart = this.getDayPart(Number(timeArray[0]));
    return {
      hour: hour,
      minute: minute,
      dayPart: dayPart,
    };
  }
  convertTo24Hour(hour, minute, dayPart) {
    let finalHour;
    if (dayPart === 'AM') {
      if (hour === '12') {
        finalHour = '00';
      }
      else {
        finalHour = hour;
      }
    }
    else {
      if (hour !== '12') {
        let hourNumber = Number(hour) + 12;
        finalHour = `${hourNumber}`;
      }
      else {
        finalHour = hour;
      }
    }
    return `${finalHour}:${minute}`;
  }
  getFormatHour(hour) {
    let result = hour;
    if (hour === 0) {
      result = 12;
    }
    if (hour > 12) {
      result = hour - 12;
    }
    if (result < 10) {
      return `0${result}`;
    }
    return `${result}`;
  }
  getFormatMinute(minute) {
    if (minute < 10) {
      return `0${minute}`;
    }
    return `${minute}`;
  }
  getDayPart(hour) {
    if (hour < 12) {
      return 'AM';
    }
    return 'PM';
  }
  formatValue(value) {
    if (value.length > 2) {
      return value.slice(0, 2);
    }
    return value;
  }
  getElement(start, end) {
    let elements = [];
    for (let i = start; i <= end; i += 1) {
      let text = `${i}`;
      if (i < 10) {
        text = `0${i}`;
      }
      const element = index.h("sas-list-item", { class: "top", value: text, text: text });
      elements.push(element);
    }
    for (let i = start; i <= end; i += 1) {
      let text = `${i}`;
      if (i < 10) {
        text = `0${i}`;
      }
      const element = index.h("sas-list-item", { class: "middle", value: text, text: text });
      elements.push(element);
    }
    for (let i = start; i <= end; i += 1) {
      let text = `${i}`;
      if (i < 10) {
        text = `0${i}`;
      }
      const element = index.h("sas-list-item", { class: "bottom", value: text, text: text });
      elements.push(element);
    }
    return elements;
  }
  render() {
    return (index.h(index.Host, null, index.h("sas-form-input-template", { label: this.label, outside: this.outside, required: this.required, value: this.value, iconRight: "far fa-clock", type: "time", disabled: this.disabled, clear: this.clear, "data-message": this.message, position: this.position, minimumPixelsAboveComponentRequired: 280 }, index.h("div", { slot: "container", class: "time-field", tabindex: -1 }, index.h("sas-list", { class: "part", id: "hour" }, this.getElement(1, 12)), index.h("sas-list", { class: "part", id: "minute" }, this.getElement(0, 59)), index.h("sas-list", { class: "part", id: "day-part" }, index.h("sas-list-item", { value: "AM", text: "AM" }), index.h("sas-list-item", { value: "PM", text: "PM" }))))));
  }
  get el() { return index.getElement(this); }
  static get watchers() { return {
    "value": ["watchHandler"]
  }; }
};
SasTimeField.style = sasTimeFieldCss;

exports.sas_time_field = SasTimeField;
