import { Event, Component, Host, h, Prop, Element, } from '@stencil/core';
import { keydownEvent } from '../../global/global';
/**
* @slot title - Used for complex title, eg: words in title are different font weight
* @slot content - Used for complex content, eg: words in content are different font weight
*/
export class SasAlert {
  constructor() {
    /**
     * Type
     */
    this.type = 'success';
    /**
     * Show
     */
    this.show = true;
    /**
     * Adds a close icon that can hide the alert
     */
    this.close = false;
    /**
     * Alert title, will be ignored if 'title' slot has slotted content
     */
    this.alertTitle = 'Title';
    /**
     * Content,  will be ignored if 'content' slot has slotted content
     */
    this.content = '';
    this.handleCloseIconClick = () => {
      this.show = false;
      this.closed.emit();
    };
  }
  getTypeIcon() {
    if (this.type === 'success') {
      return 'far fa-check-circle';
    }
    if (this.type === 'warning') {
      return 'far fa-exclamation-circle';
    }
    if (this.type === 'error') {
      return 'far fa-exclamation-triangle';
    }
    return 'far fa-info-circle';
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        h("sas-icon", { class: 'type-icon', icon: this.getTypeIcon() }),
        h("div", { class: "title" },
          h("slot", { name: "title" },
            h("span", null, this.alertTitle))),
        this.close
          ? h("sas-icon", { class: "close-icon", tabindex: 0, icon: "far fa-times", onClick: this.handleCloseIconClick, onKeyDown: keydownEvent })
          : null,
        h("div", { class: "content-container" },
          h("slot", { name: "content" }, this.content !== ''
            ? h("div", { class: "content" }, this.content)
            : null)))));
  }
  static get is() { return "sas-alert"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-alert.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-alert.css"]
  }; }
  static get properties() { return {
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'success' | 'info' | 'warning' | 'error'",
        "resolved": "\"error\" | \"info\" | \"success\" | \"warning\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Type"
      },
      "attribute": "type",
      "reflect": true,
      "defaultValue": "'success'"
    },
    "show": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show"
      },
      "attribute": "show",
      "reflect": true,
      "defaultValue": "true"
    },
    "close": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Adds a close icon that can hide the alert"
      },
      "attribute": "close",
      "reflect": true,
      "defaultValue": "false"
    },
    "alertTitle": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Alert title, will be ignored if 'title' slot has slotted content"
      },
      "attribute": "alert-title",
      "reflect": false,
      "defaultValue": "'Title'"
    },
    "content": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Content,  will be ignored if 'content' slot has slotted content"
      },
      "attribute": "content",
      "reflect": false,
      "defaultValue": "''"
    }
  }; }
  static get events() { return [{
      "method": "closed",
      "name": "closed",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the component is clicked"
      },
      "complexType": {
        "original": "null",
        "resolved": "null",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
}
