import { Component, Host, h, Prop, Event, } from '@stencil/core';
export class SasAvatar {
  constructor() {
    /**
     * Type
     */
    this.type = 'small';
    /**
     * Name of avatar, display initials if no image src
     */
    this.name = 'Unknown';
    /**
     * Image source
     */
    this.src = '';
    /**
     * Disabled
     */
    this.disabled = false;
    /**
     * Group
     */
    this.group = false;
    this.handleClick = (event) => {
      this.avatarClicked.emit(event.target.name);
    };
  }
  getContent() {
    if (Boolean(this.src)) {
      return h("img", { src: this.src });
    }
    if (this.type !== 'small') {
      return h("sas-icon", { icon: "fas fa-user" });
    }
    return h("span", { class: "text-container" }, this.getInitial());
  }
  getInitial() {
    if (Boolean(this.name)) {
      const nameArray = this.name.split(' ');
      let initial = this.getFirstValidLetter(nameArray[0]);
      if (nameArray.length > 1) {
        initial += this.getFirstValidLetter(nameArray[1]);
      }
      return initial;
    }
    return '';
  }
  getFirstValidLetter(name) {
    for (let i = 0; i < name.length; i++) {
      if (/^[a-z]$/i.test(name[i])) {
        return name[i].toUpperCase();
      }
    }
    return '';
  }
  render() {
    return (h(Host, { onClick: this.handleClick },
      h("div", { class: "container" }, this.getContent())));
  }
  static get is() { return "sas-avatar"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-avatar.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-avatar.css"]
  }; }
  static get properties() { return {
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'small' | 'medium' | 'large'",
        "resolved": "\"large\" | \"medium\" | \"small\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Type"
      },
      "attribute": "type",
      "reflect": true,
      "defaultValue": "'small'"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Name of avatar, display initials if no image src"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "'Unknown'"
    },
    "src": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Image source"
      },
      "attribute": "src",
      "reflect": true,
      "defaultValue": "''"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "group": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Group"
      },
      "attribute": "group",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "avatarClicked",
      "name": "avatarClicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the component is clicked"
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }]; }
}
