import { 
// eslint-disable-next-line no-unused-vars
Component, Host, h, Prop, State, } from '@stencil/core';
import { keydownEvent } from '../../global/global';
export class SasBreadcrumb {
  constructor() {
    /**
     * Breadcrumb item array
     */
    this.items = [];
    this.showAll = false;
    this.showAllBreadcrumb = () => {
      this.showAll = true;
    };
  }
  getBreadcrumbs() {
    const breadcrumbs = [];
    let breadcrumb;
    let divider;
    if (this.items.length > 5 && !this.showAll) {
      breadcrumb = h("li", null,
        h("a", { class: this.getDisabled(this.items[0].disabled), tabindex: this.getTabindex(this.items[0].disabled), href: this.items[0].href }, this.items[0].text));
      breadcrumbs.push(breadcrumb);
      divider = h("li", { class: "divider" }, "/");
      breadcrumbs.push(divider);
      breadcrumb = h("li", null,
        h("a", { tabindex: 0, onClick: this.showAllBreadcrumb, onKeyDown: keydownEvent }, "..."));
      breadcrumbs.push(breadcrumb);
      divider = h("li", { class: "divider" }, "/");
      breadcrumbs.push(divider);
      breadcrumb = h("li", null,
        h("a", { class: this.getDisabled(this.items[this.items.length - 1].disabled), tabindex: this.getTabindex(this.items[this.items.length - 1].disabled), href: this.items[this.items.length - 1].href }, this.items[this.items.length - 1].text));
      breadcrumbs.push(breadcrumb);
    }
    else {
      this.items.forEach((item, index) => {
        breadcrumb = h("li", null,
          h("a", { class: this.getDisabled(item.disabled), tabindex: this.getTabindex(item.disabled), href: item.href }, item.text));
        breadcrumbs.push(breadcrumb);
        if (index !== this.items.length - 1) {
          divider = h("li", { class: "divider" }, "/");
          breadcrumbs.push(divider);
        }
      });
    }
    return breadcrumbs;
  }
  getDisabled(disabled) {
    if (disabled) {
      return 'disabled';
    }
    return '';
  }
  getTabindex(disabled) {
    if (disabled) {
      return -1;
    }
    return 0;
  }
  render() {
    return (h(Host, null,
      h("ul", null, this.getBreadcrumbs())));
  }
  static get is() { return "sas-breadcrumb"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-breadcrumb.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-breadcrumb.css"]
  }; }
  static get properties() { return {
    "items": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "BreadCrumbItem[]",
        "resolved": "BreadCrumbItem[]",
        "references": {
          "BreadCrumbItem": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Breadcrumb item array"
      },
      "defaultValue": "[]"
    }
  }; }
  static get states() { return {
    "showAll": {}
  }; }
}
