import { Component, Element, Prop, Host, h, Event, Watch, Listen, } from '@stencil/core';
/**
 * @slot default - used for buttons
 */
export class SasButtonGroup {
  constructor() {
    /**
     * Active Button Index
     */
    this.values = [];
    /**
     * Enable select multiple buttons
     */
    this.multiple = false;
  }
  componentDidLoad() {
    this.addActiveClass(this.values);
  }
  watchHandler(newValue) {
    this.addActiveClass(newValue);
    this.buttonGroupUpdate.emit(newValue);
  }
  buttonclickedHandler(event) {
    const index = this.getButtonIndex(event.target);
    if (this.multiple) {
      let set = new Set(this.values);
      if (event.target.classList.contains('button-group-active')) {
        set.delete(index);
      }
      else {
        set.add(index);
      }
      this.values = Array.from(set);
    }
    else {
      this.values = [index];
    }
  }
  getButtonIndex(element) {
    let index = 0;
    let prevNode = element.previousElementSibling;
    while (prevNode) {
      index++;
      prevNode = prevNode.previousElementSibling;
    }
    return index;
  }
  addActiveClass(activeButtonIndex) {
    const buttons = this.el.querySelectorAll('sas-button');
    const set = new Set(activeButtonIndex);
    buttons.forEach((button, index) => {
      if (set.has(index)) {
        button.classList.add('button-group-active');
      }
      else {
        button.classList.remove('button-group-active');
      }
    });
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        h("slot", null))));
  }
  static get is() { return "sas-button-group"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-button-group.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-button-group.css"]
  }; }
  static get properties() { return {
    "values": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "number[]",
        "resolved": "number[]",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Active Button Index"
      },
      "defaultValue": "[]"
    },
    "multiple": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enable select multiple buttons"
      },
      "attribute": "multiple",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "buttonGroupUpdate",
      "name": "buttonGroupUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the values are changed by user interaction"
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "values",
      "methodName": "watchHandler"
    }]; }
  static get listeners() { return [{
      "name": "buttonclicked",
      "method": "buttonclickedHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
