import { Event, Component, Prop, h, Host, State, Element, } from '@stencil/core';
import { keydownEvent } from '../../global/global';
/**
 * @slot default - default slot
 */
export class SasButton {
  constructor() {
    /**
     * Type
     */
    this.type = 'primary';
    /**
     * Size
     */
    this.size = 'default';
    /**
     * Text
     */
    this.text = 'button';
    /**
     * Icon left
     */
    this.iconLeft = null;
    /**
     * Icon
     */
    this.icon = null;
    /**
     * Icon position
     */
    this.iconPosition = 'left';
    /**
     * Expands the button to 100% of available space
     */
    this.block = false;
    /**
     * Adds a loading icon animation
     */
    this.loading = false;
    /**
     * Disabled
     */
    this.disabled = false;
    /**
     * Designates the component as anchor and applies the href attribute.
     */
    this.href = null;
    /**
     * Designates the target attribute. This should only be applied when using the href prop.
     */
    this.target = '';
    this.tabindexNumber = 0;
    this.clickEvent = (event) => {
      const index = this.getClickedElementIndex(event.target);
      this.buttonClicked.emit(index);
      this.buttonclicked.emit(index);
    };
  }
  componentWillRender() {
    if (this.disabled) {
      this.tabindexNumber = -1;
    }
    else {
      this.tabindexNumber = 0;
    }
    const slot = this.el.querySelector('[slot="buttonText"]');
    if (slot && this.text === 'button') {
      this.text = slot.innerHTML;
    }
  }
  getClickedElementIndex(clickedElement) {
    let index = 0;
    let prevSibling = clickedElement.previousElementSibling;
    while (prevSibling) {
      if (clickedElement.tagName === prevSibling.tagName) {
        index += 1;
      }
      prevSibling = prevSibling.previousElementSibling;
    }
    return index;
  }
  render() {
    return (h(Host, { tabindex: this.tabindexNumber, onClick: this.clickEvent, onKeyDown: keydownEvent },
      h("div", { class: "container" },
        Boolean(this.href)
          ? h("a", { href: this.href, target: this.target, class: "anchor" })
          : null,
        this.loading
          ? h("sas-icon", { class: "loading-icon", icon: "fas fa-spinner fa-spin" })
          : null,
        Boolean(this.iconLeft)
          ? h("sas-icon", { class: "button-icon", icon: this.iconLeft })
          : null,
        h("div", { class: "button-main-content" },
          this.icon && this.iconPosition === 'left'
            ? h("sas-icon", { class: "button-icon", icon: this.icon })
            : null,
          Boolean(this.text)
            ? h("div", { class: "button-text" }, this.text)
            : null,
          this.icon && this.iconPosition === 'right'
            ? h("sas-icon", { class: "button-icon", icon: this.icon })
            : null),
        Boolean(this.iconRight)
          ? h("sas-icon", { class: "button-icon", icon: this.iconRight })
          : null,
        h("slot", null))));
  }
  static get is() { return "sas-button"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-button.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-button.css"]
  }; }
  static get properties() { return {
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'primary' | 'secondary' | 'text'",
        "resolved": "\"primary\" | \"secondary\" | \"text\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Type"
      },
      "attribute": "type",
      "reflect": true,
      "defaultValue": "'primary'"
    },
    "size": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'exlarge' | 'large' | 'default' | 'small'",
        "resolved": "\"default\" | \"exlarge\" | \"large\" | \"small\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Size"
      },
      "attribute": "size",
      "reflect": true,
      "defaultValue": "'default'"
    },
    "text": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Text"
      },
      "attribute": "text",
      "reflect": true,
      "defaultValue": "'button'"
    },
    "iconLeft": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon left"
      },
      "attribute": "icon-left",
      "reflect": true,
      "defaultValue": "null"
    },
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon"
      },
      "attribute": "icon",
      "reflect": true,
      "defaultValue": "null"
    },
    "iconPosition": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'left' | 'right'",
        "resolved": "\"left\" | \"right\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon position"
      },
      "attribute": "icon-position",
      "reflect": true,
      "defaultValue": "'left'"
    },
    "iconRight": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon right"
      },
      "attribute": "icon-right",
      "reflect": true
    },
    "block": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Expands the button to 100% of available space"
      },
      "attribute": "block",
      "reflect": true,
      "defaultValue": "false"
    },
    "loading": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Adds a loading icon animation"
      },
      "attribute": "loading",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "href": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Designates the component as anchor and applies the href attribute."
      },
      "attribute": "href",
      "reflect": true,
      "defaultValue": "null"
    },
    "target": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Designates the target attribute. This should only be applied when using the href prop."
      },
      "attribute": "target",
      "reflect": true,
      "defaultValue": "''"
    }
  }; }
  static get states() { return {
    "tabindexNumber": {}
  }; }
  static get events() { return [{
      "method": "buttonClicked",
      "name": "buttonClicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the component is clicked (will be deprecated)"
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "buttonclicked",
      "name": "buttonclicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the component is clicked"
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
}
