// sas-button.stories.js
import readme from './readme.md';
// import './sas-button.tsx';

export default {
  title: 'Basic/Button',
  parameters: {
    notes: {
      markdown: readme,
    },
    actions: {
      handles: ['buttonClicked', 'sas-button'],
    },
  },
};

const typeOptions = ['primary', 'secondary', 'text'];
const sizeOptions = ['small', 'default', 'large', 'exlarge'];
const iconPositionOptions = ['left', 'center', 'right'];

export const Basic = (args) => `
  <sas-button
    type="${args.type}"
    size="${args.size}"
    text="${args.text}"
    icon-left="${args.iconLeft}"
    icon="${args.icon}"
    icon-position="${args.iconPosition}"
    icon-right="${args.iconRight}"
    block="${args.block}"
    loading="${args.loading}"
    disabled="${args.disabled}"
    href="${args.href}"
    target="${args.target}"
  ></sas-button>
`;

Basic.args = {
  type: 'primary',
  size: 'default',
  text: 'button',
  iconLeft: '',
  icon: '',
  iconPosition: 'center',
  iconRight: '',
  block: false,
  loading: false,
  disabled: false,
  href: 'https://www.google.com/',
  target: '_blank',
};

Basic.argTypes = {
  type: {
    control: {
      type: 'select',
      options: typeOptions,
    },
  },
  size: {
    control: {
      type: 'select',
      options: sizeOptions,
    },
  },
  iconPosition: {
    control: {
      type: 'select',
      options: iconPositionOptions,
    },
  },
};

export const Type = () => `
  <h3>Type primary</h3>
  <sas-button type="primary" text="primary"></sas-button>
  <h3>Type secondary</h3>
  <sas-button type="secondary" text="secondary"></sas-button>
  <h3>Type text</h3>
  <sas-button type="text" text="text"></sas-button>
`;

Type.parameters = {
  controls: { hideNoControlsWarning: true },
};

export const Size = () => `
  <h3>Size exlarge</h3>
  <sas-button size="exlarge" text="exlarge"></sas-button>
  <h3>Size large</h3>
  <sas-button size="large" text="large"></sas-button>
  <h3>Size default</h3>
  <sas-button size="default" text="default"></sas-button>
  <h3>Size small</h3>
  <sas-button size="small" text="small"></sas-button>
`;

Size.parameters = {
  controls: { hideNoControlsWarning: true },
};

export const Icon = () => `
  <h3>Left icon</h3>
  <sas-button icon-left="fas fa-cog" text="icon left" block></sas-button>
  <h3>Right icon</h3>
  <sas-button text="icon right" icon-right="fas fa-cog" block></sas-button>
  <h3>Content left icon</h3>
  <sas-button icon="fas fa-cog" icon-position="left" text="icon left" block></sas-button>
  <h3>Content right icon</h3>
  <sas-button icon="fas fa-cog" icon-position="right" text="icon right" block></sas-button>
  <h3>All icons</h3>
  <sas-button icon-left="fas fa-cog" icon="fas fa-cog" icon-position="right" text="icon right" icon-right="fas fa-cog" block></sas-button>
`;

Icon.parameters = {
  controls: { hideNoControlsWarning: true },
};

export const Status = () => `
  <h3>Block</h3>
  <sas-button block text="block"></sas-button>
  <h3>Loading</h3>
  <sas-button loading text="loading"></sas-button>
  <h3>Disabled</h3>
  <sas-button disabled text="disabled"></sas-button>
  <h3>Link</h3>
  <sas-button href="https://www.google.com/" target="_blank" text="link"></sas-button>
`;

Status.parameters = {
  controls: { hideNoControlsWarning: true },
};
