import { Component, Host, h, Prop, Event, } from '@stencil/core';
/**
 * @slot icon - used for custom icons
 * @slot title - used for title
 * @slot content - used for content
 * @slot list - used for dropdown menu
 */
export class SasCard {
  constructor() {
    /**
     * Display as block
     */
    this.block = false;
    /**
     * Src
     */
    this.src = '';
    /**
     * lazy load image
     */
    this.lazyImage = false;
    /**
     * Scale when hover
     */
    this.scale = '1';
    /**
     * Needs repair
     */
    this.repair = false;
    /**
     * Has attachment
     */
    this.attachment = false;
    /**
     * Disabled
     */
    this.disabled = false;
    /**
     * Card title, will be ignored if 'card-title' slot has slotted content
     */
    this.cardTitle = '';
    this.clickHandler = () => {
      this.cardimageclicked.emit();
    };
    this.pointerOverHandler = (event) => {
      event.target.style.transform = `scale(${this.scale})`;
    };
    this.pointerOutHandler = (event) => {
      event.target.style.transform = `scale(1)`;
    };
  }
  getIcons() {
    const icons = [];
    let icon;
    if (this.repair) {
      icon = h("sas-icon", { type: "round", icon: "fas fa-wrench" });
      icons.push(icon);
    }
    if (this.attachment) {
      icon = h("sas-icon", { type: "round", icon: "fas fa-clipboard-list" });
      icons.push(icon);
    }
    if (this.disabled) {
      icon = h("sas-icon", { type: "round", icon: "fas fa-ban" });
      icons.push(icon);
    }
    return icons;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        h("div", { class: "image-container" }, Boolean(this.src)
          ? h("sas-image", { class: "image", src: this.src, onClick: this.clickHandler, onPointerOver: this.pointerOverHandler, onPointerOut: this.pointerOutHandler, lazy: this.lazyImage })
          : h("sas-icon", { class: "noimage", icon: "fas fa-image" })),
        h("div", { class: "icon-container" },
          this.getIcons(),
          h("slot", { name: "icon" })),
        h("div", { class: "text-container" },
          h("div", { class: "title" },
            h("span", { class: "title-content" },
              h("slot", { name: "title" },
                h("span", null, this.cardTitle))),
            h("sas-dropdown", { "position-h": "left" },
              h("sas-icon", { slot: "control", tabindex: "0", class: "ellipsis-icon", icon: "fas fa-ellipsis-h" }),
              h("slot", { name: "list", slot: "menu" }))),
          h("div", { class: "content" },
            h("slot", { name: "content" }))))));
  }
  static get is() { return "sas-card"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-card.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-card.css"]
  }; }
  static get properties() { return {
    "block": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Display as block"
      },
      "attribute": "block",
      "reflect": true,
      "defaultValue": "false"
    },
    "src": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Src"
      },
      "attribute": "src",
      "reflect": true,
      "defaultValue": "''"
    },
    "lazyImage": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "lazy load image"
      },
      "attribute": "lazy-image",
      "reflect": true,
      "defaultValue": "false"
    },
    "scale": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Scale when hover"
      },
      "attribute": "scale",
      "reflect": true,
      "defaultValue": "'1'"
    },
    "repair": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Needs repair"
      },
      "attribute": "repair",
      "reflect": true,
      "defaultValue": "false"
    },
    "attachment": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Has attachment"
      },
      "attribute": "attachment",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "cardTitle": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Card title, will be ignored if 'card-title' slot has slotted content"
      },
      "attribute": "card-title",
      "reflect": true,
      "defaultValue": "''"
    }
  }; }
  static get events() { return [{
      "method": "cardimageclicked",
      "name": "cardimageclicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the image is clicked"
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }]; }
}
