import { Component, Host, h, Prop, Watch, Element } from '@stencil/core';
import { animate } from '../../global/global';
export class SasCarousel {
  constructor() {
    this.originalScrollLeft = 0;
    this.startClientX = 0;
    /**
     * Slide index
     */
    this.index = 0;
    /**
     * Slides array
     */
    this.slides = [];
    this.goToLeftSlide = (event) => {
      event.stopPropagation();
      this.index = this.getValidIndex(this.index - 1);
    };
    this.goToRightSlide = (event) => {
      event.stopPropagation();
      this.index = this.getValidIndex(this.index + 1);
    };
    this.touchStartHandler = (event) => {
      event.preventDefault();
      const container = this.el.shadowRoot.querySelector('.slide-container');
      this.originalScrollLeft = container.scrollLeft;
      this.startClientX = event.touches[0].clientX;
    };
    this.touchMoveHandler = (event) => {
      event.preventDefault();
      const currentClientX = event.touches[0].clientX;
      const offset = this.startClientX - currentClientX;
      this.startClientX = currentClientX;
      const container = this.el.shadowRoot.querySelector('.slide-container');
      container.scrollLeft += offset;
    };
    this.touchEndHandler = (event) => {
      event.preventDefault();
      const container = this.el.shadowRoot.querySelector('.slide-container');
      const currentScrollLeft = container.scrollLeft;
      const slideWidth = container.getBoundingClientRect().width;
      if (this.originalScrollLeft < currentScrollLeft) {
        this.index = Math.ceil(currentScrollLeft / slideWidth);
      }
      else {
        this.index = Math.floor(currentScrollLeft / slideWidth);
      }
    };
  }
  componentDidLoad() {
    this.scroll(false);
  }
  watchValueHandler() {
    this.scroll();
  }
  getValidIndex(index) {
    const maxIndex = this.el.shadowRoot.querySelector('.slide-container').children.length;
    if (index < 0) {
      return maxIndex - 1;
    }
    if (index > maxIndex - 1) {
      return 0;
    }
    return index;
  }
  async scroll(smooth = true) {
    const container = this.el.shadowRoot.querySelector('.slide-container');
    const currentScrollLeft = container.scrollLeft;
    const slideWidth = container.getBoundingClientRect().width;
    const targetScrollLeft = slideWidth * this.index;
    const offset = targetScrollLeft - currentScrollLeft;
    if (smooth) {
      const drawFunction = function (progress) {
        container.scrollLeft = currentScrollLeft + progress * offset;
      };
      await animate({
        duration: 250,
        timing: function (timeFraction) {
          return timeFraction;
        },
        draw: drawFunction,
      });
    }
    else {
      container.scrollLeft = targetScrollLeft;
    }
  }
  render() {
    return (h(Host, { onTouchStart: this.touchStartHandler, onTouchMove: this.touchMoveHandler, onTouchEnd: this.touchEndHandler },
      h("div", { class: "container" },
        h("sas-icon", { class: "icon left-icon", icon: "fas fa-chevron-left", onClick: this.goToLeftSlide, onTouchEnd: this.goToLeftSlide }),
        h("div", { class: "slide-container" }, this.slides.map(slide => h("img", { class: "slide", src: slide.src }))),
        h("div", { class: "slide-info" },
          h("div", null, this.slides[this.index].name),
          h("div", null,
            this.index + 1,
            " / ",
            this.slides.length)),
        h("sas-icon", { class: "icon right-icon", icon: "fas fa-chevron-right", onClick: this.goToRightSlide, onTouchEnd: this.goToRightSlide }))));
  }
  static get is() { return "sas-carousel"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-carousel.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-carousel.css"]
  }; }
  static get properties() { return {
    "index": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Slide index"
      },
      "attribute": "index",
      "reflect": true,
      "defaultValue": "0"
    },
    "slides": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Slide[]",
        "resolved": "Slide[]",
        "references": {
          "Slide": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Slides array"
      },
      "defaultValue": "[]"
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "index",
      "methodName": "watchValueHandler"
    }]; }
}
