import { Component, Host, h, Prop, Listen, Element, } from '@stencil/core';
export class SasCheckboxSubgroup {
  constructor() {
    /**
     *  Value
     */
    this.value = null;
    /**
     *  Text
     */
    this.text = '';
    /**
     *  Checked
     */
    this.checked = false;
    /**
     *  Disabled
     */
    this.disabled = false;
    /**
     *  Checkbox icon
     */
    this.icon = '';
    /**
     *  Status of children checkboxes
     */
    this.status = 'none';
    this.clickHandler = () => {
      let checkboxElements = this.el.querySelectorAll('sas-checkbox');
      if (this.icon === 'fas fa-check') {
        checkboxElements.forEach((checkboxElement) => {
          if (checkboxElement.checked) {
            checkboxElement.click();
          }
        });
      }
      else {
        checkboxElements.forEach((checkboxElement) => {
          if (!checkboxElement.checked) {
            checkboxElement.click();
          }
        });
      }
    };
  }
  checkboxUpdateHandler() {
    let checkboxElements = this.el.querySelectorAll('sas-checkbox');
    let checkedNumber = 0;
    checkboxElements.forEach((checkboxElement) => {
      if (checkboxElement.checked) {
        checkedNumber += 1;
      }
    });
    if (checkedNumber === 0) {
      this.status = 'none';
      this.checked = false;
      this.icon = '';
    }
    else if (checkedNumber > 0 && checkedNumber < checkboxElements.length) {
      this.status = 'part';
      this.checked = true;
      this.icon = 'far fa-horizontal-rule';
    }
    else {
      this.status = 'all';
      this.checked = true;
      this.icon = 'far fa-check';
    }
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container", tabindex: -1, onClick: this.clickHandler },
        h("div", { class: "checkbox" },
          h("sas-icon", { icon: this.icon })),
        h("div", { class: "text" }, this.text)),
      h("div", { class: 'sub-group' },
        h("slot", null))));
  }
  static get is() { return "sas-checkbox-subgroup"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-checkbox-subgroup.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-checkbox-subgroup.css"]
  }; }
  static get properties() { return {
    "value": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "CheckboxValue",
        "resolved": "boolean | number | string",
        "references": {
          "CheckboxValue": {
            "location": "import",
            "path": "../sas-checkbox/type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Value"
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "null"
    },
    "text": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Text"
      },
      "attribute": "text",
      "reflect": true,
      "defaultValue": "''"
    },
    "checked": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Checked"
      },
      "attribute": "checked",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "icon": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Checkbox icon"
      },
      "attribute": "icon",
      "reflect": false,
      "defaultValue": "''"
    },
    "status": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Status of children checkboxes"
      },
      "attribute": "status",
      "reflect": false,
      "defaultValue": "'none'"
    }
  }; }
  static get elementRef() { return "el"; }
  static get listeners() { return [{
      "name": "checkboxUpdate",
      "method": "checkboxUpdateHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
