import { Component, Host, h, Prop, Event, Element, Watch, Method, } from '@stencil/core';
import { keydownEvent } from '../../global/global';
export class SasCheckbox {
  constructor() {
    this.tabindexNumber = 0;
    /**
     * name
     */
    this.name = null;
    /**
     *  Value
     */
    this.value = null;
    /**
     *  Text
     */
    this.text = '';
    /**
     *  Index
     */
    this.index = -1;
    /**
     *  Text Position
     */
    this.textPosition = 'right';
    /**
     *  Align
     */
    this.align = 'left';
    /**
     *  Check Status
     */
    this.checked = false;
    /**
     *  Disabled
     */
    this.disabled = false;
    /**
     *  Icon
     */
    this.icon = 'far fa-check';
    /**
     *  Error
     */
    this.error = false;
    /**
     *  Required
     */
    this.required = false;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
    };
    /**
     * Only shows when prop error is true, auto generated
     */
    this.message = '';
    /**
     * Only used for multiselect
     */
    this.exclusive = false;
    this.clickEvent = (event) => {
      event.stopPropagation();
      if (!this.disabled) {
        this.checked = !this.checked;
        const checkboxInfo = {
          value: this.value,
          text: this.text,
          isChecked: this.checked,
          exclusive: this.exclusive,
        };
        this.checkboxUpdate.emit(checkboxInfo);
        this.checkboxupdate.emit(checkboxInfo);
      }
    };
  }
  watchHandler() {
    this.resetErrorStatus();
  }
  componentWillRender() {
    if (this.disabled) {
      this.tabindexNumber = -1;
    }
    else {
      this.tabindexNumber = 0;
    }
  }
  /**
   * Validate
   */
  async validate() {
    let valueMissing = false;
    if (this.required && !this.checked) {
      valueMissing = true;
    }
    this.validity = {
      valid: !valueMissing,
      valueMissing: valueMissing,
    };
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    await this.validate();
    this.error = !this.validity.valid;
    if (!this.validity.valid) {
      this.message = this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    this.error = false;
    this.message = '';
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      return `Please check this field`;
    }
  }
  render() {
    return (h(Host, { onClick: this.clickEvent, onKeyDown: keydownEvent },
      h("div", { class: "container", "data-message": this.message },
        this.textPosition === 'left' && Boolean(this.text)
          ? h("div", { class: "text", tabindex: -1 }, this.text)
          : null,
        h("div", { class: "checkbox", tabindex: this.tabindexNumber }, this.checked
          ? h("sas-icon", { icon: this.icon })
          : null),
        this.textPosition === 'right' && Boolean(this.text)
          ? h("div", { class: "text" }, this.text)
          : null)));
  }
  static get is() { return "sas-checkbox"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-checkbox.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-checkbox.css"]
  }; }
  static get properties() { return {
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "null"
    },
    "value": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "CheckboxValue",
        "resolved": "boolean | number | string",
        "references": {
          "CheckboxValue": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Value"
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "null"
    },
    "text": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Text"
      },
      "attribute": "text",
      "reflect": true,
      "defaultValue": "''"
    },
    "index": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Index"
      },
      "attribute": "index",
      "reflect": true,
      "defaultValue": "-1"
    },
    "textPosition": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'left' | 'right'",
        "resolved": "\"left\" | \"right\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Text Position"
      },
      "attribute": "text-position",
      "reflect": true,
      "defaultValue": "'right'"
    },
    "align": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'left' | 'right' | 'space-between'",
        "resolved": "\"left\" | \"right\" | \"space-between\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Align"
      },
      "attribute": "align",
      "reflect": true,
      "defaultValue": "'left'"
    },
    "checked": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Check Status"
      },
      "attribute": "checked",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon"
      },
      "attribute": "icon",
      "reflect": true,
      "defaultValue": "'far fa-check'"
    },
    "error": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Error"
      },
      "attribute": "error",
      "reflect": true,
      "defaultValue": "false"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Required"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "validity": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "{ valid: boolean; valueMissing: boolean; }",
        "resolved": "{ valid: boolean; valueMissing: boolean; }",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Validity"
      },
      "defaultValue": "{\n    valid: true,\n    valueMissing: false,\n  }"
    },
    "message": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Only shows when prop error is true, auto generated"
      },
      "attribute": "message",
      "reflect": true,
      "defaultValue": "''"
    },
    "exclusive": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Only used for multiselect"
      },
      "attribute": "exclusive",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "checkboxUpdate",
      "name": "checkboxUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction (will be deprecated)"
      },
      "complexType": {
        "original": "CheckboxUpdateEventDetail",
        "resolved": "CheckboxUpdateEventDetail",
        "references": {
          "CheckboxUpdateEventDetail": {
            "location": "import",
            "path": "./type"
          }
        }
      }
    }, {
      "method": "checkboxupdate",
      "name": "checkboxupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction"
      },
      "complexType": {
        "original": "CheckboxUpdateEventDetail",
        "resolved": "CheckboxUpdateEventDetail",
        "references": {
          "CheckboxUpdateEventDetail": {
            "location": "import",
            "path": "./type"
          }
        }
      }
    }]; }
  static get methods() { return {
    "validate": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Validate",
        "tags": []
      }
    },
    "updateErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show error status if value is not valid (only for components internal use)",
        "tags": []
      }
    },
    "resetErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clear error status (only for components internal use)",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "checked",
      "methodName": "watchHandler"
    }]; }
}
