import { Component, Host, h, Prop, Element, Event, Method, Watch } from '@stencil/core';
export class SasDateRange {
  constructor() {
    this.message = '';
    /**
     * Label
     */
    this.label = '';
    /**
      * Label position
      */
    this.outside = false;
    /**
     * name
     */
    this.name = '';
    /**
     * Min date
     */
    this.min = '';
    /**
     * Max date
     */
    this.max = '';
    /**
     * From date
     */
    this.from = '';
    /**
     * To date
     */
    this.to = '';
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * Disabled
     */
    this.disabled = false;
    /**
     * Default container position
     */
    this.position = null;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
    };
    this.fromDateUpdateHander = (event) => {
      this.from = event.detail;
      this.emitEvent();
    };
    this.toDateUpdateHander = (event) => {
      this.to = event.detail;
      this.emitEvent();
    };
  }
  watchFromHandler() {
    this.resetErrorStatus();
  }
  watchToHandler() {
    this.resetErrorStatus();
  }
  /**
   * Validate
   */
  async validate() {
    let valueMissing = true;
    if (this.required && Boolean(this.from) && Boolean(this.to)) {
      valueMissing = false;
    }
    this.validity = {
      valid: !valueMissing,
      valueMissing: valueMissing,
    };
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    await this.validate();
    this.message = this.getMessage();
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    this.validity = {
      valid: true,
      valueMissing: false,
    };
  }
  emitEvent() {
    this.daterangeupdate.emit({
      from: this.from,
      to: this.to,
    });
  }
  getMessage() {
    let message = '';
    if (!this.validity.valid) {
      if (this.validity.valueMissing) {
        let fieldName = Boolean(this.label) ? this.label : 'Field';
        return `${fieldName} is required`;
      }
    }
    return message;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container", "data-message": this.message },
        Boolean(this.label)
          ? h("label", { class: this.outside ? 'outside-label' : 'inside-label' },
            this.label,
            this.required
              ? h("span", { class: "required-symbol" }, "*")
              : null)
          : null,
        h("sas-date-field", { class: "date-field", icon: "", value: this.from, min: this.min, max: Boolean(this.to) ? this.to : this.max, position: this.position, onDatefieldupdate: this.fromDateUpdateHander }),
        h("sas-icon", { class: "icon", icon: "far fa-long-arrow-alt-right" }),
        h("sas-date-field", { class: "date-field", icon: "", value: this.to, min: Boolean(this.from) ? this.from : this.min, max: this.max, position: this.position, "position-h": "right", onDatefieldupdate: this.toDateUpdateHander }))));
  }
  static get is() { return "sas-date-range"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-date-range.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-date-range.css"]
  }; }
  static get properties() { return {
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label"
      },
      "attribute": "label",
      "reflect": true,
      "defaultValue": "''"
    },
    "outside": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label position"
      },
      "attribute": "outside",
      "reflect": true,
      "defaultValue": "false"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "''"
    },
    "min": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Min date"
      },
      "attribute": "min",
      "reflect": true,
      "defaultValue": "''"
    },
    "max": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Max date"
      },
      "attribute": "max",
      "reflect": true,
      "defaultValue": "''"
    },
    "from": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "From date"
      },
      "attribute": "from",
      "reflect": true,
      "defaultValue": "''"
    },
    "to": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "To date"
      },
      "attribute": "to",
      "reflect": true,
      "defaultValue": "''"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Shows a star symbol behind label"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "position": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "FormInputTemplateVerticalPosition",
        "resolved": "\"bottom\" | \"top\"",
        "references": {
          "FormInputTemplateVerticalPosition": {
            "location": "import",
            "path": "../sas-form-input-template/type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Default container position"
      },
      "attribute": "position",
      "reflect": true,
      "defaultValue": "null"
    },
    "validity": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "{ valid: boolean; valueMissing: boolean; }",
        "resolved": "{ valid: boolean; valueMissing: boolean; }",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Validity"
      },
      "defaultValue": "{\n    valid: true,\n    valueMissing: false,\n  }"
    }
  }; }
  static get events() { return [{
      "method": "daterangeupdate",
      "name": "daterangeupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction"
      },
      "complexType": {
        "original": "object",
        "resolved": "object",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "validate": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Validate",
        "tags": []
      }
    },
    "updateErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show error status if value is not valid (only for components internal use)",
        "tags": []
      }
    },
    "resetErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clear error status (only for components internal use)",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "from",
      "methodName": "watchFromHandler"
    }, {
      "propName": "to",
      "methodName": "watchToHandler"
    }]; }
}
