import { Component, Host, h, Prop, Element, Event, Method } from '@stencil/core';
import { animate } from '../../global/global';
/**
 * @slot summary - details summary
 * @slot content - details content
 */
export class SasDetails {
  constructor() {
    /**
     * Open
     */
    this.open = false;
    /**
     * icon
     */
    this.icon = 'fas fa-chevron-down';
    /**
     * add gap between summary and content automatically
     */
    this.autoGap = true;
    this.height = 0;
    this.clickHandler = async () => {
      this.toggleIcon();
      this.toggleContent();
      const isOpen = await this.checkIsOpen();
      this.detailsupdate.emit(isOpen);
    };
  }
  componentDidLoad() {
    let summaryElement = this.el.querySelector('[slot="summary"]');
    summaryElement.tabIndex = 0;
    let contentElement = this.el.querySelector('[slot="content"]');
    this.height = this.getHeight();
    if (!this.open) {
      contentElement.style.height = '0';
    }
    else {
      let container = this.el.shadowRoot.querySelector('.container');
      container.style.overflow = 'visible';
      if (this.hasContent(contentElement)) {
        this.addSummaryMarginBottom();
      }
    }
  }
  /**
   * Check if content is open or not
   */
  async checkIsOpen() {
    const icon = this.el.shadowRoot.querySelector('sas-icon');
    return icon.classList.contains('transform');
  }
  getHeight() {
    let contentElement = this.el.querySelector('[slot="content"]');
    return contentElement.getBoundingClientRect().height;
  }
  hasContent(contentElement) {
    if (contentElement && contentElement.innerHTML) {
      return true;
    }
    else {
      return false;
    }
  }
  toggleIcon() {
    let icon = this.el.shadowRoot.querySelector('sas-icon');
    if (icon !== null) {
      icon.classList.toggle('transform');
    }
  }
  async toggleContent() {
    let container = this.el.shadowRoot.querySelector('.container');
    let contentElement = this.el.querySelector('[slot="content"]');
    if (this.hasContent(contentElement)) {
      const shouldOpen = await this.checkIsOpen();
      const currentHeight = this.getHeight();
      this.height = currentHeight > 0 ? currentHeight : this.height;
      let drawFunction;
      if (shouldOpen) {
        drawFunction = (progress) => {
          contentElement.style.height = `${progress * this.height}px`;
        };
      }
      else {
        drawFunction = (progress) => {
          contentElement.style.height = `${(1 - progress) * this.height}px`;
        };
      }
      container.style.overflow = 'hidden';
      if (shouldOpen) {
        this.addSummaryMarginBottom();
      }
      await animate({
        duration: 250,
        timing: function (timeFraction) {
          return timeFraction;
        },
        draw: drawFunction,
      });
      if (shouldOpen) {
        contentElement.style.height = 'auto';
        container.style.overflow = 'visible';
      }
      else {
        this.removeSummaryMarginBottom();
        container.style.overflow = 'hidden';
      }
    }
  }
  addSummaryMarginBottom() {
    let summaryContainer = this.el.shadowRoot.querySelector('.summary-container');
    summaryContainer.classList.add('margin-bottom');
  }
  removeSummaryMarginBottom() {
    let summaryContainer = this.el.shadowRoot.querySelector('.summary-container');
    summaryContainer.classList.remove('margin-bottom');
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        h("div", { class: "summary-container", onClick: this.clickHandler },
          h("span", { class: "title" },
            h("slot", { name: "summary" })),
          h("sas-icon", { class: `
                ${this.open ? 'transform' : ''}
                ${Boolean(this.icon) ? '' : 'hide-icon'}
              `, icon: this.icon })),
        h("slot", { name: "content" }))));
  }
  static get is() { return "sas-details"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-details.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-details.css"]
  }; }
  static get properties() { return {
    "open": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Open"
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "false"
    },
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "icon"
      },
      "attribute": "icon",
      "reflect": true,
      "defaultValue": "'fas fa-chevron-down'"
    },
    "autoGap": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "add gap between summary and content automatically"
      },
      "attribute": "auto-gap",
      "reflect": true,
      "defaultValue": "true"
    }
  }; }
  static get events() { return [{
      "method": "detailsupdate",
      "name": "detailsupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the component is toggled by user interaction"
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "checkIsOpen": {
      "complexType": {
        "signature": "() => Promise<boolean>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "Check if content is open or not",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
}
