import Swal from 'sweetalert2';

function init(options) {
	let content = getContentHTML(options);
	if (content === 'Dom element, needs append') {
		content = ' ';
	}
	
	return Swal.mixin({
		title: getTitleHTML(options),
		html: content,
		footer: getFooterHTML(options),
		width: options.width ?? '600px',
		showConfirmButton: false,
		padding: "30px",
		customClass: {
			popup: 'sas-dialog-swal-custom-popup',
			header: 'sas-dialog-swal-custom-header',
			title: 'sas-dialog-swal-custom-title',
			content: 'sas-dialog-swal-custom-content',
			footer: 'sas-dialog-swal-custom-footer',
		},
		didOpen: () => {
			addListener(options);
			preventKeyEvent();
		}
	});
}

function getTitleHTML(options) {
	let titleHTML = '';
	if (options.title) {
		if (options.type === 'alert') {
			const iconHTML = `<sas-icon data-status="${options.status}" icon="${getIcon(options.status)}" size="large"></sas-icon>`;
			const textHTML = `<span tabindex="0">${options.title}</span>`;
			titleHTML = iconHTML + textHTML;
		} else {
			titleHTML = `<span tabindex="0">${options.title}</span>`; 
		}
	}67

	if(options.showCloseIcon === true) {
		titleHTML += `<sas-icon id="sas-dialog-swal-cancel-icon" icon="fas fa-times"></sas-icon>`;
	}

	return titleHTML;
}

function getIcon(status) {
	if (status === 'success') {
		return 'far fa-check-circle';
	}
	if (status === 'warning') {
		return 'far fa-exclamation-circle';
	}
	if (status === 'error') {
		return 'far fa-exclamation-triangle';
	}
	return 'far fa-info-circle';
}

function getContentHTML(options) {
	const content = options.content;
	let contentHTML = '';
	if (content) {
		if (Array.isArray(content)) { // array
			content.forEach((content, index) => {
				let marginTop = '0';
				if (index !== 0) {
					marginTop = '10px'
				}
				contentHTML += `<div style="margin-top: ${marginTop}">${content}</div>`;
			});
		} else { // not array
			if (typeof content === 'object') { // dom
				contentHTML = 'Dom element, needs append';
			} else {
				contentHTML = content;
			}
		}
	}
	return contentHTML;
}

function getFooterHTML(options) {
	const button = options.button;
	let footerHTML = '';
	if (button) {
		if (Array.isArray(button)) { // array
			button.forEach((singleButton, index) => {
				footerHTML += getButtonHTML(singleButton, index);
			});
		} else { // not array
			footerHTML = getButtonHTML(button, 0);
		}
	}
	return footerHTML;
}

function getButtonHTML(button, index) {
	let type;
	let text;
	if (typeof button === 'object') {
		if ('type' in button) {
			type = button.type;
		}
		text = button.text;
	} else {
		if (index === 0) {
			type = 'secondary';
		} else {
			type = 'primary';
		}
		text = button;
	}
	return `<sas-button type="${type}" text="${text}"></sas-button>`;
}

function addListener(options) {
	const buttonContainer = document.querySelector('.sas-dialog-swal-custom-footer');
	buttonContainer.addEventListener('buttonClicked', (event) => {
		event.stopPropagation();
		const buttonText = event.target.text;
		const customEvent = new CustomEvent('dialogUpdate', {
			bubbles: true,
			cancelable: true,
			detail: buttonText,
		});
		buttonContainer.dispatchEvent(customEvent);
	});

	if(options.showCloseIcon === true) {
		const closeIcon = document.querySelector('#sas-dialog-swal-cancel-icon');
		closeIcon.addEventListener('click', (event) => {
			event.stopPropagation();

			const customEvent = new CustomEvent('dialogCancelIconClicked', {
				bubbles: true,
				cancelable: true
			});

			closeIcon.dispatchEvent(customEvent);
		});
	}
}

function preventKeyEvent() {
	const container = document.querySelector('#swal2-html-container');
	container.addEventListener('keyup', event => event.stopPropagation());
	container.addEventListener('keydown', event => event.stopPropagation());
}

export class SasDialog {
	dialog;
	options;
	
	constructor(options = {}) {
		this.options = options;
		this.dialog = init(options);
		this.fire();
	}

	fire() {
		this.dialog.fire();
		let content = getContentHTML(this.options);
		if (content === 'Dom element, needs append') {
			document.querySelector('#swal2-content').append(this.options.content);
		}
	}

	close() {
		this.dialog.close();
	}

	getElement() {
		return this.dialog.getContainer();
	}
}
