// sas-dialog.stories.js
import readme from './readme.md';

export default {
  title: 'Higher Order/Dialog',
  parameters: {
    notes: {
      markdown: readme,
    },
    actions: {
      handles: ['dialogUpdate', 'sas-dialog'],
    },
  },
};

export const Basic = () => `
  <sas-button text="Open" onclick="openDialog()"></sas-button>
  <script>
    function openDialog() {
      let dialog = new SasDialog({
        title: 'Dialog title',
        content: 'Dialog content',
        button: 'Done',
      });

      dialog.getElement().addEventListener('dialogUpdate', () => {
        dialog.close();
      });
    }
  </script>
`;

export const TypeAlert = () => `
  <h3>Type alert with status success</h3>
  <sas-button text="Success" onclick="openSuccess()"></sas-button>
  <script>
    function openSuccess() {
      let dialog = new SasDialog({
        type: 'alert',
        status: 'success',
        title: 'Success alert title',
        content: 'Success alert content',
        button: 'Done',
      });

      dialog.getElement().addEventListener('dialogUpdate', () => {
        dialog.close();
      });
    }
  </script>

  <h3>Type alert with status info</h3>
  <sas-button text="Info" onclick="openInfo()"></sas-button>
  <script>
    function openInfo() {
      let dialog = new SasDialog({
        type: 'alert',
        status: 'info',
        title: 'Info alert title',
        content: 'Info alert content',
        button: 'Done',
      });

      dialog.getElement().addEventListener('dialogUpdate', () => {
        dialog.close();
      });
    }
  </script>

  <h3>Type alert with status warning</h3>
  <sas-button text="Warning" onclick="openWarning()"></sas-button>
  <script>
    function openWarning() {
      let dialog = new SasDialog({
        type: 'alert',
        status: 'warning',
        title: 'Warning alert title',
        content: 'Warning alert content',
        button: 'Done',
      });

      dialog.getElement().addEventListener('dialogUpdate', () => {
        dialog.close();
      });
    }
  </script>

  <h3>Type alert with status error</h3>
  <sas-button text="Error" onclick="openError()"></sas-button>
  <script>
    function openError() {
      let dialog = new SasDialog({
        type: 'alert',
        status: 'error',
        title: 'Error alert title',
        content: 'Error alert content',
        button: 'Done',
      });

      dialog.getElement().addEventListener('dialogUpdate', () => {
        dialog.close();
      });
    }
  </script>
`;

export const Status = () => `
  <h3>Default size</h3>
  <sas-button text="Default" onclick="openDefault()"></sas-button>
  <script>
    function openDefault() {
      let dialog = new SasDialog({
        title: 'Dialog title',
        content: 'Dialog content',
        button: 'Done',
      });

      dialog.getElement().addEventListener('dialogUpdate', () => {
        dialog.close();
      });
    }
  </script>

  <h3>Custom size (current is 100%)</h3>
  <sas-button text="Custom" onclick="openLarge()"></sas-button>
  <script>
    function openLarge() {
      let dialog = new SasDialog({
        width: '100%',
        title: 'Dialog title',
        content: 'Dialog content',
        button: 'Done',
      });

      dialog.getElement().addEventListener('dialogUpdate', () => {
        dialog.close();
      });
    }
  </script>

  <h3>Complex</h3>
  <sas-button text="Complex" onclick="openDialog()"></sas-button>
  <script>
    function openDialog() {
      const contentHTML = "<div><sas-date-field></sas-date-field></div>"
      let dialog = new SasDialog({
        title: 'Dialog title',
        content: [contentHTML, 'Click button will alert button text', 'Then close dialog'],
        button: ['Cancel', 'Save', 'Confirm'],
      });

      dialog.getElement().addEventListener('dialogUpdate', (event) => {
        alert('Clicked button text is "' + event.detail + '"');
        dialog.close();
      });
    }
  </script>
`;
