import { Component, Host, Prop, h, Element, Listen, Method, } from '@stencil/core';
/**
 * @slot control - used for content which can toggle menu
 * @slot menu - used for menu content
 */
export class SasDropdown {
  constructor() {
    /**
     * Display as block element
     */
    this.block = false;
    /**
     * Close menu or not after click menu item
     */
    this.clickClose = true;
    /**
     * Default vertical position if top and bottom are both available
     */
    this.positionV = 'bottom';
    /**
     * Default horizontal position if left and right are both available
     */
    this.positionH = 'right';
    this.toggleList = () => {
      const list = this.el.shadowRoot.querySelector('.list');
      const height = list.getBoundingClientRect().height;
      if (height > 0) {
        list.classList.add('hide');
      }
      else {
        list.classList.remove('hide');
      }
      this.updatePosition();
    };
    this.clickEvent = (event) => {
      event.stopPropagation();
    };
  }
  componentDidRender() {
    this.fixPosition();
  }
  listItemClickEventUsedInternalHandler() {
    this.close();
  }
  /**
   * Close dropdown programmatically
   */
  async close() {
    const list = this.el.shadowRoot.querySelector('.list');
    list.classList.add('hide');
  }
  // only works when no proper position for menu
  // set to default position
  fixPosition() {
    const list = this.el.shadowRoot.querySelector('.list');
    const listPosition = list.getBoundingClientRect();
    if (listPosition.top < 0) {
      this.positionV = 'bottom';
    }
    if (listPosition.left < 0) {
      this.positionH = 'right';
    }
  }
  updatePosition() {
    const list = this.el.shadowRoot.querySelector('.list');
    const listPosition = list.getBoundingClientRect();
    if (listPosition.top < 0) {
      this.positionV = 'bottom';
    }
    else if (listPosition.bottom > document.documentElement.clientHeight) {
      this.positionV = 'top';
    }
    if (listPosition.left < 0) {
      this.positionH = 'right';
    }
    else if (listPosition.right > document.documentElement.clientWidth) {
      this.positionH = 'left';
    }
  }
  render() {
    return (h(Host, { tabindex: 0 },
      h("div", { class: "container" },
        h("div", { class: "control", 
          // mouseDown event is faster than focus event, click event is slower than focus event
          onMouseDown: this.toggleList },
          h("slot", { name: "control" })),
        h("div", { class: `list hide ${this.positionH} ${this.positionV}`, tabindex: 0, onClick: this.clickEvent },
          h("slot", { name: "menu" })))));
  }
  static get is() { return "sas-dropdown"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-dropdown.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-dropdown.css"]
  }; }
  static get properties() { return {
    "block": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Display as block element"
      },
      "attribute": "block",
      "reflect": true,
      "defaultValue": "false"
    },
    "clickClose": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Close menu or not after click menu item"
      },
      "attribute": "click-close",
      "reflect": true,
      "defaultValue": "true"
    },
    "positionV": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Default vertical position if top and bottom are both available"
      },
      "attribute": "position-v",
      "reflect": true,
      "defaultValue": "'bottom'"
    },
    "positionH": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Default horizontal position if left and right are both available"
      },
      "attribute": "position-h",
      "reflect": true,
      "defaultValue": "'right'"
    }
  }; }
  static get methods() { return {
    "close": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Close dropdown programmatically",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get listeners() { return [{
      "name": "listItemClickEventUsedInternal",
      "method": "listItemClickEventUsedInternalHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
