import { Component, Element, Event, Host, Listen, Prop, Watch, h, } from '@stencil/core';
import { keydownEvent } from '../../global/global';
export class SasFormInputTemplate {
  constructor() {
    this.containerVerticalPosition = null;
    this.containerHorizontalPosition = null;
    /**
     * Label
     */
    this.label = '';
    /**
     * Label position
     */
    this.outside = false;
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * Placeholder
     */
    this.placeholder = '';
    /**
     * Placeholder
     */
    this.type = '';
    /**
     * Value
     */
    this.value = '';
    /**
     * Max value, only works with type number, range and date
     */
    this.max = null;
    /**
     * Min value, only works with type number, range and date
     */
    this.min = null;
    /**
     * Icon Right
     */
    this.iconRight = '';
    /**
     * Readonly
     */
    this.readonly = false;
    /**
     * Error
     */
    this.error = false;
    /**
     * Disabled
     */
    this.disabled = false;
    /**
     * Clear
     */
    this.clear = false;
    /**
     * Container vertical position
     */
    this.position = null;
    /**
     * Container horizontal position
     */
    this.positionH = null;
    /**
     * Show or hide content under input
     */
    this.show = false;
    /**
     * Close menu or not after click menu item
     */
    this.clickClose = false;
    /**
     * Minimum amount of space required above component, before rendering container above component
     */
    this.minimumPixelsAboveComponentRequired = 0;
    /**
     * Minimum amount of space required to the right of the component, before rendering container to the right of the component
     */
    this.minimumPixelsRightOfComponentRequired = 0;
    this.handleInputClick = (event) => {
      console.log("handleInputClick");
      event.stopPropagation();
      if (!this.disabled) {
        const container = this.el.shadowRoot.querySelector('.container');
        const height = container.getBoundingClientRect().height;
        console.log(`handleInputClick: container`);
        console.log(container);
        console.log(`handleInputClick: height: ${height}`);
        // if (height > 0) {
        //   this.show = false;
        // } else {
        //   this.show = true;
        // }
        this.show = true;
        console.log(`handleInputClick: show: ${this.show}`);
        this.inputContainerClick.emit();
      }
    };
    this.handleContainerClick = (event) => {
      console.log("handleContainerClick");
      event.stopPropagation();
      console.log(`handleContainerClick: clickClose: ${this.clickClose}`);
      // if (this.clickClose) {
      //   setTimeout(() => {
      //     this.show = false;
      //   }, 100);
      // }
    };
    this.updateContainerPosition = () => {
      console.log("updateContainerPosition");
      this.updateVerticalContainerPosition();
      this.updateContainerHorizontalPosition();
    };
    this.updateVerticalContainerPosition = () => {
      if (Boolean(this.position)) {
        this.containerVerticalPosition = this.position;
      }
      else {
        const top = this.el.getBoundingClientRect().top;
        const clientHeight = document.documentElement.clientHeight;
        console.log(`updateVerticalContainerPosition: top: ${top}`);
        console.log(`updateVerticalContainerPosition: clientHeight: ${clientHeight}`);
        if (clientHeight < top * 2 && top > this.minimumPixelsAboveComponentRequired) {
          this.containerVerticalPosition = 'top';
        }
        else {
          this.containerVerticalPosition = 'bottom';
        }
      }
      console.log(`updateVerticalContainerPosition: containerVerticalPosition: ${this.containerVerticalPosition}`);
    };
    this.updateContainerHorizontalPosition = () => {
      if (Boolean(this.positionH)) {
        this.containerHorizontalPosition = this.positionH;
      }
      else {
        const left = this.el.getBoundingClientRect().left;
        const clientWidth = document.documentElement.clientWidth;
        console.log(`updateContainerHorizontalPosition: left: ${left}`);
        console.log(`updateContainerHorizontalPosition: clientWidth: ${clientWidth}`);
        if (clientWidth - this.minimumPixelsRightOfComponentRequired > left) {
          this.containerHorizontalPosition = 'left';
        }
        else {
          this.containerHorizontalPosition = 'right';
        }
      }
      console.log(`updateContainerHorizontalPosition: containerHorizontalPosition: ${this.containerHorizontalPosition}`);
    };
  }
  showHandler(newValue) {
    if (newValue) {
      this.updateContainerPosition();
      this.showContainer();
    }
    else {
      this.hideContainer();
    }
  }
  listItemClickEventUsedInternalHandler() {
    // keep focus so dropdown list will not hide if there are dropdown components inside dropdown list
    this.el.focus();
  }
  showContainer() {
    console.log("showContainer");
    const element = this.el.shadowRoot.querySelector('.container');
    console.log("showContainer: element");
    console.log(element);
    element.classList.remove('hide');
  }
  hideContainer() {
    console.log("hideContainer");
    const element = this.el.shadowRoot.querySelector('.container');
    console.log("hideContainer: element");
    console.log(element);
    element.classList.add('hide');
  }
  render() {
    console.log("sas-form-input-template: render()");
    return (h(Host, { tabindex: 0, "data-message": "" },
      h("sas-input", { label: this.label, outside: this.outside, required: this.required, type: this.type, placeholder: this.placeholder, "icon-right": this.iconRight, value: this.value, min: this.min, max: this.max, readonly: this.readonly, disabled: this.disabled, clear: this.clear, onKeyDown: keydownEvent, tabindex: 0, 
        // mouseDown event is faster than focus event, click event is slower than focus event
        onMouseDown: this.handleInputClick },
        h("slot", { name: "input" })),
      h("div", { class: `container hide ${this.containerVerticalPosition} ${this.containerHorizontalPosition}`, onMouseDown: this.handleContainerClick },
        h("slot", { name: "container" }))));
  }
  static get is() { return "sas-form-input-template"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-form-input-template.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-form-input-template.css"]
  }; }
  static get properties() { return {
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label"
      },
      "attribute": "label",
      "reflect": true,
      "defaultValue": "''"
    },
    "outside": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label position"
      },
      "attribute": "outside",
      "reflect": true,
      "defaultValue": "false"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Shows a star symbol behind label"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Placeholder"
      },
      "attribute": "placeholder",
      "reflect": false,
      "defaultValue": "''"
    },
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Placeholder"
      },
      "attribute": "type",
      "reflect": false,
      "defaultValue": "''"
    },
    "value": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Value"
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "''"
    },
    "max": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Max value, only works with type number, range and date"
      },
      "attribute": "max",
      "reflect": true,
      "defaultValue": "null"
    },
    "min": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Min value, only works with type number, range and date"
      },
      "attribute": "min",
      "reflect": true,
      "defaultValue": "null"
    },
    "iconRight": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon Right"
      },
      "attribute": "icon-right",
      "reflect": false,
      "defaultValue": "''"
    },
    "readonly": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Readonly"
      },
      "attribute": "readonly",
      "reflect": false,
      "defaultValue": "false"
    },
    "error": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Error"
      },
      "attribute": "error",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "clear": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Clear"
      },
      "attribute": "clear",
      "reflect": true,
      "defaultValue": "false"
    },
    "position": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "FormInputTemplateVerticalPosition",
        "resolved": "\"bottom\" | \"top\"",
        "references": {
          "FormInputTemplateVerticalPosition": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Container vertical position"
      },
      "attribute": "position",
      "reflect": true,
      "defaultValue": "null"
    },
    "positionH": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "FormInputTemplateHorizontalPosition",
        "resolved": "\"left\" | \"right\"",
        "references": {
          "FormInputTemplateHorizontalPosition": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Container horizontal position"
      },
      "attribute": "position-h",
      "reflect": true,
      "defaultValue": "null"
    },
    "show": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show or hide content under input"
      },
      "attribute": "show",
      "reflect": true,
      "defaultValue": "false"
    },
    "clickClose": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Close menu or not after click menu item"
      },
      "attribute": "click-close",
      "reflect": true,
      "defaultValue": "false"
    },
    "minimumPixelsAboveComponentRequired": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Minimum amount of space required above component, before rendering container above component"
      },
      "attribute": "minimum-pixels-above-component-required",
      "reflect": true,
      "defaultValue": "0"
    },
    "minimumPixelsRightOfComponentRequired": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Minimum amount of space required to the right of the component, before rendering container to the right of the component"
      },
      "attribute": "minimum-pixels-right-of-component-required",
      "reflect": true,
      "defaultValue": "0"
    }
  }; }
  static get events() { return [{
      "method": "inputContainerClick",
      "name": "inputContainerClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the container is clicked by user (only for components internal use)"
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "show",
      "methodName": "showHandler"
    }]; }
  static get listeners() { return [{
      "name": "listItemClickEventUsedInternal",
      "method": "listItemClickEventUsedInternalHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
