import { 
// eslint-disable-next-line no-unused-vars
Component, Host, h, Prop, Element, } from '@stencil/core';
/**
 * @slot default - default slot
 */
export class SasImage {
  constructor() {
    /**
     * Image blob | MediaSource | object
     */
    this.imageObject = null;
    /**
     * Image source
     */
    this.src = '';
    /**
     * Lazy load
     */
    this.lazy = false;
  }
  componentDidLoad() {
    this.updateSrc();
  }
  getSrc() {
    if (this.imageObject) {
      if ('url' in this.imageObject) {
        return this.imageObject.url;
      }
      else {
        const src = URL.createObjectURL(this.imageObject);
        return src;
      }
    }
    return this.src;
  }
  updateSrc() {
    const src = this.getSrc();
    const hoseElement = this.el;
    let imageElement = hoseElement.shadowRoot.querySelector('img');
    if (this.lazy) {
      let observer = new IntersectionObserver(function (entries) {
        entries.forEach(function (entry) {
          if (entry.isIntersecting) {
            imageElement.src = src;
            observer.unobserve(hoseElement);
          }
        });
      });
      observer.observe(hoseElement);
    }
    else {
      imageElement.src = src;
    }
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        h("img", { class: "image" }),
        h("slot", null))));
  }
  static get is() { return "sas-image"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-image.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-image.css"]
  }; }
  static get properties() { return {
    "imageObject": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Image blob | MediaSource | object"
      },
      "attribute": "image-object",
      "reflect": false,
      "defaultValue": "null"
    },
    "src": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Image source"
      },
      "attribute": "src",
      "reflect": true,
      "defaultValue": "''"
    },
    "lazy": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Lazy load"
      },
      "attribute": "lazy",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get elementRef() { return "el"; }
}
