import { Component, Element, Event, Host, Method, h } from '@stencil/core';
import intlTelInput from 'intl-tel-input';
export class SasInputTel {
  componentDidLoad() {
    var _a, _b, _c, _d;
    const input = this.el.shadowRoot.querySelector("#phone");
    const placeholder = this.el.getAttribute('data-placeholder');
    if (Boolean(placeholder)) {
      input.placeholder = placeholder;
    }
    this.iti = intlTelInput(input, {
      initialCountry: (_a = this.el.getAttribute('data-country')) !== null && _a !== void 0 ? _a : 'au',
      placeholderNumberType: (_b = this.el.getAttribute('data-placeholderNumberType')) !== null && _b !== void 0 ? _b : 'MOBILE',
      preferredCountries: ['au', 'ca', 'gb', 'nz', 'us'],
      utilsScript: 'https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/18.2.1/js/utils.min.js',
    });
    const thisClone = this;
    this.iti.promise.then(() => {
      if (Boolean(thisClone.el.getAttribute('data-value'))) {
        thisClone.update(thisClone.el.getAttribute('data-value'));
      }
    });
    input.addEventListener('countrychange', function () {
      thisClone.update();
    });
    input.addEventListener('input', function () {
      thisClone.update();
    });
    const input2 = this.el.shadowRoot.querySelector("#phone2");
    this.iti2 = intlTelInput(input2, {
      initialCountry: (_c = this.el.getAttribute('data-country')) !== null && _c !== void 0 ? _c : 'au',
      nationalMode: false,
      placeholderNumberType: (_d = this.el.getAttribute('data-placeholderNumberType')) !== null && _d !== void 0 ? _d : 'MOBILE',
      preferredCountries: ['au', 'ca', 'gb', 'nz', 'us'],
      utilsScript: 'https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/18.2.1/js/utils.min.js',
    });
  }
  /**
   * Check the componnet validity (only for components internal use)
   */
  async getValidity() {
    return this.iti.isPossibleNumber();
  }
  /**
   * Set country (only for components internal use)
   */
  async setCountry(country) {
    this.iti.setCountry(country);
  }
  /**
   * Set placeholder (only for components internal use)
   */
  async setPlaceholder(type) {
    this.iti.setPlaceholderNumberType(type);
  }
  /**
   * Set value and trigger inputTelUpdate event programmatically (only for components internal use)
   */
  async update(value = 'default value used internal') {
    const input = this.el.shadowRoot.querySelector("#phone");
    if (value != 'default value used internal') {
      input.value = this.getFilteredValue(value);
    }
    else {
      input.value = this.getFilteredValue(input.value);
    }
    let isValidNumber = false;
    // only check format
    // parent input component will check value missing or not
    if (this.iti.isPossibleNumber() || input.value === '') {
      this.iti.setNumber(input.value);
      isValidNumber = true;
    }
    if (input.value.length === 3) {
      isValidNumber = true;
    }
    const object = {
      country: this.iti.getSelectedCountryData().iso2,
      value: this.getFormatNumber(isValidNumber),
      isValidNumber: isValidNumber,
    };
    this.inputTelUpdate.emit(object);
  }
  getFilteredValue(value) {
    return value.replace(/[^0-9\+]/g, '');
  }
  getFormatNumber(isValidNumber) {
    const input = this.el.shadowRoot.querySelector("#phone");
    const input2 = this.el.shadowRoot.querySelector("#phone2");
    if (isValidNumber) {
      const internationalNumber = this.iti.getNumber();
      this.iti2.setNumber(internationalNumber);
      return input2.value;
    }
    return input.value;
  }
  render() {
    return (h(Host, null,
      h("input", { id: "phone", type: "tel" }),
      h("div", { id: "phone2Container" },
        h("input", { id: "phone2", type: "tel" }))));
  }
  static get is() { return "sas-input-tel"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-input-tel.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-input-tel.css"]
  }; }
  static get events() { return [{
      "method": "inputTelUpdate",
      "name": "inputTelUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed (only for components internal use)"
      },
      "complexType": {
        "original": "object",
        "resolved": "object",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "getValidity": {
      "complexType": {
        "signature": "() => Promise<any>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<any>"
      },
      "docs": {
        "text": "Check the componnet validity (only for components internal use)",
        "tags": []
      }
    },
    "setCountry": {
      "complexType": {
        "signature": "(country: any) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set country (only for components internal use)",
        "tags": []
      }
    },
    "setPlaceholder": {
      "complexType": {
        "signature": "(type: any) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set placeholder (only for components internal use)",
        "tags": []
      }
    },
    "update": {
      "complexType": {
        "signature": "(value?: string) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLInputElement": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set value and trigger inputTelUpdate event programmatically (only for components internal use)",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
}
