import { Component, Host, h, Prop, Event, Element, Watch } from '@stencil/core';
export class SasInputTextarea {
  constructor() {
    this.lineHeight = 24;
    this.baseScrollHeight = -1;
    /**
     * Placeholder
     */
    this.placeholder = '';
    /**
     * Maximum number of characters allowed
     */
    this.maxlength = 1000;
    /**
      * Minimum number of characters allowed
      */
    this.minlength = 0;
    /**
     * Value
     */
    this.value = '';
    this.handleInput = (event) => {
      this.resize();
      this.inputTextareaUpdate.emit(event.target.value);
    };
  }
  watchHandler() {
    const element = this.el.shadowRoot.querySelector('.textarea');
    element.value = this.value;
    this.resize();
  }
  componentDidLoad() {
    this.resize();
  }
  resize() {
    if (this.baseScrollHeight === -1) {
      this.baseScrollHeight = this.getBaseScrollHeight();
    }
    const currentScrollHeight = this.getCurrentScrollHeight();
    const rows = Math.ceil((currentScrollHeight - this.baseScrollHeight) / this.lineHeight) + 1;
    const element = this.el.shadowRoot.querySelector('.textarea');
    element.rows = rows;
  }
  getBaseScrollHeight() {
    const element = this.el.shadowRoot.querySelector('.hidden-textarea');
    element.rows = 1;
    element.value = '';
    return element.scrollHeight;
  }
  getCurrentScrollHeight() {
    const element = this.el.shadowRoot.querySelector('.hidden-textarea');
    element.value = this.value;
    const currentScrollHeight = element.scrollHeight;
    element.value = '';
    return currentScrollHeight;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container", "data-count": Boolean(this.maxlength) && Boolean(this.value) ? this.maxlength - this.value.length : '' },
        h("textarea", { class: "hidden-textarea", rows: 1 }),
        h("textarea", { class: "textarea", rows: 1, placeholder: this.placeholder, maxlength: this.maxlength, minlength: this.minlength, value: this.value, onInput: this.handleInput }))));
  }
  static get is() { return "sas-input-textarea"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-input-textarea.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-input-textarea.css"]
  }; }
  static get properties() { return {
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Placeholder"
      },
      "attribute": "placeholder",
      "reflect": true,
      "defaultValue": "''"
    },
    "maxlength": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Maximum number of characters allowed"
      },
      "attribute": "maxlength",
      "reflect": true,
      "defaultValue": "1000"
    },
    "minlength": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Minimum number of characters allowed"
      },
      "attribute": "minlength",
      "reflect": true,
      "defaultValue": "0"
    },
    "value": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Value"
      },
      "attribute": "value",
      "reflect": true,
      "defaultValue": "''"
    }
  }; }
  static get events() { return [{
      "method": "inputTextareaUpdate",
      "name": "inputTextareaUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed (only for components internal use)"
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "watchHandler"
    }]; }
}
