import { Component, Host, h, Prop, Event, Element, Method, Watch, Listen, } from '@stencil/core';
import { HTMLDecodeString } from '../../global/global';
export class SasInput {
  constructor() {
    this.isUpdateFromSubcomponent = false;
    /**
     * Type
     */
    this.type = 'text';
    /**
     * Placeholder
     */
    this.placeholder = '';
    /**
     * name
     */
    this.name = '';
    /**
     * Value
     */
    this.value = '';
    /**
     * Label
     */
    this.label = '';
    /**
     * Label position
     */
    this.outside = false;
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * Maximum number of characters allowed
     */
    this.maxlength = 1000;
    /**
     * Minimum number of characters allowed
     */
    this.minlength = 0;
    /**
     * Max value, only works with type number, range and date
     */
    this.max = null;
    /**
     * Min value, only works with type number, range and date
     */
    this.min = null;
    /**
     * Shows an icon on left
     */
    this.iconLeft = '';
    /**
     * Shows an icon on right, will be overrided if type is password
     */
    this.iconRight = '';
    /**
     * Show a clear icon to clear input
     */
    this.clear = false;
    /**
     * Readonly
     */
    this.readonly = false;
    /**
     * Disabled
     */
    this.disabled = false;
    /**
     * Changes to false automatically when value updates
     */
    this.error = false;
    /**
     * Only shows when prop error is true, auto generated
     */
    this.message = '';
    // type tel props
    /**
     * Country
     */
    this.country = 'au';
    /**
     * Placeholder Number Type
     */
    this.placeholderNumberType = 'MOBILE';
    /**
     * phone number validity
     */
    this.isValidNumber = true;
    this.handleInput = (event) => {
      this.value = event.target.value;
      this.inputUpdate.emit(event.target.value);
      this.inputupdate.emit(event.target.value);
    };
    this.focusEvent = () => {
      this.inputFocus.emit();
    };
    this.blurEvent = () => {
      this.inputBlur.emit();
    };
    this.clickEvent = (event) => {
      if (this.type === 'date' || this.type === "time") {
        event.preventDefault();
      }
    };
    this.togglePassword = (event) => {
      const icon = event.target.icon;
      const input = this.el.shadowRoot.querySelector('input');
      if (icon === 'fas fa-eye-slash') {
        event.target.icon = 'fas fa-eye';
        input.type = 'text';
      }
      else {
        event.target.icon = 'fas fa-eye-slash';
        input.type = 'password';
      }
    };
    this.iconRightClickHandler = () => {
      this.inputiconrightclicked.emit();
    };
    this.clearInput = (event) => {
      event.stopPropagation();
      // force date field and time field to clear incomplete input
      // as when input not complete, value not update
      // clear value will not trigger re-render
      if (this.type === 'date' || this.type === 'time') {
        const inputElement = this.el.shadowRoot.querySelector('input');
        inputElement.value = '';
      }
      this.value = '';
      this.update();
      this.inputClear.emit();
    };
  }
  watchCountryHandler() {
    if (this.type === 'tel') {
      const inputTel = this.el.shadowRoot.querySelector('sas-input-tel');
      inputTel.setCountry(this.country);
    }
  }
  async watchValueHandler() {
    if (this.isUpdateFromSubcomponent) {
      this.isUpdateFromSubcomponent = false;
    }
    else {
      if (this.type === 'tel') {
        const inputTel = this.el.shadowRoot.querySelector('sas-input-tel');
        inputTel.update(this.value);
      }
      else {
        this.value = HTMLDecodeString(this.value);
      }
    }
    this.resetErrorStatus();
  }
  watchPlaceholderNumberTypeHandler() {
    if (this.type === 'tel') {
      const inputTel = this.el.shadowRoot.querySelector('sas-input-tel');
      inputTel.setPlaceholder(this.placeholderNumberType);
    }
  }
  async componentWillLoad() {
    this.value = HTMLDecodeString(this.value);
    this.resetErrorStatus();
  }
  inputTelUpdateHandler(event) {
    if (this.country !== event.detail.country) {
      this.country = event.detail.country;
    }
    if (this.isValidNumber !== event.detail.isValidNumber) {
      this.isValidNumber = event.detail.isValidNumber;
    }
    if (this.value !== event.detail.value) {
      this.isUpdateFromSubcomponent = true;
      this.value = event.detail.value;
      this.inputUpdate.emit(event.detail.value);
      this.inputupdate.emit(event.detail.value);
    }
  }
  inputTextareaUpdateHandler(event) {
    this.value = event.detail;
    this.inputUpdate.emit(event.detail);
    this.inputupdate.emit(event.detail);
  }
  /**
   * Trigger inputupdate event programmatically
   */
  async update() {
    const value = this.value;
    this.inputUpdate.emit(value);
    this.inputupdate.emit(value);
  }
  /**
   * Validate
   */
  async validate(customErrorMessage = '') {
    const input = document.createElement('input');
    if (this.type !== 'textarea') {
      input.type = this.type;
    }
    input.required = this.required;
    input.maxLength = this.maxlength;
    input.minLength = this.minlength;
    input.max = this.max;
    input.min = this.min;
    input.value = Boolean(this.value) ? this.value.trim() : '';
    let isCustomError = false;
    if (customErrorMessage !== '') {
      isCustomError = true;
    }
    // check tooShort mannually
    // as tooShort not trigger by using js to set value
    let validity = {
      badInput: input.validity.badInput,
      customError: isCustomError,
      patternMismatch: input.validity.patternMismatch,
      rangeOverflow: input.validity.rangeOverflow,
      rangeUnderflow: input.validity.rangeUnderflow,
      stepMismatch: input.validity.stepMismatch,
      tooLong: input.validity.tooLong,
      tooShort: input.validity.tooShort,
      typeMismatch: input.validity.typeMismatch,
      valid: input.validity.valid && !isCustomError,
      valueMissing: input.validity.valueMissing,
    };
    if (Boolean(this.value)) {
      const length = this.value.length;
      if (length < this.minlength) {
        validity.tooShort = true;
        validity.valid = false;
      }
    }
    if (this.type === 'tel') {
      if (!this.isValidNumber) {
        validity.typeMismatch = true;
        validity.valid = false;
      }
    }
    this.validity = validity;
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus(errorMessage = '') {
    await this.validate(errorMessage);
    this.error = !this.validity.valid;
    if (this.error) {
      this.message = this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    this.error = false;
    this.message = '';
  }
  /**
   * Set custom error message and validate
   */
  async setCustomValidity(errorMessage = 'custom validation error message') {
    this.message = errorMessage;
    await this.updateErrorStatus(errorMessage);
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      const prefix = Boolean(this.label) ? this.label : 'Field';
      return `${prefix} is required`;
    }
    if (validity.typeMismatch || validity.badInput) {
      return `Invalid ${this.type} format`;
    }
    if (validity.rangeOverflow) {
      if (this.type === 'date' && this.value[0] !== '0') {
        this.inputUpdate.emit('reset request');
        this.inputupdate.emit('reset request');
      }
      return `${this.label} must be less or equal to ${this.max}`;
    }
    if (validity.rangeUnderflow) {
      if (this.type === 'date' && this.value[0] !== '0') {
        this.inputUpdate.emit('reset request');
        this.inputupdate.emit('reset request');
      }
      return `${this.label} must be larger or equal to ${this.min}`;
    }
    if (validity.tooShort) {
      return `${this.label}'s length must be larger or equal to ${this.minlength} characters`;
    }
    return this.message;
  }
  getLeftIcon() {
    if (Boolean(this.iconLeft)) {
      const icon = this.iconLeft.slice(7);
      if (icon === 'search') {
        return "\uf002" + ' ';
      }
    }
    return '';
  }
  generateRightIcon() {
    let rightIcon = null;
    if (this.type === 'password') {
      rightIcon = h("sas-icon", { class: "input-right-icon", icon: "fas fa-eye-slash", onClick: this.togglePassword });
    }
    else {
      if (Boolean(this.iconRight)) {
        rightIcon = h("sas-icon", { class: "input-right-icon", icon: this.iconRight, onClick: this.iconRightClickHandler });
      }
    }
    return rightIcon;
  }
  render() {
    return (h(Host
    // onBlur = {() => this.updateErrorStatus()}
    , null,
      h("div", { class: "input-container", "data-message": this.message },
        Boolean(this.label)
          ? h("label", { class: this.outside ? 'outside-label' : 'inside-label' },
            this.label,
            this.required
              ? h("span", { class: "required-symbol" }, "*")
              : null)
          : null,
        this.type === 'tel'
          ? h("sas-input-tel", { "data-country": this.country, "data-value": this.value, "data-placeholder": this.placeholder, "data-placeholderNumberType": this.placeholderNumberType })
          : null,
        this.type === 'textarea'
          ? h("sas-input-textarea", { placeholder: this.getLeftIcon() + this.placeholder, maxlength: this.maxlength, minlength: this.minlength, value: this.value })
          : null,
        this.type === 'date'
          ? h("div", { class: "input-date-container" },
            h("input", { class: "input", type: "date", placeholder: this.getLeftIcon() + this.placeholder, readonly: this.readonly, required: true, maxlength: this.maxlength, minlength: this.minlength, max: this.max, min: this.min, value: this.value, onFocus: this.focusEvent, onInput: this.handleInput, onBlur: this.blurEvent, onClick: this.clickEvent }),
            localStorage.getItem('platform') === 'ios' // prevent user interact with input field to trigger native date picker
              ? h("div", { class: "input-date-cover" })
              : null)
          : null,
        this.type !== 'tel' && this.type !== 'textarea' && this.type !== 'date'
          ? h("input", { class: "input", type: this.type, placeholder: this.getLeftIcon() + this.placeholder, readonly: this.readonly, required: this.required || this.type === 'date', maxlength: this.maxlength, minlength: this.minlength, max: this.max, min: this.min, value: this.value, onFocus: this.focusEvent, onInput: this.handleInput, onBlur: this.blurEvent, onClick: this.clickEvent })
          : null,
        h("slot", null),
        h("div", { class: "right-icon-container" },
          this.clear
            ? h("sas-icon", { class: "clear-icon", type: "round", icon: "fas fa-times", onMouseDown: this.clearInput })
            : null,
          this.generateRightIcon()))));
  }
  static get is() { return "sas-input"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-input.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-input.css"]
  }; }
  static get properties() { return {
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Type"
      },
      "attribute": "type",
      "reflect": true,
      "defaultValue": "'text'"
    },
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Placeholder"
      },
      "attribute": "placeholder",
      "reflect": true,
      "defaultValue": "''"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "''"
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Value"
      },
      "attribute": "value",
      "reflect": true,
      "defaultValue": "''"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label"
      },
      "attribute": "label",
      "reflect": true,
      "defaultValue": "''"
    },
    "outside": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label position"
      },
      "attribute": "outside",
      "reflect": true,
      "defaultValue": "false"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Shows a star symbol behind label"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "maxlength": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Maximum number of characters allowed"
      },
      "attribute": "maxlength",
      "reflect": true,
      "defaultValue": "1000"
    },
    "minlength": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Minimum number of characters allowed"
      },
      "attribute": "minlength",
      "reflect": true,
      "defaultValue": "0"
    },
    "max": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Max value, only works with type number, range and date"
      },
      "attribute": "max",
      "reflect": true,
      "defaultValue": "null"
    },
    "min": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Min value, only works with type number, range and date"
      },
      "attribute": "min",
      "reflect": true,
      "defaultValue": "null"
    },
    "iconLeft": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Shows an icon on left"
      },
      "attribute": "icon-left",
      "reflect": true,
      "defaultValue": "''"
    },
    "iconRight": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Shows an icon on right, will be overrided if type is password"
      },
      "attribute": "icon-right",
      "reflect": true,
      "defaultValue": "''"
    },
    "clear": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show a clear icon to clear input"
      },
      "attribute": "clear",
      "reflect": true,
      "defaultValue": "false"
    },
    "readonly": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Readonly"
      },
      "attribute": "readonly",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "validity": {
      "type": "any",
      "mutable": true,
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Validity"
      },
      "attribute": "validity",
      "reflect": false
    },
    "error": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Changes to false automatically when value updates"
      },
      "attribute": "error",
      "reflect": true,
      "defaultValue": "false"
    },
    "message": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Only shows when prop error is true, auto generated"
      },
      "attribute": "message",
      "reflect": true,
      "defaultValue": "''"
    },
    "country": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Country"
      },
      "attribute": "country",
      "reflect": true,
      "defaultValue": "'au'"
    },
    "placeholderNumberType": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Placeholder Number Type"
      },
      "attribute": "placeholder-number-type",
      "reflect": true,
      "defaultValue": "'MOBILE'"
    },
    "isValidNumber": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "phone number validity"
      },
      "attribute": "is-valid-number",
      "reflect": true,
      "defaultValue": "true"
    }
  }; }
  static get events() { return [{
      "method": "inputUpdate",
      "name": "inputUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction (will be deprecated)"
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }, {
      "method": "inputupdate",
      "name": "inputupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction"
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }, {
      "method": "inputiconrightclicked",
      "name": "inputiconrightclicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the right icon is clicked by user"
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "inputClear",
      "name": "inputClear",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is cleared by user (only for components internal use)"
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "inputFocus",
      "name": "inputFocus",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the component is focused"
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "inputBlur",
      "name": "inputBlur",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the component is blurred"
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "update": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Trigger inputupdate event programmatically",
        "tags": []
      }
    },
    "validate": {
      "complexType": {
        "signature": "(customErrorMessage?: string) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Validate",
        "tags": []
      }
    },
    "updateErrorStatus": {
      "complexType": {
        "signature": "(errorMessage?: string) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show error status if value is not valid (only for components internal use)",
        "tags": []
      }
    },
    "resetErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clear error status (only for components internal use)",
        "tags": []
      }
    },
    "setCustomValidity": {
      "complexType": {
        "signature": "(errorMessage?: string) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set custom error message and validate",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "country",
      "methodName": "watchCountryHandler"
    }, {
      "propName": "value",
      "methodName": "watchValueHandler"
    }, {
      "propName": "placeholderNumberType",
      "methodName": "watchPlaceholderNumberTypeHandler"
    }]; }
  static get listeners() { return [{
      "name": "inputTelUpdate",
      "method": "inputTelUpdateHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "inputTextareaUpdate",
      "method": "inputTextareaUpdateHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
