// sas-input.stories.js
import readme from './readme.md';

export default {
  title: 'Basic/Input',
  parameters: {
    notes: {
      markdown: readme,
    },
    actions: {
      handles: ['inputupdate', 'sas-input'],
    },
  },
};

const placeholderNumberTypeOptions = ['MOBILE', 'FIXED_LINE']

export const TypeText = (args) => `
  <h3>Type text</h3>
  <sas-input
    type="${args.type}"
    label="${args.label}"
    outside="${args.outside}"
    placeholder="${args.placeholder}"
    required="${args.required}"
    value="${args.value}"
    icon-left="${args.iconLeft}"
    icon-right="${args.iconRight}"
    disabled="${args.disabled}"
    clear="${args.clear}"
    maxlength="${args.maxlength}"
    minlength="${args.minlength}"
  >
  </sas-input>
`;

TypeText.args = {
  type: 'text',
  label: 'Label',
  outside: false,
  placeholder: 'Placeholder',
  required: true,
  value: '',
  iconLeft: 'fas fa-search',
  iconRight: 'fas fa-chevron-down',
  disabled: false,
  clear: false,
  maxlength: 100,
  minlength: 0,
};

export const TypeTextarea = (args) => `
  <h3>Type textarea</h3>
  <sas-input
    type="${args.type}"
    label="${args.label}"
    outside="${args.outside}"
    placeholder="${args.placeholder}"
    required="${args.required}"
    value="${args.value}"
    icon-left="${args.iconLeft}"
    icon-right="${args.iconRight}"
    disabled="${args.disabled}"
    clear="${args.clear}"
    maxlength="${args.maxlength}"
    minlength="${args.minlength}"
  >
  </sas-input>
`;

TypeTextarea.args = {
  type: 'textarea',
  label: 'Label',
  outside: false,
  placeholder: 'Placeholder',
  required: true,
  value: '',
  iconLeft: 'fas fa-search',
  iconRight: 'fas fa-chevron-down',
  disabled: false,
  clear: false,
  maxlength: 100,
  minlength: 0,
};

export const TypeTel = (args) => `
  <h3>Type tel</h3>
  <sas-input
    type="${args.type}"
    label="${args.label}"
    outside="${args.outside}"
    required="${args.required}"
    value="${args.value}"
    icon-right="${args.iconRight}"
    disabled="${args.disabled}"
    clear="${args.clear}"
    country="${args.country}"
    placeholder-number-type="${args.placeholderNumberType}"
  >
  </sas-input>
`;

TypeTel.args = {
  type: 'tel',
  label: 'Label',
  outside: false,
  required: true,
  value: '',
  iconRight: 'fas fa-chevron-down',
  disabled: false,
  clear: false,
  country: 'au',
  placeholderNumberType: 'MOBILE'
};

TypeTel.argTypes = {
  placeholderNumberType: {
    control: {
      type: 'select',
      options: placeholderNumberTypeOptions,
    },
  },
};

export const TypePassword = (args) => `
  <h3>Type password</h3>
  <sas-input
    type="${args.type}"
    label="${args.label}"
    outside="${args.outside}"
    placeholder="${args.placeholder}"
    required="${args.required}"
    value="${args.value}"
    icon-left="${args.iconLeft}"
    disabled="${args.disabled}"
    clear="${args.clear}"
    maxlength="${args.maxlength}"
    minlength="${args.minlength}"
  >
  </sas-input>
`;

TypePassword.args = {
  type: 'password',
  label: 'Label',
  outside: false,
  placeholder: 'Placeholder',
  required: true,
  value: '',
  iconLeft: 'fas fa-search',
  disabled: false,
  clear: false,
  maxlength: 100,
  minlength: 0,
};

export const Status = (args) => `
  <h3>Default</h3>
  <sas-input
    label="label"
  >
  </sas-input>
  <h3>Label outside</h3>
  <sas-input
    label="label outside"
    outside
  >
  </sas-input>
  <h3>Required</h3>
  <sas-input
    label="reqruied"
    required
  >
  </sas-input>
  <h3>Has left icon</h3>
  <sas-input
    label="Has left icon"
    icon-left="fas fa-search"
  >
  </sas-input>
  <h3>Has right icon</h3>
  <sas-input
    label="Has right icon"
    icon-right="fas fa-chevron-down"
  >
  </sas-input>
  <h3>Has clear icon</h3>
  <sas-input
    label="Has clear icon"
    clear
  >
  </sas-input>
  <h3>Has max length (current max is 10)</h3>
  <sas-input
    label="Has max length 10"
    maxlength="10"
  >
  </sas-input>
  <h3>Has min length (current min is 5, min length only works when validate)</h3>
  <sas-input
    label="Has min length 5"
    minlength="5"
  >
  </sas-input>
  <h3>Disabled</h3>
  <sas-input
    label="Disabled"
    disabled
  >
  </sas-input>
`;
