import { Component, Host, h, Prop, Event, Element, } from '@stencil/core';
import { keydownEvent } from '../../global/global';
/**
 * @slot icon - used for left icon
 * @slot text - used for text
 * @slot iconRight - used for right icon
 */
export class SasListItem {
  constructor() {
    /**
     * Icon left
     */
    this.icon = '';
    /**
     * Value
     */
    this.value = null;
    /**
     * Text
     */
    this.text = '';
    /**
     * Subtext
     */
    this.subtext = '';
    /**
     * Set subtext position
     */
    this.subtextPosition = 'bottom';
    /**
     * Icon Right
     */
    this.iconRight = '';
    /**
     * Used for single selection
     */
    this.selected = false;
    /**
     * Disabled
     */
    this.disabled = false;
    /**
     * Index
     */
    this.index = -1;
    /**
     * Designates the component as anchor and applies the href attribute.
     */
    this.href = '';
    /**
     * Designates the target attribute. This should only be applied when using the href prop.
     */
    this.target = '';
    this.clickEvent = () => {
      const obj = {
        value: this.value,
        text: this.text,
      };
      this.listItemClicked.emit(obj);
      this.listItemClickEventUsedInternal.emit();
    };
  }
  render() {
    return (h(Host, { tabindex: 0 },
      h("div", { class: "container", onKeyDown: keydownEvent, onClick: this.clickEvent },
        Boolean(this.href)
          ? h("a", { href: this.href, target: this.target, class: "anchor" })
          : null,
        h("slot", { name: "icon" }, Boolean(this.icon)
          ? h("sas-icon", { class: "icon-left", icon: this.icon })
          : null),
        h("slot", { name: "text" }, Boolean(this.text)
          ? h("div", { class: "text-container" },
            h("div", { class: "text" }, this.text),
            Boolean(this.subtext)
              ? h("div", { class: "subtext" }, this.subtext)
              : null)
          : null),
        h("slot", { name: "iconRight" }, Boolean(this.iconRight)
          ? h("sas-icon", { class: "icon-right", icon: this.iconRight })
          : null))));
  }
  static get is() { return "sas-list-item"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-list-item.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-list-item.css"]
  }; }
  static get properties() { return {
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon left"
      },
      "attribute": "icon",
      "reflect": true,
      "defaultValue": "''"
    },
    "value": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "ListItemValue",
        "resolved": "boolean | number | string",
        "references": {
          "ListItemValue": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Value"
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "null"
    },
    "text": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Text"
      },
      "attribute": "text",
      "reflect": true,
      "defaultValue": "''"
    },
    "subtext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Subtext"
      },
      "attribute": "subtext",
      "reflect": true,
      "defaultValue": "''"
    },
    "subtextPosition": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'bottom' | 'right'",
        "resolved": "\"bottom\" | \"right\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set subtext position"
      },
      "attribute": "subtext-position",
      "reflect": true,
      "defaultValue": "'bottom'"
    },
    "iconRight": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon Right"
      },
      "attribute": "icon-right",
      "reflect": true,
      "defaultValue": "''"
    },
    "selected": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Used for single selection"
      },
      "attribute": "selected",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "index": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Index"
      },
      "attribute": "index",
      "reflect": true,
      "defaultValue": "-1"
    },
    "href": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Designates the component as anchor and applies the href attribute."
      },
      "attribute": "href",
      "reflect": true,
      "defaultValue": "''"
    },
    "target": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Designates the target attribute. This should only be applied when using the href prop."
      },
      "attribute": "target",
      "reflect": true,
      "defaultValue": "''"
    }
  }; }
  static get events() { return [{
      "method": "listItemClicked",
      "name": "listItemClicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the component is clicked"
      },
      "complexType": {
        "original": "object",
        "resolved": "object",
        "references": {}
      }
    }, {
      "method": "listItemClickEventUsedInternal",
      "name": "listItemClickEventUsedInternal",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the component is clicked (only for components internal use)"
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
}
