import { Component, Element, Host, Listen, Prop, h, } from '@stencil/core';
import ResizeObserver from 'resize-observer-polyfill';
/**
 * @slot bulk-action - used for bulk action content
 * @slot search - used for search content
 * @slot filter - used for filter content
 * @slot button-list - used for button list content
 */
export class SasMenuBar {
  constructor() {
    this.moreFilterButtonWidth = 190;
    /**
     * Sticky
     */
    this.sticky = false;
    this.clickEvent = (event) => {
      event.stopPropagation();
    };
    this.clearFilter = () => {
      // select component
      const filters = this.el.querySelectorAll('[slot="filter"] > sas-select');
      filters.forEach((filter) => {
        filter.value = filter.defaultValue || '';
        filter.update(); // trigger selectUpdate event
      });
      const moreFilters = this.el.querySelectorAll('sas-select[slot="more-filter-slot"]');
      moreFilters.forEach((filter) => {
        filter.value = filter.defaultValue || '';
        filter.update(); // trigger selectUpdate event
      });
      // date field component
      const dateFieldFilters = this.el.querySelectorAll('[slot="filter"] > sas-date-field');
      dateFieldFilters.forEach((filter) => {
        filter.value = '';
        filter.update(); // trigger dateFieldUpdate event
      });
      const moreDateFieldFilters = this.el.querySelectorAll('sas-date-field[slot="more-filter-slot"]');
      moreDateFieldFilters.forEach((filter) => {
        filter.value = '';
        filter.update(); // trigger dateFieldUpdate event
      });
      // multiselect component
      const multiFilters = this.el.querySelectorAll('[slot="filter"] > sas-multiselect');
      multiFilters.forEach((filter) => {
        filter.values = [];
        filter.update(); // trigger selectUpdate event
      });
      const moreMultiFilters = this.el.querySelectorAll('sas-multiselect[slot="more-filter-slot"]');
      moreMultiFilters.forEach((filter) => {
        filter.values = [];
        filter.update(); // trigger selectUpdate event
      });
      //input component
      const inputs = this.el.querySelectorAll('[slot="filter"] > sas-input');
      inputs.forEach((filter) => {
        filter.value = '';
        filter.update(); // trigger selectUpdate event
      });
      const moreInputs = this.el.querySelectorAll('sas-input[slot="more-filter-slot"]');
      moreInputs.forEach((filter) => {
        filter.value = '';
        filter.update(); // trigger selectUpdate event
      });
      //daterange component
      const ranges = this.el.querySelectorAll('[slot="filter"] > sas-date-range');
      ranges.forEach((filter) => {
        filter.from = null;
        filter.to = null;
      });
      const moreranges = this.el.querySelectorAll('sas-date-range[slot="more-filter-slot"]');
      moreranges.forEach((filter) => {
        filter.from = null;
        filter.to = null;
      });
      //composite component
      const comps = this.el.querySelectorAll('[slot="filter"] > composite');
      comps.forEach((filter) => {
        filter.children[1].value = '';
      });
      const morecomps = this.el.querySelectorAll('composite[slot="more-filter-slot"]');
      morecomps.forEach((filter) => {
        filter.children[1].value = '';
      });
    };
  }
  componentDidLoad() {
    const resizeObserver = new ResizeObserver((entries) => {
      entries.forEach((entry) => {
        console.log("resizeObserver: entry");
        console.log(entry);
        this.updateLayout();
      });
    });
    const filterContainer = this.el.shadowRoot.querySelector('.filter');
    resizeObserver.observe(filterContainer);
  }
  selectUpdateEventUsedInternalHandler(event) {
    if (event.target.slot === 'more-filter-slot') {
      this.updateBadgeNumber();
    }
  }
  multiselectUpdateEventUsedInternalHandler(event) {
    if (event.target.slot === 'more-filter-slot') {
      this.updateBadgeNumber();
    }
  }
  dateFieldUpdateEventUsedInternalHandler(event) {
    if (event.target.slot === 'more-filter-slot') {
      this.updateBadgeNumber();
    }
  }
  inputUpdateEventUsedInternalHandler(event) {
    if (event.target.slot === 'more-filter-slot') {
      this.updateBadgeNumber();
    }
  }
  inputinputContainerClickEventInternalHandler(event) {
    console.log("inputinputContainerClickEventInternalHandler");
    console.log(event.target.slot);
  }
  updateLayout() {
    let shouldKeepFocus = false;
    const focusedElement = document.activeElement;
    if (focusedElement.slot === 'more-filter-slot') {
      shouldKeepFocus = true;
    }
    this.resetFilter();
    this.hideOverlapElement();
    this.updateBadgeNumber();
    if (shouldKeepFocus) {
      focusedElement.focus();
    }
  }
  resetFilter() {
    const filters = this.el.querySelectorAll('[slot="more-filter-slot"]');
    const filterContainer = this.el.querySelector('[slot="filter"]');
    filters.forEach((filter) => {
      filter.slot = "";
      filterContainer.append(filter);
    });
  }
  hideOverlapElement() {
    const filters = this.el.querySelectorAll('[slot="filter"] > *');
    let filtersArray = [];
    for (let i = 0; i < filters.length; i += 1) {
      let acceptedTags = ['SAS-SELECT', 'SAS-MULTISELECT', 'SAS-DATE-FIELD', 'SAS-DATE-RANGE', 'SAS-INPUT', 'COMPOSITE'];
      if (acceptedTags.includes(filters[i].tagName)) {
        filtersArray.push(filters[i]);
      }
    }
    const buttonList = this.el.shadowRoot.querySelector('.button-list');
    let startIndex = -1;
    for (let i = 0; i < filtersArray.length; i += 1) {
      let distance = this.moreFilterButtonWidth;
      if (i === filtersArray.length - 1) {
        distance = 0;
      }
      if (this.checkOverlap(filtersArray[i], buttonList, distance)) {
        startIndex = i;
        break;
      }
    }
    const dropdown = this.el.shadowRoot.querySelector('sas-dropdown');
    if (startIndex === -1) {
      dropdown.classList.add('hide');
    }
    else {
      dropdown.classList.remove('hide');
      for (let i = startIndex; i < filtersArray.length; i += 1) {
        filtersArray[i].slot = 'more-filter-slot';
        this.el.append(filtersArray[i]);
      }
    }
  }
  checkOverlap(domLeft, domRight, distance) {
    const rectLeft = domLeft.getBoundingClientRect();
    const rectRight = domRight.getBoundingClientRect();
    const overlap = rectRight.left - rectLeft.right < distance;
    return overlap;
  }
  getFilterNumber() {
    const moreFilters = this.el.querySelectorAll('[slot="more-filter-slot"]');
    if (moreFilters.length > 0) {
      let filterNumber = 0;
      moreFilters.forEach((filter) => {
        if (filter.tagName === 'SAS-SELECT') {
          if (filter.value !== null
            && filter.value !== ''
            && filter.value !== filter.defaultValue) {
            filterNumber += 1;
          }
        }
        else if (filter.tagName === 'SAS-MULTISELECT') {
          if (filter.values.length > 0) {
            filterNumber += 1;
          }
        }
        else if (filter.tagName === 'SAS-DATE-FIELD') {
          if (Boolean(filter.value)) {
            filterNumber += 1;
          }
        }
        else if (filter.tagName === 'SAS-INPUT') {
          if (filter.value !== null
            && filter.value !== '') {
            filterNumber += 1;
          }
        }
        else if (filter.tagName === 'SAS-DATE-RANGE') {
          let from = filter.from;
          let to = filter.to;
          if (((from != null && from != '') || (to != null && to != ''))) {
            filterNumber += 1;
          }
        }
        else if (filter.tagName === 'COMPOSITE') {
          let val = filter.children[1].value;
          console.log('comp', val);
          if (val != null && val != '') {
            filterNumber += 1;
          }
        }
      });
      return filterNumber;
    }
  }
  updateBadgeNumber() {
    const badge = this.el.shadowRoot.querySelector('sas-badge');
    badge.number = this.getFilterNumber();
  }
  render() {
    return (h(Host, { onClick: this.clickEvent },
      h("div", { class: "menu-bar-part bulk-action" },
        h("slot", { name: "bulk-action" })),
      h("div", { class: "menu-bar-part search" },
        h("slot", { name: "search" })),
      h("div", { class: "menu-bar-part filter" },
        h("slot", { name: "filter" }),
        h("sas-dropdown", { class: "more-filter-button", positionH: "left", clickClose: false },
          h("sas-button", { slot: "control", type: "text", icon: "far fa-sliders-h", text: "More Filters" },
            h("sas-badge", { type: "notification", inline: true })),
          h("div", { class: "menu", slot: "menu" },
            h("div", { class: "more-filter" },
              h("slot", { name: "more-filter-slot" })),
            h("div", { class: "filter-control" },
              h("sas-button", { type: "secondary", text: "Clear Filters", block: true, onClick: this.clearFilter }))))),
      h("div", { class: "menu-bar-part button-list" },
        h("slot", { name: "button-list" }))));
  }
  static get is() { return "sas-menu-bar"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-menu-bar.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-menu-bar.css"]
  }; }
  static get properties() { return {
    "sticky": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Sticky"
      },
      "attribute": "sticky",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get elementRef() { return "el"; }
  static get listeners() { return [{
      "name": "selectUpdateEventUsedInternal",
      "method": "selectUpdateEventUsedInternalHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "multiselectUpdateEventUsedInternal",
      "method": "multiselectUpdateEventUsedInternalHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "dateFieldUpdateEventUsedInternal",
      "method": "dateFieldUpdateEventUsedInternalHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "inputupdate",
      "method": "inputUpdateEventUsedInternalHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "inputContainerClick",
      "method": "inputinputContainerClickEventInternalHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
