import { 
// eslint-disable-next-line no-unused-vars
Component, Host, Prop, h, Element, Listen, Watch, Event, State, Method, } from '@stencil/core';
import { HTMLDecodeString } from '../../global/global';
export class SasMultiselect {
  constructor() {
    this.checkedChipNumber = 1;
    this.formatOptions = [];
    /**
     * Chip object array
     */
    this.chipArray = [];
    /**
     * Type
     */
    this.type = 'chip';
    /**
     * Search, only works with type chip
     */
    this.search = false;
    /**
     * Filter Text, value of current filter term
     */
    this.filterText = '';
    /**
     * No match action
     */
    this.noMatchAction = 'show_no_match_tip';
    /**
     * No match tip
     */
    this.noMatchTip = 'No data available';
    /**
     * Label
     */
    this.label = '';
    /**
     * Label position
     */
    this.outside = false;
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * disabled
     */
    this.disabled = false;
    /**
     * Clear
     */
    this.clear = true;
    /**
     * Placeholder
     */
    this.placeholder = 'Please select';
    /**
     * Options
     */
    this.options = [];
    /**
     * name
     */
    this.name = '';
    /**
     * Selected values
     */
    this.values = [];
    /**
     * Complete selected data
     */
    this.rawData = [];
    /**
     * Minimum selected values, not use this prop when has sub group
     */
    this.min = 0;
    /**
     * Maximum selected values, not use this prop when has sub group
     */
    this.max = 0;
    /**
     * Maximum chips number, only works in type chip
     */
    this.maxChip = 1;
    /**
     * Show select all checkbox
     */
    this.selectAll = false;
    /**
     * Show select all checkbox text
     */
    this.selectAllText = 'All';
    /**
     * Default container position
     */
    this.position = null;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
      rangeUnderflow: false,
      rangeOverflow: false,
    };
    /**
     * Only shows when input is invalid, auto generated
     */
    this.message = '';
    /**
     * Input value
     */
    this.value = null; // use state decorator to trigger render in componentDidLoad function if has preselected values
    this.inputEvent = (event) => {
      this.getClearText();
      this.filterText = event.target.value;
      this.filter();
      const formInputTemplate = this.el.shadowRoot.querySelector('sas-form-input-template');
      formInputTemplate.show = true;
    };
    this.addNewOptionClickEvent = () => {
      // Add new option
      const newOption = {
        value: this.filterText,
        text: this.filterText,
        disabled: false,
        exclusive: false,
        addedByNoMatchAddNewOptionButton: true,
      };
      this.options.push(newOption);
      // Select new option
      this.values.push(this.filterText);
      // Clear filter input
      const filterInput = this.el.shadowRoot.querySelector(".filter");
      filterInput.value = "";
      this.filterText = "";
      // Re-render component
      this.filter();
      this.updateStatus();
      this.update();
    };
    this.chipMouseDownEvent = (event) => {
      // not trigger container show / hide
      event.stopPropagation();
    };
    this.clearAll = () => {
      if (this.getClearText() === "Clear") {
        this.clearFilterText();
      }
      else {
        this.clearSelection();
      }
      this.getClearText();
    };
    this.getNoMatchTipTextContent = () => {
      if (this.noMatchAction === "show_no_match_add_new_option") {
        return this.filterText;
      }
      else if (this.noMatchAction === "show_no_match_tip") {
        return this.noMatchTip;
      }
    };
  }
  watchValuesHandler() {
    this.updateStatus();
    this.multiselectUpdateEventUsedInternal.emit(this.rawData);
  }
  watchOptionsHandler() {
    this.updateStatus();
  }
  componentWillLoad() {
    this.convertLightDomToShadowDom();
    this.updateStatus();
  }
  checkboxUpdateHandler(event) {
    this.chipArray = this.getChipArray(event.detail);
    const checkboxGroup = this.getCheckboxGroup();
    this.values = [...checkboxGroup.values];
    this.value = this.getInputValue();
    this.multiselectUpdate.emit(this.rawData);
    this.multiselectupdate.emit(this.rawData);
  }
  chipUpdateHandler(event) {
    const clickedChipObj = this.chipArray.find(item => item.text === event.detail);
    const checkboxGroup = this.getCheckboxGroup();
    const checkboxes = checkboxGroup.shadowRoot.querySelectorAll('sas-checkbox');
    if (typeof clickedChipObj.value === 'object') { // all subgroup selected
      checkboxes.forEach(checkbox => {
        if (clickedChipObj.value.includes(checkbox.value)) {
          checkbox.click();
        }
      });
    }
    else {
      checkboxes.forEach(checkbox => {
        if (clickedChipObj.value === checkbox.value) {
          checkbox.click();
        }
      });
    }
    const inputContainer = this.el.shadowRoot.querySelector('.input-container');
    inputContainer.focus();
  }
  inputContainerClickHandler() {
    if (this.values.length > 0) {
      const firstCheckedElement = this.getFirstCheckedCheckbox();
      firstCheckedElement.parentElement.scrollTop = firstCheckedElement.offsetTop;
    }
  }
  /**
   * Trigger multiselectupdate event programmatically
   */
  async update() {
    this.multiselectUpdate.emit(this.rawData);
    this.multiselectupdate.emit(this.rawData);
  }
  /**
   * Validate
   */
  async validate() {
    const length = this.values.length;
    let valueMissing = false;
    if (this.required && length === 0) {
      valueMissing = true;
    }
    let rangeUnderflow = false;
    if (this.min !== 0 && length < this.min) {
      rangeUnderflow = true;
    }
    let rangeOverflow = false;
    if (this.max !== 0 && length > this.max) {
      rangeOverflow = true;
    }
    this.validity = {
      valid: !valueMissing && !rangeUnderflow && !rangeOverflow,
      valueMissing: valueMissing,
      rangeUnderflow: rangeUnderflow,
      rangeOverflow: rangeOverflow,
    };
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    await this.validate();
    const formInputTemplate = this.el.shadowRoot.querySelector('sas-form-input-template');
    if (formInputTemplate !== null) {
      formInputTemplate.error = !this.validity.valid;
    }
    if (!this.validity.valid) {
      this.message = this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    const formInputTemplate = this.el.shadowRoot.querySelector('sas-form-input-template');
    if (formInputTemplate !== null) {
      formInputTemplate.error = false;
    }
    this.message = '';
  }
  convertLightDomToShadowDom() {
    const checkboxes = this.el.querySelectorAll('sas-checkbox');
    if (checkboxes && checkboxes.length > 0) {
      let lightDomValues = [];
      let lightDomOptions = [];
      checkboxes.forEach((checkbox) => {
        const option = {
          value: checkbox.value,
          text: checkbox.text,
          disabled: checkbox.disabled,
          exclusive: checkbox.exclusive,
          addedByNoMatchAddNewOptionButton: false,
        };
        if (checkbox.checked) {
          lightDomValues.push(checkbox.value);
        }
        lightDomOptions.push(option);
      });
      this.el.innerHTML = '';
      if (this.values.length === 0) {
        this.values = lightDomValues;
      }
      this.options = lightDomOptions;
    }
  }
  updateStatus() {
    this.formatOptions = this.getFormatOptions(this.options);
    let validValues = this.getValidValues(this.values, this.formatOptions);
    if (!this.compareArray(this.values, validValues)) {
      this.values = [...validValues];
    }
    this.rawData = this.getRawData(this.values, this.formatOptions);
    this.chipArray = this.getChipArray(this.rawData);
    this.value = this.getInputValue();
    this.resetErrorStatus();
  }
  getFormatOptions(options) {
    const formatOptions = options.map(option => {
      if ('sub' in option) {
        const formatSubOptions = option.sub.map((subOption) => {
          return {
            value: subOption.value,
            text: HTMLDecodeString(subOption.text),
            disabled: subOption.disabled ? true : false,
            exclusive: subOption.exclusive ? true : false,
            addedByNoMatchAddNewOptionButton: subOption.addedByNoMatchAddNewOptionButton ? true : false,
          };
        });
        return {
          value: option.value,
          text: HTMLDecodeString(option.text),
          disabled: option.disabled ? true : false,
          addedByNoMatchAddNewOptionButton: option.addedByNoMatchAddNewOptionButton ? true : false,
          sub: formatSubOptions,
        };
      }
      else {
        return {
          value: option.value,
          text: HTMLDecodeString(option.text),
          disabled: option.disabled ? true : false,
          exclusive: option.exclusive ? true : false,
          addedByNoMatchAddNewOptionButton: option.addedByNoMatchAddNewOptionButton ? true : false,
        };
      }
    });
    return formatOptions;
  }
  getValidValues(values, options) {
    const existValues = this.getExistValues(values, options);
    const validValuesByCheckExclusive = this.getValidValuesByCheckExclusive(existValues, options);
    const validValues = this.getValidValuesByCheckMax(validValuesByCheckExclusive, this.max);
    return validValues;
  }
  getExistValues(values, options) {
    let validValues = [];
    if (options.length === 0) {
      validValues = [...values];
    }
    else {
      for (let i = 0; i < values.length; i++) {
        for (let j = 0; j < options.length; j++) {
          if (options[j].sub) {
            for (let m = 0; m < options[j].sub.length; m++) {
              if (options[j].sub[m].value === values[i]) {
                validValues.push(values[i]);
              }
            }
          }
          else {
            if (options[j].value === values[i]) {
              validValues.push(values[i]);
            }
          }
        }
      }
    }
    return validValues;
  }
  getValidValuesByCheckExclusive(values, options) {
    const exclusiveValue = this.getExclusiveValue(values, options);
    if (exclusiveValue !== null) {
      return [exclusiveValue];
    }
    else {
      return [...values];
    }
  }
  getExclusiveValue(values, options) {
    let exclusiveValue = null;
    outer: for (let i = 0; i < values.length; i++) {
      for (let j = 0; j < options.length; j++) {
        if (options[j].sub) {
          for (let m = 0; m < options[j].sub.length; m++) {
            if (options[j].sub[m].value === values[i]
              && options[j].sub[m].exclusive) {
              exclusiveValue = values[i];
              break outer;
            }
          }
        }
        else {
          if (options[j].value === values[i]
            && options[j].exclusive) {
            exclusiveValue = values[i];
            break outer;
          }
        }
      }
    }
    return exclusiveValue;
  }
  getValidValuesByCheckMax(values, max) {
    if (max && values.length >= max) {
      return values.slice(0, max);
    }
    else {
      return [...values];
    }
  }
  compareArray(arrayOne, arrayTwo) {
    return (arrayOne.length === arrayTwo.length
      && arrayOne.every((element, index) => element === arrayTwo[index]));
  }
  getRawData(values, options) {
    let rawData = [];
    if (values.length > 0) {
      values.forEach((value) => {
        options.forEach((option) => {
          if (option.sub && option.sub.length > 0) {
            option.sub.forEach((sub) => {
              if (sub.value === value) {
                const item = rawData.find(item => item.value === option.value);
                if (item) {
                  item.sub.push(sub);
                }
                else {
                  rawData.push({
                    value: option.value,
                    text: option.text,
                    addedByNoMatchAddNewOptionButton: option.addedByNoMatchAddNewOptionButton,
                    sub: [sub],
                  });
                }
              }
            });
          }
          else {
            if (option.value === value) {
              rawData.push(option);
            }
          }
        });
      });
    }
    return rawData;
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      return `${this.label} is required`;
    }
    if (validity.rangeOverflow) {
      return `Please select at most ${this.max} options`;
    }
    if (validity.rangeUnderflow) {
      return `Please select at least ${this.min} options`;
    }
  }
  checkShadowDom() {
    if (this.options.length > 0) {
      return true;
    }
    return false;
  }
  getCheckboxGroup() {
    if (this.formatOptions.length > 0) {
      return this.el.shadowRoot.querySelector('sas-checkbox-group');
    }
    return this.el.querySelector('sas-checkbox-group');
  }
  getInputValue() {
    if (this.values.length === 0) {
      return '';
    }
    else {
      if (this.type === 'text') {
        if (this.values.length === 1) {
          return this.chipArray[0].text;
        }
        else {
          return `${this.values.length} selected`;
        }
      }
      else {
        let newValue = ' ';
        let i = 0;
        while (i < this.values.length) {
          newValue += ' ';
          i += 1;
        }
        return newValue; // trigger re-render
      }
    }
  }
  getChipArray(formatValues) {
    let chipArray = [];
    if (formatValues.length > 0) {
      formatValues.forEach((formatValue) => {
        let chipObj;
        if (formatValue.sub && formatValue.sub.length > 0) {
          const option = this.formatOptions.find(option => option.value === formatValue.value);
          if (formatValue.sub.length === option.sub.length) { // all subgroup selected
            const valueArray = [];
            formatValue.sub.forEach((subObj) => {
              valueArray.push(subObj.value);
            });
            let chipText = formatValue.text;
            if (chipText !== 'All' && chipText.slice(0, 4) !== 'All ') {
              chipText = 'All' + chipText;
            }
            chipObj = {
              value: valueArray,
              text: chipText,
            };
            chipArray.push(chipObj);
          }
          else {
            formatValue.sub.forEach((subObj) => {
              chipObj = {
                value: subObj.value,
                text: subObj.text,
              };
              chipArray.push(chipObj);
            });
          }
        }
        else {
          chipArray.push(formatValue);
        }
      });
    }
    if (chipArray.length > 0) {
      this.checkedChipNumber = chipArray.length;
      if (this.checkedChipNumber > this.maxChip) {
        this.checkedChipNumber = this.maxChip;
      }
    }
    else {
      this.checkedChipNumber = 1;
    }
    return chipArray;
  }
  filter() {
    let checkboxes;
    let checkboxSubgroups;
    let count = 0;
    const hasShadowDom = this.checkShadowDom();
    if (hasShadowDom) {
      const checkboxGroup = this.getCheckboxGroup();
      if (checkboxGroup !== null) {
        checkboxes = checkboxGroup.shadowRoot.querySelectorAll('sas-checkbox');
        checkboxSubgroups = checkboxGroup.shadowRoot.querySelectorAll('sas-checkbox-subgroup');
      }
      else {
        checkboxes = [];
        checkboxSubgroups = [];
      }
    }
    else {
      checkboxes = this.el.querySelectorAll('sas-checkbox');
      checkboxSubgroups = this.el.querySelectorAll('sas-checkbox-subgroup');
    }
    checkboxes.forEach(checkbox => {
      let textLowerCase;
      if (this.filterText) {
        textLowerCase = this.filterText.toLowerCase();
      }
      else {
        textLowerCase = '';
      }
      const checkboxTextLowserCase = checkbox.text.toLowerCase();
      let isMatch = false;
      if (checkboxTextLowserCase) {
        isMatch = checkboxTextLowserCase.includes(textLowerCase);
      }
      if (isMatch) {
        count += 1;
        if (hasShadowDom) {
          checkbox.classList.remove('hide');
        }
        else {
          checkbox.classList.remove('d-none');
        }
      }
      else {
        if (hasShadowDom) {
          checkbox.classList.add('hide');
        }
        else {
          checkbox.classList.add('d-none');
        }
      }
      const noMatchTip = this.el.shadowRoot.querySelector('#no-match-tip');
      if (count !== 0) {
        noMatchTip.classList.add('hide');
      }
      else {
        noMatchTip.classList.remove('hide');
      }
    });
    // hide sub group if need
    checkboxSubgroups.forEach(checkboxSubgroup => {
      const children = checkboxSubgroup.querySelectorAll('sas-checkbox');
      let isAllHide = true;
      for (let i = 0; i < children.length; i++) {
        if (!children[i].classList.contains('hide') && !children[i].classList.contains('d-none')) {
          isAllHide = false;
          break;
        }
      }
      if (isAllHide) {
        checkboxSubgroup.classList.add('hide');
      }
      else {
        checkboxSubgroup.classList.remove('hide');
      }
    });
  }
  getChipElement() {
    let chipElements = [];
    for (let i = 0; i < this.maxChip; i += 1) {
      const chipElement = h("sas-chip", { style: { maxWidth: this.getChipWidth() }, class: this.getChipClass(this.chipArray.length > i), close: true, text: this.getChipText(i), uppercase: false, bold: false, onMouseDown: this.chipMouseDownEvent });
      chipElements.push(chipElement);
    }
    return chipElements;
  }
  getChipWidth() {
    const width = `calc(${100 / this.checkedChipNumber}% - ${70 / this.checkedChipNumber}px - 10px)`;
    return width;
  }
  getChipClass(bool) {
    if (bool) {
      return '';
    }
    return 'hide';
  }
  getChipText(index) {
    if (this.chipArray[index]) {
      return this.chipArray[index].text;
    }
    return '';
  }
  getFirstCheckedCheckbox() {
    return this.el.shadowRoot.querySelector('sas-form-input-template').querySelector('sas-checkbox-group').shadowRoot.querySelector('sas-checkbox[checked]');
  }
  clearFilterText() {
    const filterInput = this.el.shadowRoot.querySelector(".filter");
    filterInput.value = "";
    this.filterText = "";
    this.filter();
  }
  clearSelection() {
    this.values = [];
    this.update();
  }
  getClearText() {
    const inputFilter = this.el.shadowRoot.querySelector('#inputFilter');
    if (inputFilter === null) {
      if (this.chipArray.length === 0) {
        this.buttonFilterText = 'Clear';
      }
      else {
        this.buttonFilterText = 'Clear Selection';
      }
    }
    else if (inputFilter.value.length === 0) {
      if (this.chipArray.length === 0) {
        this.buttonFilterText = 'Clear';
      }
      else {
        this.buttonFilterText = 'Clear Selection';
      }
    }
    else {
      this.buttonFilterText = 'Clear';
    }
    return this.buttonFilterText;
  }
  render() {
    return (h(Host, { tabindex: "0" },
      h("sas-form-input-template", { label: this.label, outside: this.outside, placeholder: this.placeholder, "icon-right": "far fa-chevron-down", required: this.required, readonly: true, value: this.value, disabled: this.disabled, "data-message": this.message, position: this.position, minimumPixelsAboveComponentRequired: 250 },
        this.type === 'chip'
          ? h("div", { class: "input-container", tabindex: -1, slot: "input" },
            this.getChipElement(),
            h("sas-chip", { class: this.getChipClass(this.chipArray.length > this.maxChip), id: "text-chip", text: `+${this.chipArray.length - this.maxChip}`, uppercase: false, bold: false, onMouseDown: this.chipMouseDownEvent }))
          : null,
        this.clear
          ? h("div", { slot: "container", class: "clear-button-container" },
            h("button", { class: "clear-button", onClick: this.clearAll }, this.getClearText()))
          : null,
        this.search
          ? h("input", { id: "inputFilter", slot: "container", class: "filter", placeholder: "filter", onInput: this.inputEvent })
          : null,
        this.formatOptions.length > 0
          ? h("sas-checkbox-group", { slot: "container", max: this.max, options: this.formatOptions, values: this.values, selectAll: this.selectAll, selectAllText: this.selectAllText })
          : h("slot", { slot: "container" }),
        h("div", { slot: "container", class: "hide", id: "no-match-tip" },
          this.getNoMatchTipTextContent(),
          this.noMatchAction === 'show_no_match_add_new_option'
            ? h("p", { onClick: this.addNewOptionClickEvent, class: "no-match-add-new-option-button" }, "Add")
            : null))));
  }
  static get is() { return "sas-multiselect"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-multiselect.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-multiselect.css"]
  }; }
  static get properties() { return {
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'text' | 'chip'",
        "resolved": "\"chip\" | \"text\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Type"
      },
      "attribute": "type",
      "reflect": true,
      "defaultValue": "'chip'"
    },
    "search": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Search, only works with type chip"
      },
      "attribute": "search",
      "reflect": true,
      "defaultValue": "false"
    },
    "filterText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Filter Text, value of current filter term"
      },
      "attribute": "filter-text",
      "reflect": true,
      "defaultValue": "''"
    },
    "noMatchAction": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'show_no_match_tip' | 'show_no_match_add_new_option'",
        "resolved": "\"show_no_match_add_new_option\" | \"show_no_match_tip\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "No match action"
      },
      "attribute": "no-match-action",
      "reflect": true,
      "defaultValue": "'show_no_match_tip'"
    },
    "noMatchTip": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "No match tip"
      },
      "attribute": "no-match-tip",
      "reflect": true,
      "defaultValue": "'No data available'"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label"
      },
      "attribute": "label",
      "reflect": true,
      "defaultValue": "''"
    },
    "outside": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label position"
      },
      "attribute": "outside",
      "reflect": true,
      "defaultValue": "false"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Shows a star symbol behind label"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "clear": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Clear"
      },
      "attribute": "clear",
      "reflect": true,
      "defaultValue": "true"
    },
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Placeholder"
      },
      "attribute": "placeholder",
      "reflect": true,
      "defaultValue": "'Please select'"
    },
    "options": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "MultiSelectOption[]",
        "resolved": "MultiSelectOption[]",
        "references": {
          "MultiSelectOption": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Options"
      },
      "defaultValue": "[]"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "''"
    },
    "values": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "CheckboxValue[]",
        "resolved": "CheckboxValue[]",
        "references": {
          "CheckboxValue": {
            "location": "import",
            "path": "../sas-checkbox/type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Selected values"
      },
      "defaultValue": "[]"
    },
    "rawData": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "any[]",
        "resolved": "any[]",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Complete selected data"
      },
      "defaultValue": "[]"
    },
    "min": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Minimum selected values, not use this prop when has sub group"
      },
      "attribute": "min",
      "reflect": true,
      "defaultValue": "0"
    },
    "max": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Maximum selected values, not use this prop when has sub group"
      },
      "attribute": "max",
      "reflect": true,
      "defaultValue": "0"
    },
    "maxChip": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Maximum chips number, only works in type chip"
      },
      "attribute": "max-chip",
      "reflect": true,
      "defaultValue": "1"
    },
    "selectAll": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show select all checkbox"
      },
      "attribute": "select-all",
      "reflect": false,
      "defaultValue": "false"
    },
    "selectAllText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show select all checkbox text"
      },
      "attribute": "select-all-text",
      "reflect": false,
      "defaultValue": "'All'"
    },
    "position": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'top' | 'bottom' | null",
        "resolved": "\"bottom\" | \"top\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Default container position"
      },
      "attribute": "position",
      "reflect": true,
      "defaultValue": "null"
    },
    "validity": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "{ valid: boolean; valueMissing: boolean; rangeUnderflow: boolean; rangeOverflow: boolean; }",
        "resolved": "{ valid: boolean; valueMissing: boolean; rangeUnderflow: boolean; rangeOverflow: boolean; }",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Validity"
      },
      "defaultValue": "{\n    valid: true,\n    valueMissing: false,\n    rangeUnderflow: false,\n    rangeOverflow: false,\n  }"
    },
    "message": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Only shows when input is invalid, auto generated"
      },
      "attribute": "message",
      "reflect": true,
      "defaultValue": "''"
    },
    "buttonFilterText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'Clear' | 'Clear Selection'",
        "resolved": "\"Clear Selection\" | \"Clear\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Property used to set the text for the button filter, can be Clear or Clear Selection"
      },
      "attribute": "button-filter-text",
      "reflect": false
    }
  }; }
  static get states() { return {
    "value": {}
  }; }
  static get events() { return [{
      "method": "multiselectUpdate",
      "name": "multiselectUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the values are changed by user interaction (will be deprecated)"
      },
      "complexType": {
        "original": "Array<object>",
        "resolved": "object[]",
        "references": {
          "Array": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "multiselectupdate",
      "name": "multiselectupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the values are changed by user interaction"
      },
      "complexType": {
        "original": "Array<object>",
        "resolved": "object[]",
        "references": {
          "Array": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "multiselectUpdateEventUsedInternal",
      "name": "multiselectUpdateEventUsedInternal",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the values are changed (only for components internal use)"
      },
      "complexType": {
        "original": "Array<object>",
        "resolved": "object[]",
        "references": {
          "Array": {
            "location": "global"
          }
        }
      }
    }]; }
  static get methods() { return {
    "update": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Trigger multiselectupdate event programmatically",
        "tags": []
      }
    },
    "validate": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Validate",
        "tags": []
      }
    },
    "updateErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show error status if value is not valid (only for components internal use)",
        "tags": []
      }
    },
    "resetErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clear error status (only for components internal use)",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "values",
      "methodName": "watchValuesHandler"
    }, {
      "propName": "options",
      "methodName": "watchOptionsHandler"
    }]; }
  static get listeners() { return [{
      "name": "checkboxgroupupdate",
      "method": "checkboxUpdateHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "chipUpdate",
      "method": "chipUpdateHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "inputContainerClick",
      "method": "inputContainerClickHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
