// sas-multiselect.stories.js
import readme from './readme.md';

export default {
  title: 'Higher Order/Multiselect',
  parameters: {
    notes: {
      markdown: readme,
    },
    actions: {
      handles: ['multiselectupdate', 'sas-multiselect'],
    },
  },
};

const typeOptions = ['chip', 'text'];
const positionOptions = ['top', 'bottom', null];
const noMatchActionOptions = ['show_no_match_tip', 'show_no_match_add_new_option'];

export const Basic = (args) => `
  <sas-multiselect
    type="${args.type}"  
    search="${args.search}"
    no-match-action="${args.noMatchAction}"
    no-match-tip="${args.noMatchTip}"
    label="${args.label}"
    outside="${args.outside}"
    placeholder="${args.placeholder}"
    max="${args.max}"
    max-chip="${args.maxChip}"
    select-all="${args.selectAll}"
    select-all-text="${args.selectAllText}"
    disabled="${args.disabled}"
    clear="${args.clear}"
    required="${args.required}"
    position="${args.position}"
  >
    <sas-checkbox-group select-all>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>
`;

Basic.args = {
  type: 'chip',
  search: false,
  noMatchAction: 'show_no_match_tip',
  noMatchTip: 'No data available',
  label: 'Label',
  outside: false,
  placeholder: 'Please select',
  max: 6,
  maxChip: 3,
  selectAll: false,
  selectAllText: 'All',
  disabled: false,
  clear: false,
  required: false,
  position: null,
};

Basic.argTypes = {
  type: {
    control: {
      type: 'select',
      options: typeOptions,
    },
  },
  position: {
    control: {
      type: 'select',
      options: positionOptions,
    },
  },
  noMatchAction: {
    control: {
      type: 'select',
      options: noMatchActionOptions,
    },
  },
};

export const Type = () => `
  <h3>Type chip</h3>  
  <sas-multiselect type="chip">
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>

  <h3>Type text</h3>  
  <sas-multiselect type="text">
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>
  <h3>Search</h3>  
  <sas-multiselect search>
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>
`;

export const Status = () => `
  <h3>Default</h3>  
  <sas-multiselect label="label">
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>

  <h3>Label outside</h3>  
  <sas-multiselect label="label outside" outside>
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>

  <h3>Required</h3>  
  <sas-multiselect label="required" required>
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>

  <h3>Has max selections (currently max is 3)</h3>  
  <sas-multiselect label="Has max selections" max="3">
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>

  <h3>Set max displayed chip (currently max is 3)</h3>  
  <sas-multiselect label="Set max displayed chip" max-chip="3">
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>

  <h3>Search</h3>  
  <sas-multiselect search>
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>

  <h3>Customize no search result text</h3>  
  <sas-multiselect search no-match-tip="customized no search result">
    <sas-checkbox-group>
      <sas-checkbox value="apple1" text="Apple1111111111111111111111111111111111"></sas-checkbox>
      <sas-checkbox value="apple2" text="Apple2"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>

  <h3>Show "Add" option button when no search result</h3>  
  <p>Use the filter input to search for a string that doesn't appear in the dropdown options, The ability to add a new option will be displayed</p>
  <sas-multiselect search no-match-action="show_no_match_add_new_option">
    <sas-checkbox-group>
      <sas-checkbox value="apple" text="Apple"></sas-checkbox>
      <sas-checkbox value="banana" text="Banana"></sas-checkbox>
      <sas-checkbox value="orange" text="Orange"></sas-checkbox>
    </sas-checkbox-group>
  </sas-multiselect>
`;
