import { Component, Element, Prop, Watch, Host, h, Event, } from '@stencil/core';
export class SasPagination {
  constructor() {
    this.validTotalIndex = this.getValidTotalIndex();
    /**
     * Total Pages
     */
    this.totalPage = 100;
    /**
     * Total Visible Indexes, odd number only
     */
    this.totalIndex = 9;
    /**
     * Current Page Index
     */
    this.currentPage = 0;
    this.updateCurrentPage = (event) => {
      this.currentPage = event.target.dataset.index;
    };
    this.goToPrevPage = () => {
      this.currentPage -= 1;
    };
    this.goToNextPage = () => {
      this.currentPage += 1;
    };
  }
  watchHandler(newValue) {
    this.addActiveClass(newValue);
    this.updateControlButtonStatus(newValue);
    this.paginationUpdate.emit(newValue);
    this.paginationupdate.emit(newValue);
  }
  componentDidLoad() {
    this.addActiveClass(this.currentPage);
    this.updateControlButtonStatus(this.currentPage);
  }
  getValidTotalIndex() {
    if (this.totalIndex % 2 === 0) {
      return this.totalIndex + 1;
    }
    return this.totalIndex;
  }
  updateControlButtonStatus(currentPage) {
    const prevButton = this.el.shadowRoot.querySelector('#prev-icon');
    const nextButton = this.el.shadowRoot.querySelector('#next-icon');
    prevButton.disabled = false;
    nextButton.disabled = false;
    if (currentPage === 0) {
      prevButton.disabled = true;
    }
    if (currentPage === this.totalPage - 1) {
      nextButton.disabled = true;
    }
  }
  generatePageIndexButtons() {
    const pageIndexButtons = [];
    let listItem;
    if (this.validTotalIndex >= this.totalPage) {
      for (let i = 0; i < this.totalPage; i += 1) {
        listItem = h("li", null,
          h("sas-button", { type: "text", text: `${i + 1}`, "data-index": i, onClick: this.updateCurrentPage }));
        pageIndexButtons.push(listItem);
      }
    }
    else {
      const divider = Math.floor(this.validTotalIndex / 2);
      if (this.currentPage < divider) { // show right ellipsis
        for (let i = 0; i < this.validTotalIndex; i += 1) {
          listItem = h("li", null,
            h("sas-button", { type: "text", text: `${i + 1}`, "data-index": i, onClick: this.updateCurrentPage }));
          if (i === this.validTotalIndex - 2) {
            listItem = h("li", null,
              h("sas-button", { type: "text", text: '...', disabled: true }));
          }
          else if (i === this.validTotalIndex - 1) {
            listItem = h("li", null,
              h("sas-button", { type: "text", text: `${this.totalPage}`, "data-index": this.totalPage - 1, onClick: this.updateCurrentPage }));
          }
          pageIndexButtons.push(listItem);
        }
      }
      else if (this.currentPage < this.totalPage - divider) { // show left and right ellipsis
        for (let i = 0; i < this.validTotalIndex; i += 1) {
          listItem = h("li", null,
            h("sas-button", { type: "text", text: `${this.currentPage - divider + i + 1}`, "data-index": this.currentPage - divider + i, onClick: this.updateCurrentPage }));
          if (i === 0) {
            listItem = h("li", null,
              h("sas-button", { type: "text", text: '1', "data-index": 0, onClick: this.updateCurrentPage }));
          }
          else if (i === 1) {
            listItem = h("li", null,
              h("sas-button", { type: "text", text: '...', disabled: true }));
          }
          else if (i === this.validTotalIndex - 2) {
            listItem = h("li", null,
              h("sas-button", { type: "text", text: '...', disabled: true }));
          }
          else if (i === this.validTotalIndex - 1) {
            listItem = h("li", null,
              h("sas-button", { type: "text", text: `${this.totalPage}`, "data-index": this.totalPage - 1, onClick: this.updateCurrentPage }));
          }
          pageIndexButtons.push(listItem);
        }
      }
      else if (this.currentPage < this.totalPage) { // show left ellipsis
        for (let i = 0; i < this.validTotalIndex; i += 1) {
          listItem = h("li", null,
            h("sas-button", { type: "text", text: `${this.totalPage - this.validTotalIndex + i + 1}`, "data-index": this.totalPage - this.validTotalIndex + i, onClick: this.updateCurrentPage }));
          if (i === 0) {
            listItem = h("li", null,
              h("sas-button", { type: "text", text: '1', "data-index": 0, onClick: this.updateCurrentPage }));
          }
          else if (i === 1) {
            listItem = h("li", null,
              h("sas-button", { type: "text", text: '...', disabled: true }));
          }
          pageIndexButtons.push(listItem);
        }
      }
    }
    return pageIndexButtons;
  }
  addActiveClass(index) {
    this.removeActiveClass();
    const clickedButton = this.getButton(index);
    if (Boolean(clickedButton)) {
      clickedButton.classList.add('pagination-active');
    }
  }
  removeActiveClass() {
    const buttons = this.el.shadowRoot.querySelectorAll('sas-button');
    buttons.forEach((item) => {
      item.classList.remove('pagination-active');
    });
  }
  getButton(index) {
    const divider = Math.floor(this.validTotalIndex / 2);
    // in two ellipsis condition, force to select the middle button
    if (this.currentPage >= divider && this.currentPage < this.totalPage - divider) {
      const buttons = this.el.shadowRoot.querySelectorAll('sas-button');
      return buttons[divider + 1];
    }
    return this.el.shadowRoot.querySelector(`sas-button[data-index='${index}']`);
  }
  render() {
    return (h(Host, null,
      h("ul", null,
        h("li", null,
          h("sas-button", { type: "text", id: "prev-icon", text: "", icon: "fas fa-chevron-left", onClick: this.goToPrevPage })),
        this.generatePageIndexButtons(),
        h("li", null,
          h("sas-button", { type: "text", id: "next-icon", text: "", icon: "fas fa-chevron-right", onClick: this.goToNextPage })))));
  }
  static get is() { return "sas-pagination"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-pagination.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-pagination.css"]
  }; }
  static get properties() { return {
    "totalPage": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Total Pages"
      },
      "attribute": "total-page",
      "reflect": true,
      "defaultValue": "100"
    },
    "totalIndex": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Total Visible Indexes, odd number only"
      },
      "attribute": "total-index",
      "reflect": true,
      "defaultValue": "9"
    },
    "currentPage": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Current Page Index"
      },
      "attribute": "current-page",
      "reflect": true,
      "defaultValue": "0"
    }
  }; }
  static get events() { return [{
      "method": "paginationUpdate",
      "name": "paginationUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the page is changed by user interaction (will be deprecated)"
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "paginationupdate",
      "name": "paginationupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the page is changed by user interaction"
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "currentPage",
      "methodName": "watchHandler"
    }]; }
}
