import { Component, Host, Prop, h } from '@stencil/core';
import { ValidateProgressGatesType } from './type';
export class SasProgressBar {
  constructor() {
    /**
     * Progress
     */
    this.progress = 50;
    /**
     * Progress Gates
     */
    this.progressGates = [
      {
        max: 34,
        color: "#D03915"
      },
      {
        max: 66,
        color: "#EBA115"
      },
      {
        max: 100,
        color: "#79D51E"
      },
    ];
  }
  getCurrentProgressGate() {
    const sortedProgressGates = this.progressGates.sort((a, b) => (a.max > b.max) ? 1 : -1);
    return sortedProgressGates.find(progressGate => {
      return progressGate.max >= this.getValidProgressNumber();
    });
  }
  getValidProgressNumber() {
    if (this.progress < 0) {
      return 0;
    }
    if (this.progress > 100) {
      return 100;
    }
    return this.progress;
  }
  getColor() {
    const currentProgressGate = this.getCurrentProgressGate();
    return currentProgressGate.color;
  }
  getBorderRadius() {
    if (this.getValidProgressNumber() === 100) {
      return '10px';
    }
    return '10px 0px 0px 10px';
  }
  componentWillRender() {
    ValidateProgressGatesType(this.progressGates);
  }
  render() {
    return (h(Host, null,
      h("div", { id: "progressBar", style: {
          width: `${this.getValidProgressNumber()}%`,
          backgroundColor: this.getColor(),
          borderRadius: this.getBorderRadius(),
        } })));
  }
  static get is() { return "sas-progress-bar"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-progress-bar.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-progress-bar.css"]
  }; }
  static get properties() { return {
    "progress": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Progress"
      },
      "attribute": "progress",
      "reflect": true,
      "defaultValue": "50"
    },
    "progressGates": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "ProgressGate[]",
        "resolved": "ProgressGate[]",
        "references": {
          "ProgressGate": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Progress Gates"
      },
      "defaultValue": "[\n    { \n      max: 34,\n      color: \"#D03915\"\n    },\n    { \n      max: 66,\n      color: \"#EBA115\"\n    },\n    { \n      max: 100,\n      color: \"#79D51E\"\n    },\n  ]"
    }
  }; }
}
