// progress-bar.stories.js
import readme from './readme.md';
import customRenderStory from "../../utils/customStoryRenderer";

export default {
  title: 'Basic/Progress Bar',
  parameters: {
    notes: {
      markdown: readme,
    },
  },
};

//#region Basic

const RenderBasic = (args) => {
  return `
  <!--
  Notes specific to implementation in Storybook
  
  class "basic-progress-bar-example" is only needed for example
  and should be omitted after code is copied
  
  Disregard flicker of color on sas-progress-bar, this is a 
  limitation of Storybook, and will not happen in regular usage
  -->

  <sas-progress-bar class="basic-progress-bar-example"
    progress="${args.progress}"
  ></sas-progress-bar>

  <script>
    var sasProgressBar = document.querySelector("sas-progress-bar");
    sasProgressBar.progressGates=${JSON.stringify(args.progressGates)}
  </script>
`
}

export const Basic = (args) => {
  // customRenderStory required to set progressGates after render, otherwise won't work in Docs tab
  return customRenderStory(
    RenderBasic(args),
    () => {
      document.querySelectorAll("sas-progress-bar.basic-progress-bar-example").forEach(sasProgressBar => {
        sasProgressBar.progressGates = args.progressGates
      })
    }
  )
};

Basic.args = {
  progress: 50,
  progressGates: [
    { 
      max: 57,
      color: "#D03915"
    },
    { 
      max: 66,
      color: "#EBA115"
    },
    { 
      max: 100,
      color: "#79D51E"
    },
  ]
};

Basic.argTypes = {
  progress: {
    control: {
      type: 'number',
      min: 0,
      max: 100,
      step: 1,
    },
    description: 'Progress Percentage ranging from 0 to 100 inclusive',
  },
  progressGates: {
    control: 'object',
    description: 'Array of Progress Gates used to determine progress bar color per progress range',
  }
};

//#endregion Basic

//#region Default Progress Gates

export const DefaultProgressGates = (args) => `
  <h3>Progress low (0 - 33%)</h3>
  <sas-progress-bar
    progress="30"
  ></sas-progress-bar>
  <h3>Progress medium (34% - 65%)</h3>
  <sas-progress-bar
    progress="50"
  ></sas-progress-bar>
  <h3>Progress high (66 - 100%)</h3>
  <sas-progress-bar
    progress="80"
  ></sas-progress-bar>
`;

//#endregion Default Progress Gates

//#region Custom Progress Gates

const RenderCustomProgressGates = (args) => {
  return `
  <!--
  Notes specific to implementation in Storybook
  
  class "custom-progress-gates-progress-bar-example" is only needed for example
  and should be omitted after code is copied
  
  Disregard flicker of color on sas-progress-bar, this is a 
  limitation of Storybook, and will not happen in regular usage
  -->

  <sas-progress-bar class="custom-progress-gates-progress-bar-example"
    progress="${args.progress}"
  ></sas-progress-bar>

  <script>
    var sasProgressBar = document.querySelector("sas-progress-bar");
    sasProgressBar.progressGates=${JSON.stringify(args.progressGates)}
  </script>
`
}

export const CustomProgressGates = (args) => {
  // customRenderStory required to set progressGates after render, otherwise won't work in Docs tab
  return customRenderStory(
    RenderCustomProgressGates(args),
    () => {
      document.querySelectorAll("sas-progress-bar.custom-progress-gates-progress-bar-example").forEach(sasProgressBar => {
        sasProgressBar.progressGates = args.progressGates
      })
    }
  )
};

CustomProgressGates.args = {
  progress: 40,
  progressGates: [
    { 
      max: 50,
      color: "blue"
    },
    { 
      max: 66,
      color: "#EBA115"
    },
    { 
      max: 100,
      color: "yellow"
    },
  ]
};

CustomProgressGates.argTypes = {
  progress: {
    control: {
      type: 'number',
      min: 0,
      max: 100,
      step: 1,
    },
    description: 'Progress Percentage ranging from 0 to 100 inclusive',
  },
  progressGates: {
    control: 'object',
    description: 'Array of Progress Gates used to determine progress bar color per progress range',
  }
};

//#endregion Custom Progress Gates