export const ValidateProgressGatesType = (progressGates) => {
  // Validate progressGates is an Array
  if (!Array.isArray(progressGates)) {
    throw new Error(`InvalidTypeError: Progress Gates must be of type: Array \n\n Value: ${progressGates} \n\n`);
  }
  // Validate progressGates Array has atleast one value
  if (progressGates.length === 0) {
    throw new Error(`InvalidValueError: Progress Gates must be of type: Array: with a length more than 0 \n\n Array Length: ${progressGates.length} \n\n`);
  }
  // Validate every value in progressGates Array has the property 'max' of type: number, with a value between 0 and 100 inclusive
  if (!progressGates.every(progressGate => {
    return progressGate.max !== undefined
      && typeof progressGate.max === "number"
      && progressGate.max >= 0
      && progressGate.max <= 100;
  })) {
    throw new Error(`InvalidTypeError: Every Progress Gate must have a property of 'max' of type: number \n\n inlcusive in range: 0 - 100 \n\n Value: ${JSON.stringify(progressGates)} \n\n`);
  }
  // Validate atleast 1 progressGates value in the progressGates Array has the property 'max' of type: number, with a value of 100
  if (!progressGates.some(progressGate => {
    return progressGate.max === 100;
  })) {
    throw new Error(`InvalidTypeError: Atleast 1 Progress Gate must have a property of 'max' of type: number \n\n with a value of: 100 \n\n Value: ${JSON.stringify(progressGates)} \n\n`);
  }
  // Validate every value in progressGates Array has the property 'color' of type: string, with a value supported in CSS.color
  if (!progressGates.every(progressGate => {
    return progressGate.color !== undefined
      && typeof progressGate.color === "string"
      && CSS.supports('color', progressGate.color);
  })) {
    throw new Error(`InvalidTypeError: Every Progress Gate must have a property of 'color' of type: string \n\n Supported in CSS.color property \n\n Value: ${JSON.stringify(progressGates)} \n\n`);
  }
};
