// sas-select.stories.js
import readme from './readme.md';

export default {
  title: 'Higher Order/Select',
  parameters: {
    notes: {
      markdown: readme,
    },
    actions: {
      handles: ['selectupdate', 'sas-select'],
    },
  },
};

const positionOptions = ['top', 'bottom', null];

export const TypeDefault = (args) => `
  <h3>Type default</h3>
  <sas-select
    type="${args.type}"
    label="${args.label}"
    outside="${args.outside}"
    placeholder="${args.placeholder}"
    disabled="${args.disabled}"
    clear="${args.clear}"
    required="${args.required}"
    position="${args.position}"
  >
    <sas-list>
      <sas-list-item value="apple" text="Apple"></sas-list-item>
      <sas-list-item value="banana" text="Banana"></sas-list-item>
      <sas-list-item value="orange" text="Orange"></sas-list-item>
    </sas-list>
  </sas-select>
`;

TypeDefault.args = {
  type: 'default',
  label: 'label',
  outside: false,
  placeholder: 'Please select',
  disabled: false,
  clear: false,
  required: false,
  position: null,
};

TypeDefault.argTypes = {
  position: {
    control: {
      type: 'select',
      options: positionOptions,
    },
  },
};

export const TypeSearch = (args) => `
<h3>Type search</h3>
  <sas-select
    type="${args.type}"
    label="${args.label}"
    outside="${args.outside}"
    placeholder="${args.placeholder}"
    disabled="${args.disabled}"
    clear="${args.clear}"
    required="${args.required}"
    no-match-tip="${args.noMatchTip}"
  >
    <sas-list>
      <sas-list-item value="apple" text="Apple"></sas-list-item>
      <sas-list-item value="banana" text="Banana"></sas-list-item>
      <sas-list-item value="orange" text="Orange"></sas-list-item>
    </sas-list>
  </sas-select>
`;

TypeSearch.args = {
  type: 'search',
  label: 'label',
  outside: false,
  placeholder: 'Please select',
  disabled: false,
  clear: false,
  required: false,
  noMatchTip: 'No data available'
};

export const TypeCreate = (args) => `
<h3>Type create</h3>
  <sas-select
    type="${args.type}"
    label="${args.label}"
    outside="${args.outside}"
    placeholder="${args.placeholder}"
    disabled="${args.disabled}"
    clear="${args.clear}"
    required="${args.required}"
  >
    <sas-list>
      <sas-list-item value="apple" text="Apple"></sas-list-item>
      <sas-list-item value="banana" text="Banana"></sas-list-item>
      <sas-list-item value="orange" text="Orange"></sas-list-item>
    </sas-list>
  </sas-select>
`;

TypeCreate.args = {
  type: 'create',
  label: 'label',
  outside: false,
  placeholder: 'Please select',
  disabled: false,
  clear: false,
  required: false,
};

export const Status = () => `
  <h3>Default</h3>
  <sas-select
    label="label"
  >
    <sas-list>
      <sas-list-item value="apple" text="Apple"></sas-list-item>
      <sas-list-item value="banana" text="Banana"></sas-list-item>
      <sas-list-item value="orange" text="Orange"></sas-list-item>
    </sas-list>
  </sas-select>

  <h3>Label outside</h3>
  <sas-select
    label="label outside"
    outside
  >
    <sas-list>
      <sas-list-item value="apple" text="Apple"></sas-list-item>
      <sas-list-item value="banana" text="Banana"></sas-list-item>
      <sas-list-item value="orange" text="Orange"></sas-list-item>
    </sas-list>
  </sas-select>

  <h3>Required</h3>
  <sas-select
    label="required"
    required
  >
    <sas-list>
      <sas-list-item value="apple" text="Apple"></sas-list-item>
      <sas-list-item value="banana" text="Banana"></sas-list-item>
      <sas-list-item value="orange" text="Orange"></sas-list-item>
    </sas-list>
  </sas-select>

  <h3>Has clear icon</h3>
  <sas-select
    label="Has clear icon"
    clear
  >
    <sas-list>
      <sas-list-item value="apple" text="Apple"></sas-list-item>
      <sas-list-item value="banana" text="Banana"></sas-list-item>
      <sas-list-item value="orange" text="Orange"></sas-list-item>
    </sas-list>
  </sas-select>

  <h3>Disabled</h3>
  <sas-select
    label="Disabled"
    disabled
  >
    <sas-list>
      <sas-list-item value="apple" text="Apple"></sas-list-item>
      <sas-list-item value="banana" text="Banana"></sas-list-item>
      <sas-list-item value="orange" text="Orange"></sas-list-item>
    </sas-list>
  </sas-select>
`;

export const Subtext = (args) => `
  <sas-select subtext-position="${args.subtextPosition}">
    <sas-list>
      <sas-list-item value="apple" text="Apple" subtext="apple sub text"></sas-list-item>
      <sas-list-item value="banana" text="Banana" subtext="banana sub text"></sas-list-item>
      <sas-list-item value="orange" text="Orange" subtext="orange sub text"></sas-list-item>
    </sas-list>
  </sas-select>
`

Subtext.args = {
  subtextPosition: 'bottom',
}

const subtextPositionOptions = ['bottom', 'right']

Subtext.argTypes = {
  subtextPosition: {
    control: {
      type: 'select',
      options: subtextPositionOptions,
    },
  },
};
