import { Component, Host, h, Prop, Element, Listen, Event, Watch, } from '@stencil/core';
export class SasStepper {
  constructor() {
    /**
     *  Total steps
     */
    this.total = 10;
    /**
     *  Current step
     */
    this.current = 0;
    /**
     *  Step names
     */
    this.names = [];
    /**
     *  Stepper Direction
     */
    this.direction = "row";
    /**
     *  Used to provide what backgound this will be displayed on, as colors need updating to allow for contrast differences
     */
    this.backgroundTheme = "light";
    this.stepNameClickedHandler = (event) => {
      event.stopPropagation();
      const stepNumber = Number(event.target.dataset.stepIndex);
      if (this.current !== stepNumber) {
        this.current = stepNumber;
      }
    };
  }
  watchHandler(newValue) {
    this.stepperUpdate.emit(newValue);
  }
  buttonclickedHandler(event) {
    event.stopPropagation();
    if (this.current !== event.detail) {
      this.current = event.detail;
    }
  }
  getSteppers() {
    const steppers = [];
    const stepperNames = this.getStepperNames();
    for (let i = 0; i < this.total; i += 1) {
      let stepper;
      if (i < this.current) {
        stepper = h("sas-button", { type: "primary", "data-status": "complete", text: `${i + 1}` },
          h("div", { onClick: this.stepNameClickedHandler, "data-step-index": `${i}`, class: "step-name" }, stepperNames[i]));
      }
      else if (i === this.current) {
        stepper = h("sas-button", { type: "text", "data-status": "active", text: `${i + 1}` },
          h("div", { onClick: this.stepNameClickedHandler, "data-step-index": `${i}`, class: "step-name" }, stepperNames[i]));
      }
      else {
        stepper = h("sas-button", { type: "text", "data-status": "inactive", text: `${i + 1}` },
          h("div", { onClick: this.stepNameClickedHandler, "data-step-index": `${i}`, class: "step-name" }, stepperNames[i]));
      }
      steppers.push(stepper);
      if (i !== this.total - 1) {
        steppers.push(h("sas-divider", null));
      }
    }
    return steppers;
  }
  getStepperNames() {
    if (this.names.length === this.total) {
      return this.names;
    }
    const names = [];
    for (let i = 0; i < this.total; i += 1) {
      names.push(`Step ${i + 1}`);
    }
    return names;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" }, this.getSteppers())));
  }
  static get is() { return "sas-stepper"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-stepper.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-stepper.css"]
  }; }
  static get properties() { return {
    "total": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Total steps"
      },
      "attribute": "total",
      "reflect": true,
      "defaultValue": "10"
    },
    "current": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Current step"
      },
      "attribute": "current",
      "reflect": true,
      "defaultValue": "0"
    },
    "names": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "string[]",
        "resolved": "string[]",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Step names"
      },
      "defaultValue": "[]"
    },
    "direction": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "\"row\" | \"column\"",
        "resolved": "\"column\" | \"row\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Stepper Direction"
      },
      "attribute": "direction",
      "reflect": true,
      "defaultValue": "\"row\""
    },
    "backgroundTheme": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "\"dark\" | \"light\"",
        "resolved": "\"dark\" | \"light\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Used to provide what backgound this will be displayed on, as colors need updating to allow for contrast differences"
      },
      "attribute": "background-theme",
      "reflect": true,
      "defaultValue": "\"light\""
    }
  }; }
  static get events() { return [{
      "method": "stepperUpdate",
      "name": "stepperupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction"
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "current",
      "methodName": "watchHandler"
    }]; }
  static get listeners() { return [{
      "name": "buttonclicked",
      "method": "buttonclickedHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
