import { Component, Host, h, Prop, Element, Watch, Listen, } from '@stencil/core';
import { animate } from '../../global/global';
export class SasSublist {
  constructor() {
    /**
     * Icon left
     */
    this.icon = '';
    /**
     * Value
     */
    this.value = null;
    /**
     * Text
     */
    this.text = '';
    /**
     *  Sub text
     */
    this.subtext = '';
    /**
     * Icon Right
     */
    this.iconRight = '';
    /**
     * Enable toggle sub list
     */
    this.toggle = false;
    /**
     * Open current sub list only
     */
    this.compact = false;
    /**
     * hide sub list
     */
    this.hideSub = true;
    /**
     * selected
     */
    this.selected = false;
    this.clickEvent = () => {
      if (this.toggle) {
        if (this.compact) {
          this.hideSiblingSubList();
        }
        this.deSelectAll();
        this.selected = true;
        this.hideSub = !this.hideSub;
      }
    };
  }
  watchHandler(newValue) {
    const subList = this.el.shadowRoot.querySelector('.list-item-container');
    let drawFunction;
    if (!newValue) { // open
      // get subList height and reset it
      subList.style.height = 'initial';
      subList.style.opacity = '0';
      subList.style.position = 'absolute';
      const height = subList.getBoundingClientRect().height;
      subList.style.height = '0';
      subList.style.opacity = '1';
      subList.style.position = 'initial';
      drawFunction = function (progress) {
        subList.style.height = `${progress * height}px`;
      };
    }
    else { // close
      const height = subList.getBoundingClientRect().height;
      drawFunction = function (progress) {
        subList.style.height = `${(1 - progress) * height}px`;
      };
    }
    animate({
      duration: 300,
      timing: function (timeFraction) {
        return timeFraction;
      },
      draw: drawFunction,
    });
  }
  listItemClickEventUsedInternalHandler(event) {
    event.stopPropagation();
    this.deSelectAll();
    this.selected = true;
  }
  hideSiblingSubList() {
    let prevSibling = this.el.previousElementSibling;
    while (Boolean(prevSibling)) {
      if (prevSibling.tagName === 'SAS-SUBLIST' && !prevSibling.hideSub) {
        prevSibling.hideSub = true;
      }
      prevSibling = prevSibling.previousElementSibling;
    }
    let nextSibling = this.el.nextElementSibling;
    while (Boolean(nextSibling)) {
      if (nextSibling.tagName === 'SAS-SUBLIST' && !nextSibling.hideSub) {
        nextSibling.hideSub = true;
      }
      nextSibling = nextSibling.nextElementSibling;
    }
  }
  deSelectAll() {
    let prevSibling = this.el.previousElementSibling;
    while (Boolean(prevSibling)) {
      if (prevSibling.tagName === 'SAS-SUBLIST' && prevSibling.selected) {
        prevSibling.selected = false;
      }
      prevSibling = prevSibling.previousElementSibling;
    }
    let nextSibling = this.el.nextElementSibling;
    while (Boolean(nextSibling)) {
      if (nextSibling.tagName === 'SAS-SUBLIST' && nextSibling.selected) {
        nextSibling.selected = false;
      }
      nextSibling = nextSibling.nextElementSibling;
    }
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        h("sas-list-item", { class: "title", icon: this.icon, text: this.text, subtext: this.subtext, "icon-right": this.iconRight, onClick: this.clickEvent }),
        h("div", { class: "list-item-container" },
          h("slot", null)))));
  }
  static get is() { return "sas-sublist"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-sublist.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-sublist.css"]
  }; }
  static get properties() { return {
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon left"
      },
      "attribute": "icon",
      "reflect": true,
      "defaultValue": "''"
    },
    "value": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "ListItemValue",
        "resolved": "boolean | number | string",
        "references": {
          "ListItemValue": {
            "location": "import",
            "path": "../sas-list-item/type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Value"
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "null"
    },
    "text": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Text"
      },
      "attribute": "text",
      "reflect": true,
      "defaultValue": "''"
    },
    "subtext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Sub text"
      },
      "attribute": "subtext",
      "reflect": true,
      "defaultValue": "''"
    },
    "iconRight": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon Right"
      },
      "attribute": "icon-right",
      "reflect": true,
      "defaultValue": "''"
    },
    "toggle": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enable toggle sub list"
      },
      "attribute": "toggle",
      "reflect": true,
      "defaultValue": "false"
    },
    "compact": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Open current sub list only"
      },
      "attribute": "compact",
      "reflect": true,
      "defaultValue": "false"
    },
    "hideSub": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "hide sub list"
      },
      "attribute": "hide-sub",
      "reflect": true,
      "defaultValue": "true"
    },
    "selected": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "selected"
      },
      "attribute": "selected",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "hideSub",
      "methodName": "watchHandler"
    }]; }
  static get listeners() { return [{
      "name": "listItemClickEventUsedInternal",
      "method": "listItemClickEventUsedInternalHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
