import { Component, Host, h, Prop, Event, Element, Watch, Method, } from '@stencil/core';
import { keydownEvent } from '../../global/global';
export class SasSwitch {
  constructor() {
    /**
     * name
     */
    this.name = null;
    /**
     *  Check Status
     */
    this.checked = false;
    /**
     *  Required
     */
    this.required = false;
    /**
     *  Error
     */
    this.error = false;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
    };
    /**
     * Only shows when prop error is true, auto generated
     */
    this.message = '';
    this.clickEvent = () => {
      this.checked = !this.checked;
      this.switchUpdate.emit(this.checked);
    };
  }
  watchHandler() {
    this.resetErrorStatus();
  }
  /**
   * Validate
   */
  async validate() {
    let valueMissing = false;
    if (this.required && !this.checked) {
      valueMissing = true;
    }
    this.validity = {
      valid: !valueMissing,
      valueMissing: valueMissing,
    };
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    await this.validate();
    this.error = !this.validity.valid;
    if (!this.validity.valid) {
      this.message = this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    this.error = false;
    this.message = '';
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      return `Please check this field`;
    }
  }
  render() {
    return (h(Host, { onClick: this.clickEvent },
      h("div", { class: "container", "data-message": this.message },
        h("div", { class: "switch", tabindex: 0, onKeyDown: keydownEvent }))));
  }
  static get is() { return "sas-switch"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-switch.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-switch.css"]
  }; }
  static get properties() { return {
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "null"
    },
    "checked": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Check Status"
      },
      "attribute": "checked",
      "reflect": true,
      "defaultValue": "false"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Required"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "error": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Error"
      },
      "attribute": "error",
      "reflect": true,
      "defaultValue": "false"
    },
    "validity": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "{ valid: boolean; valueMissing: boolean; }",
        "resolved": "{ valid: boolean; valueMissing: boolean; }",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Validity"
      },
      "defaultValue": "{\n    valid: true,\n    valueMissing: false,\n  }"
    },
    "message": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Only shows when prop error is true, auto generated"
      },
      "attribute": "message",
      "reflect": true,
      "defaultValue": "''"
    }
  }; }
  static get events() { return [{
      "method": "switchUpdate",
      "name": "switchupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction"
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "validate": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Validate",
        "tags": []
      }
    },
    "updateErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show error status if value is not valid (only for components internal use)",
        "tags": []
      }
    },
    "resetErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clear error status (only for components internal use)",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "checked",
      "methodName": "watchHandler"
    }]; }
}
