import { Component, Host, h, Prop, State, Watch, } from '@stencil/core';
export class SasTable {
  constructor() {
    /**
     * Table Headers
     */
    this.headers = [];
    /**
     * Table Items
     */
    this.items = [];
    this.sortedItems = [];
    this.sort = (event) => {
      let key;
      let iconElement;
      let sortAble = true;
      // handle condition when click on sas-icon
      if (event.target.tagName === 'TH') {
        key = event.target.getAttribute('data-value');
        iconElement = event.target.querySelector('sas-icon');
        if (!iconElement) {
          sortAble = false;
        }
      }
      else {
        key = event.target.parentNode.getAttribute('data-value');
        iconElement = event.target;
      }
      if (sortAble) {
        if (iconElement.icon === 'fas fa-sort-up') {
          iconElement.icon = 'fas fa-sort-down';
          this.sortedItems.sort((a, b) => {
            if (a[key] < b[key]) {
              return 1;
            }
            if (a[key] > b[key]) {
              return -1;
            }
            return 0;
          });
        }
        else {
          iconElement.icon = 'fas fa-sort-up';
          this.sortedItems.sort((a, b) => {
            if (a[key] < b[key]) {
              return -1;
            }
            if (a[key] > b[key]) {
              return 1;
            }
            return 0;
          });
        }
        this.sortedItems = [...this.sortedItems];
      }
    };
  }
  componentWillLoad() {
    this.sortedItems = [...this.items];
  }
  watchHandler(newValue) {
    this.sortedItems = [...newValue];
  }
  generateTableContent(item) {
    const content = Object.values(item).map(value => h("td", null, value));
    return content;
  }
  render() {
    return (h(Host, null,
      h("table", null,
        h("thead", null,
          h("tr", null, this.headers.map((header) => h("th", { "data-value": header.value, onClick: this.sort },
            header.text,
            header.sortAble
              ? h("sas-icon", { icon: "fas fa-sort" })
              : null)))),
        h("tbody", null, this.sortedItems.map((item) => h("tr", null, this.generateTableContent(item)))))));
  }
  static get is() { return "sas-table"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-table.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-table.css"]
  }; }
  static get properties() { return {
    "headers": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Header[]",
        "resolved": "Header[]",
        "references": {
          "Header": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Table Headers"
      },
      "defaultValue": "[]"
    },
    "items": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "object[]",
        "resolved": "object[]",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Table Items"
      },
      "defaultValue": "[]"
    }
  }; }
  static get states() { return {
    "sortedItems": {}
  }; }
  static get watchers() { return [{
      "propName": "items",
      "methodName": "watchHandler"
    }]; }
}
