import { Component, Host, h, Prop, Listen, Element, Event, Watch, Method, } from '@stencil/core';
export class SasTimeField {
  constructor() {
    this.isUpdateTimeFieldList = true;
    /**
     * Label
     */
    this.label = '';
    /**
     * Label position
     */
    this.outside = false;
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * name
     */
    this.name = '';
    /**
     * Value
     */
    this.value = '';
    /**
     * Max value
     */
    this.max = '';
    /**
     * Min value
     */
    this.min = '';
    /**
     * disabled
     */
    this.disabled = false;
    /**
     * Clear
     */
    this.clear = false;
    /**
     * Default container position
     */
    this.position = null;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
      rangeUnderflow: false,
      rangeOverflow: false,
    };
    /**
     * Only shows when prop error is true, auto generated
     */
    this.message = '';
  }
  watchHandler() {
    if (this.isUpdateTimeFieldList) {
      this.updateTimeFieldList();
    }
    else {
      this.isUpdateTimeFieldList = true;
    }
    this.resetErrorStatus();
  }
  componentDidLoad() {
    this.infiniteScroll();
  }
  inputUpdateHandler(event) {
    this.value = event.detail;
    this.timeFieldUpdate.emit(event.detail);
    this.timefieldupdate.emit(event.detail);
  }
  inputContainerClickHandler() {
    this.updateTimeFieldList();
  }
  listItemClickedHandler() {
    const hourList = this.el.shadowRoot.querySelector('#hour');
    const minuteList = this.el.shadowRoot.querySelector('#minute');
    const dayPartList = this.el.shadowRoot.querySelector('#day-part');
    if (Boolean(hourList.value) && Boolean(minuteList.value) && Boolean(dayPartList.value)) {
      const currentValue = this.convertTo24Hour(this.formatValue(hourList.value), this.formatValue(minuteList.value), dayPartList.value);
      if (this.value !== currentValue) {
        this.isUpdateTimeFieldList = false;
        this.value = currentValue;
        this.timeFieldUpdate.emit(currentValue);
        this.timefieldupdate.emit(currentValue);
      }
    }
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async validate() {
    let valueMissing = false;
    if (this.required && !Boolean(this.value)) {
      valueMissing = true;
    }
    let rangeUnderflow = false;
    if (Boolean(this.min) && Boolean(this.value)) {
      if (this.value < this.min) {
        rangeUnderflow = true;
      }
    }
    let rangeOverflow = false;
    if (Boolean(this.max) && Boolean(this.value)) {
      if (this.value > this.max) {
        rangeOverflow = true;
      }
    }
    this.validity = {
      valid: !valueMissing && !rangeUnderflow && !rangeOverflow,
      valueMissing: valueMissing,
      rangeUnderflow: rangeUnderflow,
      rangeOverflow: rangeOverflow,
    };
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    await this.validate();
    const formInputTemplate = this.el.shadowRoot.querySelector('sas-form-input-template');
    if (Boolean(formInputTemplate)) {
      formInputTemplate.error = !this.validity.valid;
    }
    if (!this.validity.valid) {
      this.message = this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    const formInputTemplate = this.el.shadowRoot.querySelector('sas-form-input-template');
    if (Boolean(formInputTemplate)) {
      formInputTemplate.error = false;
    }
    this.message = '';
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      return `${this.label} is required`;
    }
    if (validity.rangeOverflow) {
      return `${this.label} must be less or equal to ${this.max}`;
    }
    if (validity.rangeUnderflow) {
      return `${this.label} must be larger or equal to ${this.min}`;
    }
  }
  infiniteScroll() {
    const observer = new IntersectionObserver((entries) => {
      entries.forEach((entry) => {
        if (entry.intersectionRatio > 0) {
          const topCurrent = entry.boundingClientRect.top;
          const parent = entry.target.parentElement;
          const value = entry.target.value;
          parent.scrollTop = 0;
          const targetElement = parent.querySelector(`sas-list-item.middle[text="${value}"]`);
          const topAll = targetElement.getBoundingClientRect().top;
          const scrollTop = topAll - topCurrent;
          parent.scrollTop = scrollTop;
        }
      });
    });
    const hourStartElement = this.el.shadowRoot.querySelector('sas-list-item.top[text="01"]');
    observer.observe(hourStartElement);
    const hourEndElement = this.el.shadowRoot.querySelector('sas-list-item.bottom[text="12"]');
    observer.observe(hourEndElement);
    const minuteStartElement = this.el.shadowRoot.querySelector('sas-list-item.top[text="00"]');
    observer.observe(minuteStartElement);
    const minuteEndElement = this.el.shadowRoot.querySelector('sas-list-item.bottom[text="59"]');
    observer.observe(minuteEndElement);
  }
  updateTimeFieldList() {
    let time;
    if (Boolean(this.value)) {
      time = this.value;
    }
    else {
      const now = new Date();
      const hour = `${now.getHours()}`;
      const minute = `${now.getMinutes()}`;
      time = `${hour}:${minute}`;
    }
    const formatTimeObj = this.convertTo12Hour(time);
    const hourList = this.el.shadowRoot.querySelector('#hour');
    hourList.value = formatTimeObj.hour;
    const selectedHour = hourList.querySelector(`sas-list-item.middle[text="${formatTimeObj.hour}"]`);
    selectedHour.parentElement.scrollTop = selectedHour.offsetTop;
    const minuteList = this.el.shadowRoot.querySelector('#minute');
    minuteList.value = formatTimeObj.minute;
    const selectedMinute = minuteList.querySelector(`sas-list-item.middle[text="${formatTimeObj.minute}"]`);
    selectedMinute.parentElement.scrollTop = selectedMinute.offsetTop;
    const dayPartList = this.el.shadowRoot.querySelector('#day-part');
    dayPartList.value = formatTimeObj.dayPart;
  }
  convertTo12Hour(time) {
    const timeArray = time.split(':');
    const hour = this.getFormatHour(Number(timeArray[0]));
    const minute = this.getFormatMinute(Number(timeArray[1]));
    const dayPart = this.getDayPart(Number(timeArray[0]));
    return {
      hour: hour,
      minute: minute,
      dayPart: dayPart,
    };
  }
  convertTo24Hour(hour, minute, dayPart) {
    let finalHour;
    if (dayPart === 'AM') {
      if (hour === '12') {
        finalHour = '00';
      }
      else {
        finalHour = hour;
      }
    }
    else {
      if (hour !== '12') {
        let hourNumber = Number(hour) + 12;
        finalHour = `${hourNumber}`;
      }
      else {
        finalHour = hour;
      }
    }
    return `${finalHour}:${minute}`;
  }
  getFormatHour(hour) {
    let result = hour;
    if (hour === 0) {
      result = 12;
    }
    if (hour > 12) {
      result = hour - 12;
    }
    if (result < 10) {
      return `0${result}`;
    }
    return `${result}`;
  }
  getFormatMinute(minute) {
    if (minute < 10) {
      return `0${minute}`;
    }
    return `${minute}`;
  }
  getDayPart(hour) {
    if (hour < 12) {
      return 'AM';
    }
    return 'PM';
  }
  formatValue(value) {
    if (value.length > 2) {
      return value.slice(0, 2);
    }
    return value;
  }
  getElement(start, end) {
    let elements = [];
    for (let i = start; i <= end; i += 1) {
      let text = `${i}`;
      if (i < 10) {
        text = `0${i}`;
      }
      const element = h("sas-list-item", { class: "top", value: text, text: text });
      elements.push(element);
    }
    for (let i = start; i <= end; i += 1) {
      let text = `${i}`;
      if (i < 10) {
        text = `0${i}`;
      }
      const element = h("sas-list-item", { class: "middle", value: text, text: text });
      elements.push(element);
    }
    for (let i = start; i <= end; i += 1) {
      let text = `${i}`;
      if (i < 10) {
        text = `0${i}`;
      }
      const element = h("sas-list-item", { class: "bottom", value: text, text: text });
      elements.push(element);
    }
    return elements;
  }
  render() {
    return (h(Host, null,
      h("sas-form-input-template", { label: this.label, outside: this.outside, required: this.required, value: this.value, iconRight: "far fa-clock", type: "time", disabled: this.disabled, clear: this.clear, "data-message": this.message, position: this.position, minimumPixelsAboveComponentRequired: 280 },
        h("div", { slot: "container", class: "time-field", tabindex: -1 },
          h("sas-list", { class: "part", id: "hour" }, this.getElement(1, 12)),
          h("sas-list", { class: "part", id: "minute" }, this.getElement(0, 59)),
          h("sas-list", { class: "part", id: "day-part" },
            h("sas-list-item", { value: "AM", text: "AM" }),
            h("sas-list-item", { value: "PM", text: "PM" }))))));
  }
  static get is() { return "sas-time-field"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-time-field.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-time-field.css"]
  }; }
  static get properties() { return {
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label"
      },
      "attribute": "label",
      "reflect": true,
      "defaultValue": "''"
    },
    "outside": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label position"
      },
      "attribute": "outside",
      "reflect": true,
      "defaultValue": "false"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Shows a star symbol behind label"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "''"
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Value"
      },
      "attribute": "value",
      "reflect": true,
      "defaultValue": "''"
    },
    "max": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Max value"
      },
      "attribute": "max",
      "reflect": true,
      "defaultValue": "''"
    },
    "min": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Min value"
      },
      "attribute": "min",
      "reflect": true,
      "defaultValue": "''"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "clear": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Clear"
      },
      "attribute": "clear",
      "reflect": true,
      "defaultValue": "false"
    },
    "position": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'top' | 'bottom' | null",
        "resolved": "\"bottom\" | \"top\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Default container position"
      },
      "attribute": "position",
      "reflect": true,
      "defaultValue": "null"
    },
    "validity": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "{ valid: boolean; valueMissing: boolean; rangeUnderflow: boolean; rangeOverflow: boolean; }",
        "resolved": "{ valid: boolean; valueMissing: boolean; rangeUnderflow: boolean; rangeOverflow: boolean; }",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Validity"
      },
      "defaultValue": "{\n    valid: true,\n    valueMissing: false,\n    rangeUnderflow: false,\n    rangeOverflow: false,\n  }"
    },
    "message": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Only shows when prop error is true, auto generated"
      },
      "attribute": "message",
      "reflect": true,
      "defaultValue": "''"
    }
  }; }
  static get events() { return [{
      "method": "timeFieldUpdate",
      "name": "timeFieldUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction (will be deprecated)"
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }, {
      "method": "timefieldupdate",
      "name": "timefieldupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction"
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "validate": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show error status if value is not valid (only for components internal use)",
        "tags": []
      }
    },
    "updateErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show error status if value is not valid (only for components internal use)",
        "tags": []
      }
    },
    "resetErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clear error status (only for components internal use)",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "watchHandler"
    }]; }
  static get listeners() { return [{
      "name": "inputUpdate",
      "method": "inputUpdateHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "inputContainerClick",
      "method": "inputContainerClickHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "listItemClicked",
      "method": "listItemClickedHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
