import { Component, Host, h, Element, Prop, } from '@stencil/core';
/**
 * @slot trigger - used for content which can trigger tooltip text
 * @slot text - used for tooltip text
 */
export class SasTooltip {
  constructor() {
    /**
     * Trigger text
     */
    this.trigger = '';
    /**
     * Tooltip text
     */
    this.text = '';
    /**
     * Default tooltip position
     */
    this.position = 'bottom';
    this.mouseOverHandler = () => {
      this.updatePosition();
      const chatBubble = this.el.shadowRoot.querySelector('sas-chat-bubble');
      chatBubble.classList.remove('hide');
    };
    this.mouseOutHandler = () => {
      const chatBubble = this.el.shadowRoot.querySelector('sas-chat-bubble');
      chatBubble.classList.add('hide');
    };
  }
  updatePosition() {
    const bubble = this.el.shadowRoot.querySelector('sas-chat-bubble');
    const bubblePosition = bubble.getBoundingClientRect();
    let newPosition = this.position;
    if (bubblePosition.top < 0) {
      newPosition = 'bottom';
    }
    if (bubblePosition.bottom > document.documentElement.clientHeight) {
      newPosition = 'top';
    }
    if (bubblePosition.left < 0) {
      newPosition = 'right';
    }
    if (bubblePosition.right > document.documentElement.clientWidth) {
      newPosition = 'left';
    }
    this.position = newPosition;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        h("div", { class: "trigger-container", tabindex: 0, onPointerOver: this.mouseOverHandler, onFocus: this.mouseOverHandler, onPointerOut: this.mouseOutHandler, onBlur: this.mouseOutHandler },
          h("slot", { name: "trigger" },
            h("span", null, this.trigger))),
        h("sas-chat-bubble", { class: "chatBubble hide", position: this.position },
          h("slot", { name: "text" },
            h("p", { class: "text" }, this.text))))));
  }
  static get is() { return "sas-tooltip"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-tooltip.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-tooltip.css"]
  }; }
  static get properties() { return {
    "trigger": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Trigger text"
      },
      "attribute": "trigger",
      "reflect": true,
      "defaultValue": "''"
    },
    "text": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Tooltip text"
      },
      "attribute": "text",
      "reflect": true,
      "defaultValue": "''"
    },
    "position": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Default tooltip position"
      },
      "attribute": "position",
      "reflect": true,
      "defaultValue": "'bottom'"
    }
  }; }
  static get elementRef() { return "el"; }
}
