import { Component, Host, h, Prop, Listen, } from '@stencil/core';
/**
 * @slot list - used for dropdown menu
 */
export class SasUserCard {
  constructor() {
    /**
     * Display as block
     */
    this.block = false;
    /**
     * Name
     */
    this.name = 'Walter White';
    /**
     * Position
     */
    this.position = 'Business Administrator';
    /**
     * Position
     */
    this.status = 'ACTIVE';
    /**
     * Badge
     */
    this.badge = false;
    /**
     * Has attachment
     */
    this.attachment = false;
    /**
     * Src
     */
    this.src = 'https://cdn.vuetifyjs.com/images/john.jpg';
  }
  switchUpdateHandler(event) {
    event.stopPropagation();
    if (event.detail) {
      this.status = 'ACTIVE';
    }
    else {
      this.status = 'INACTIVE';
    }
  }
  getDisabledValue() {
    if (this.status.toUpperCase() === 'ARCHIVED') {
      return true;
    }
    return false;
  }
  getSwitchClass() {
    if (this.status.toUpperCase() === 'ARCHIVED') {
      return 'hide';
    }
    return '';
  }
  getCheckedValue() {
    const status = this.status.toUpperCase();
    if (status === 'ACTIVE') {
      return true;
    }
    return false;
  }
  getIcons() {
    const icons = [];
    let icon;
    if (this.badge) {
      icon = h("li", null,
        h("sas-icon", { icon: "fas fa-id-badge" }));
      icons.push(icon);
    }
    if (this.attachment) {
      icon = h("li", null,
        h("sas-icon", { icon: "fas fa-clipboard-list" }));
      icons.push(icon);
    }
    return icons;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        h("sas-avatar", { type: "medium", src: this.src, disabled: this.getDisabledValue() }),
        h("sas-dropdown", { class: "option", "position-h": "left" },
          h("sas-icon", { tabindex: "0", class: "ellipsis-icon", slot: "control", icon: "fal fa-ellipsis-h" }),
          h("slot", { name: "list", slot: "menu" })),
        h("div", { class: "content" },
          h("div", { class: "title" }, this.name),
          h("div", { class: "subtitle" }, this.position),
          h("div", { class: "control" },
            h("sas-switch", { class: this.getSwitchClass(), checked: this.getCheckedValue() }),
            h("div", { class: "status" }, this.status.toUpperCase()),
            h("ul", null, this.getIcons()))))));
  }
  static get is() { return "sas-user-card"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-user-card.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-user-card.css"]
  }; }
  static get properties() { return {
    "block": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Display as block"
      },
      "attribute": "block",
      "reflect": true,
      "defaultValue": "false"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "'Walter White'"
    },
    "position": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Position"
      },
      "attribute": "position",
      "reflect": true,
      "defaultValue": "'Business Administrator'"
    },
    "status": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Position"
      },
      "attribute": "status",
      "reflect": true,
      "defaultValue": "'ACTIVE'"
    },
    "badge": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Badge"
      },
      "attribute": "badge",
      "reflect": true,
      "defaultValue": "false"
    },
    "attachment": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Has attachment"
      },
      "attribute": "attachment",
      "reflect": true,
      "defaultValue": "false"
    },
    "src": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Src"
      },
      "attribute": "src",
      "reflect": true,
      "defaultValue": "'https://cdn.vuetifyjs.com/images/john.jpg'"
    }
  }; }
  static get listeners() { return [{
      "name": "switchupdate",
      "method": "switchUpdateHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
