import { r as registerInstance, c as createEvent, h, H as Host } from './index-82af0817.js';

const sasAvatarCss = ":host{display:inline-flex}:host .container{display:inline-flex;justify-content:center;align-items:center;background-color:var(--s-400);border-radius:50%;cursor:pointer;margin-left:var(--avatar-margin-left, 0);transition:var(--avatar-transition, 0)}:host sas-icon{--icon-color:white}:host img{max-width:100%;max-height:100%;border-radius:inherit}:host .text-container{display:flex;align-items:center;font-weight:500;-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;}:host(:hover) .container{transform:var(--avatar-hover-transform, none)}:host([type=small]) .container{max-height:var(--avatar-max-height, initial);min-height:var(--avatar-min-height, initial);height:var(--avatar-height, 34px);max-width:var(--avatar-max-width, initial);min-width:var(--avatar-min-width, initial);width:var(--avatar-width, 34px);background-color:var(--avatar-small-background);color:var(--avatar-small-text);font-size:12px}:host([type=medium]) .container{max-height:var(--avatar-max-height, initial);min-height:var(--avatar-min-height, initial);height:var(--avatar-height, 70px);max-width:var(--avatar-max-width, initial);min-width:var(--avatar-min-width, initial);width:var(--avatar-width, 70px)}:host([type=medium]) .container sas-icon{--icon-size:45px}:host([type=large]) .container{max-height:var(--avatar-max-height, initial);min-height:var(--avatar-min-height, initial);height:var(--avatar-height, 136px);max-width:var(--avatar-max-width, initial);min-width:var(--avatar-min-width, initial);width:var(--avatar-width, 136px)}:host([type=large]) .container sas-icon{--icon-size:90px}:host([disabled]:not([disabled=false])) .container{opacity:0.6;pointer-events:none}";

let SasAvatar = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.avatarClicked = createEvent(this, "avatarClicked", 7);
    /**
     * Type
     */
    this.type = 'small';
    /**
     * Name of avatar, display initials if no image src
     */
    this.name = 'Unknown';
    /**
     * Image source
     */
    this.src = '';
    /**
     * Disabled
     */
    this.disabled = false;
    /**
     * Group
     */
    this.group = false;
    this.handleClick = (event) => {
      this.avatarClicked.emit(event.target.name);
    };
  }
  getContent() {
    if (Boolean(this.src)) {
      return h("img", { src: this.src });
    }
    if (this.type !== 'small') {
      return h("sas-icon", { icon: "fas fa-user" });
    }
    return h("span", { class: "text-container" }, this.getInitial());
  }
  getInitial() {
    if (Boolean(this.name)) {
      const nameArray = this.name.split(' ');
      let initial = this.getFirstValidLetter(nameArray[0]);
      if (nameArray.length > 1) {
        initial += this.getFirstValidLetter(nameArray[1]);
      }
      return initial;
    }
    return '';
  }
  getFirstValidLetter(name) {
    for (let i = 0; i < name.length; i++) {
      if (/^[a-z]$/i.test(name[i])) {
        return name[i].toUpperCase();
      }
    }
    return '';
  }
  render() {
    return (h(Host, { onClick: this.handleClick }, h("div", { class: "container" }, this.getContent())));
  }
};
SasAvatar.style = sasAvatarCss;

export { SasAvatar as sas_avatar };
