import { r as registerInstance, h, H as Host } from './index-82af0817.js';
import { k as keydownEvent } from './global-8d66eb74.js';

const sasBreadcrumbCss = ":host{display:block;font-size:14px;font-weight:500}:host ul{list-style-type:none;display:inline-flex;margin:0;padding:0}:host a{color:var(--secondary);text-decoration:none}:host a:hover,:host a:focus{text-decoration:underline;cursor:pointer;outline:none}:host .disabled{color:var(--s-400);font-weight:normal;pointer-events:none}:host .divider{padding:0 15px}";

let SasBreadcrumb = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    /**
     * Breadcrumb item array
     */
    this.items = [];
    this.showAll = false;
    this.showAllBreadcrumb = () => {
      this.showAll = true;
    };
  }
  getBreadcrumbs() {
    const breadcrumbs = [];
    let breadcrumb;
    let divider;
    if (this.items.length > 5 && !this.showAll) {
      breadcrumb = h("li", null, h("a", { class: this.getDisabled(this.items[0].disabled), tabindex: this.getTabindex(this.items[0].disabled), href: this.items[0].href }, this.items[0].text));
      breadcrumbs.push(breadcrumb);
      divider = h("li", { class: "divider" }, "/");
      breadcrumbs.push(divider);
      breadcrumb = h("li", null, h("a", { tabindex: 0, onClick: this.showAllBreadcrumb, onKeyDown: keydownEvent }, "..."));
      breadcrumbs.push(breadcrumb);
      divider = h("li", { class: "divider" }, "/");
      breadcrumbs.push(divider);
      breadcrumb = h("li", null, h("a", { class: this.getDisabled(this.items[this.items.length - 1].disabled), tabindex: this.getTabindex(this.items[this.items.length - 1].disabled), href: this.items[this.items.length - 1].href }, this.items[this.items.length - 1].text));
      breadcrumbs.push(breadcrumb);
    }
    else {
      this.items.forEach((item, index) => {
        breadcrumb = h("li", null, h("a", { class: this.getDisabled(item.disabled), tabindex: this.getTabindex(item.disabled), href: item.href }, item.text));
        breadcrumbs.push(breadcrumb);
        if (index !== this.items.length - 1) {
          divider = h("li", { class: "divider" }, "/");
          breadcrumbs.push(divider);
        }
      });
    }
    return breadcrumbs;
  }
  getDisabled(disabled) {
    if (disabled) {
      return 'disabled';
    }
    return '';
  }
  getTabindex(disabled) {
    if (disabled) {
      return -1;
    }
    return 0;
  }
  render() {
    return (h(Host, null, h("ul", null, this.getBreadcrumbs())));
  }
};
SasBreadcrumb.style = sasBreadcrumbCss;

export { SasBreadcrumb as sas_breadcrumb };
