import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-82af0817.js';

const sasButtonGroupCss = ":host{--button-background-color:var(--button-text-background);--button-border:1px solid var(--s-200);--button-border-radius:0;--button-color:var(--s-400);--button-margin:0 0 0 -1px}:host .container{display:inline-block;font-size:0;margin-left:1px}::slotted(sas-button:first-child){--button-border-radius:5px 0 0 5px}::slotted(sas-button:last-child){--button-border-radius:0 5px 5px 0}::slotted(sas-button.button-group-active){--button-background-color:var(--s-200);--button-color:#214F67}";

let SasButtonGroup = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.buttonGroupUpdate = createEvent(this, "buttonGroupUpdate", 7);
    /**
     * Active Button Index
     */
    this.values = [];
    /**
     * Enable select multiple buttons
     */
    this.multiple = false;
  }
  componentDidLoad() {
    this.addActiveClass(this.values);
  }
  watchHandler(newValue) {
    this.addActiveClass(newValue);
    this.buttonGroupUpdate.emit(newValue);
  }
  buttonclickedHandler(event) {
    const index = this.getButtonIndex(event.target);
    if (this.multiple) {
      let set = new Set(this.values);
      if (event.target.classList.contains('button-group-active')) {
        set.delete(index);
      }
      else {
        set.add(index);
      }
      this.values = Array.from(set);
    }
    else {
      this.values = [index];
    }
  }
  getButtonIndex(element) {
    let index = 0;
    let prevNode = element.previousElementSibling;
    while (prevNode) {
      index++;
      prevNode = prevNode.previousElementSibling;
    }
    return index;
  }
  addActiveClass(activeButtonIndex) {
    const buttons = this.el.querySelectorAll('sas-button');
    const set = new Set(activeButtonIndex);
    buttons.forEach((button, index) => {
      if (set.has(index)) {
        button.classList.add('button-group-active');
      }
      else {
        button.classList.remove('button-group-active');
      }
    });
  }
  render() {
    return (h(Host, null, h("div", { class: "container" }, h("slot", null))));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "values": ["watchHandler"]
  }; }
};
SasButtonGroup.style = sasButtonGroupCss;

export { SasButtonGroup as sas_button_group };
