import { r as registerInstance, h, H as Host, g as getElement } from './index-82af0817.js';
import { a as animate } from './global-8d66eb74.js';

const sasCarouselCss = ":host{display:block}:host .container{position:relative;background-color:black}:host .slide-container{display:flex;align-items:center;width:100%;overflow-x:scroll;white-space:nowrap;-ms-overflow-style:none;scrollbar-width:none;}:host .slide-container::-webkit-scrollbar{display:none;}:host .icon{position:absolute;top:50%;transform:translateY(-50%);--icon-size:30px;--icon-color:white;opacity:0.5}:host .left-icon{left:20px}:host .right-icon{right:20px}:host .slide{flex:0 0 100%;max-height:100vh;max-width:100vw;object-fit:contain}:host .slide-info{display:grid;row-gap:5px;position:absolute;left:50%;transform:translateX(-50%);bottom:10px;color:white;text-align:center;opacity:0.5}";

let SasCarousel = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.originalScrollLeft = 0;
    this.startClientX = 0;
    /**
     * Slide index
     */
    this.index = 0;
    /**
     * Slides array
     */
    this.slides = [];
    this.goToLeftSlide = (event) => {
      event.stopPropagation();
      this.index = this.getValidIndex(this.index - 1);
    };
    this.goToRightSlide = (event) => {
      event.stopPropagation();
      this.index = this.getValidIndex(this.index + 1);
    };
    this.touchStartHandler = (event) => {
      event.preventDefault();
      const container = this.el.shadowRoot.querySelector('.slide-container');
      this.originalScrollLeft = container.scrollLeft;
      this.startClientX = event.touches[0].clientX;
    };
    this.touchMoveHandler = (event) => {
      event.preventDefault();
      const currentClientX = event.touches[0].clientX;
      const offset = this.startClientX - currentClientX;
      this.startClientX = currentClientX;
      const container = this.el.shadowRoot.querySelector('.slide-container');
      container.scrollLeft += offset;
    };
    this.touchEndHandler = (event) => {
      event.preventDefault();
      const container = this.el.shadowRoot.querySelector('.slide-container');
      const currentScrollLeft = container.scrollLeft;
      const slideWidth = container.getBoundingClientRect().width;
      if (this.originalScrollLeft < currentScrollLeft) {
        this.index = Math.ceil(currentScrollLeft / slideWidth);
      }
      else {
        this.index = Math.floor(currentScrollLeft / slideWidth);
      }
    };
  }
  componentDidLoad() {
    this.scroll(false);
  }
  watchValueHandler() {
    this.scroll();
  }
  getValidIndex(index) {
    const maxIndex = this.el.shadowRoot.querySelector('.slide-container').children.length;
    if (index < 0) {
      return maxIndex - 1;
    }
    if (index > maxIndex - 1) {
      return 0;
    }
    return index;
  }
  async scroll(smooth = true) {
    const container = this.el.shadowRoot.querySelector('.slide-container');
    const currentScrollLeft = container.scrollLeft;
    const slideWidth = container.getBoundingClientRect().width;
    const targetScrollLeft = slideWidth * this.index;
    const offset = targetScrollLeft - currentScrollLeft;
    if (smooth) {
      const drawFunction = function (progress) {
        container.scrollLeft = currentScrollLeft + progress * offset;
      };
      await animate({
        duration: 250,
        timing: function (timeFraction) {
          return timeFraction;
        },
        draw: drawFunction,
      });
    }
    else {
      container.scrollLeft = targetScrollLeft;
    }
  }
  render() {
    return (h(Host, { onTouchStart: this.touchStartHandler, onTouchMove: this.touchMoveHandler, onTouchEnd: this.touchEndHandler }, h("div", { class: "container" }, h("sas-icon", { class: "icon left-icon", icon: "fas fa-chevron-left", onClick: this.goToLeftSlide, onTouchEnd: this.goToLeftSlide }), h("div", { class: "slide-container" }, this.slides.map(slide => h("img", { class: "slide", src: slide.src }))), h("div", { class: "slide-info" }, h("div", null, this.slides[this.index].name), h("div", null, this.index + 1, " / ", this.slides.length)), h("sas-icon", { class: "icon right-icon", icon: "fas fa-chevron-right", onClick: this.goToRightSlide, onTouchEnd: this.goToRightSlide }))));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "index": ["watchValueHandler"]
  }; }
};
SasCarousel.style = sasCarouselCss;

export { SasCarousel as sas_carousel };
