import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-82af0817.js';
import { k as keydownEvent } from './global-8d66eb74.js';

const sasDateFieldCss = ":host{display:block}:host sas-form-input-template{--form-input-template-container-min-width:none;--form-input-template-container-max-width:370px;--input-height:var(--date-field-height, 40px)}:host .container{padding:30px 25px}:host .container:focus{outline:none}:host sas-icon{--icon-color:var(--s-500)}:host .title{display:grid;grid-template-columns:repeat(7, 1fr);justify-items:center;padding-bottom:25px}:host .title:focus,:host .date-table:focus{outline:none}:host .title-content{font-weight:bold;grid-column-start:3;grid-column-end:6;display:flex;align-items:center}:host .disabled{color:var(--s-300);pointer-events:none}";

let SasDateField = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.dateFieldUpdate = createEvent(this, "dateFieldUpdate", 7);
    this.datefieldupdate = createEvent(this, "datefieldupdate", 7);
    this.dateFieldUpdateEventUsedInternal = createEvent(this, "dateFieldUpdateEventUsedInternal", 7);
    this.titles = [
      { value: '1', text: 'Mo', sortAble: false },
      { value: '2', text: 'Tu', sortAble: false },
      { value: '3', text: 'We', sortAble: false },
      { value: '4', text: 'Th', sortAble: false },
      { value: '5', text: 'Fr', sortAble: false },
      { value: '6', text: 'Sa', sortAble: false },
      { value: '0', text: 'Su', sortAble: false },
    ];
    /**
     * Label
     */
    this.label = '';
    /**
     * Label position
     */
    this.outside = false;
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * name
     */
    this.name = '';
    /**
     * Value, ISO format "YYYY-MM-DDTHH:mm:ss.sssZ"
     */
    this.value = '';
    /**
     * Max value
     */
    this.max = '';
    /**
     * Min value
     */
    this.min = '';
    /**
     * disabled
     */
    this.disabled = false;
    /**
     * Clear
     */
    this.clear = false;
    /**
     * Icon
     */
    this.icon = 'far fa-calendar-alt';
    /**
     * Container vertical position
     */
    this.position = null;
    /**
     * Container horizontal position
     */
    this.positionH = null;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
      rangeUnderflow: false,
      rangeOverflow: false,
    };
    /**
     * Only shows when prop error is true, auto generated
     */
    this.message = '';
    /**
     * Custom error message for when a required value is missing. "message" will overide this if given.
     */
    this.valueMissingMessage = '';
    /**
      * Custom error message for when a max was set and the value exceeds it. "message" will overide this if given.
      */
    this.rangeOverflowMessage = '';
    /**
      * Custom error message for when a min was set and the value exceeds it. "message" will overide this if given.
      */
    this.rangeUnderflowMessage = '';
    this.goToPrevMonth = (event) => {
      event.stopPropagation();
      const year = this.localDate.getFullYear();
      const month = this.localDate.getMonth();
      const date = this.localDate.getDate();
      const prevMonthDate = new Date(year, month - 1, date);
      const minDate = new Date(this.min);
      if (prevMonthDate < minDate) {
        this.value = new Date(this.min).toISOString();
      }
      else {
        this.value = prevMonthDate.toISOString();
      }
    };
    this.goToNextMonth = (event) => {
      event.stopPropagation();
      const year = this.localDate.getFullYear();
      const month = this.localDate.getMonth();
      const date = this.localDate.getDate();
      const nextMonthDate = new Date(year, month + 1, date);
      const maxDate = new Date(this.max);
      if (nextMonthDate > maxDate) {
        this.value = new Date(this.max).toISOString();
      }
      else {
        this.value = nextMonthDate.toISOString();
      }
    };
    this.goToPrevYear = (event) => {
      event.stopPropagation();
      const year = this.localDate.getFullYear();
      const month = this.localDate.getMonth();
      const date = this.localDate.getDate();
      const prevYearDate = new Date(year - 1, month, date);
      const minDate = new Date(this.min);
      if (prevYearDate < minDate) {
        this.value = new Date(this.min).toISOString();
      }
      else {
        this.value = prevYearDate.toISOString();
      }
    };
    this.goToNextYear = (event) => {
      event.stopPropagation();
      const year = this.localDate.getFullYear();
      const month = this.localDate.getMonth();
      const date = this.localDate.getDate();
      const nextYearDate = new Date(year + 1, month, date);
      const maxDate = new Date(this.max);
      if (nextYearDate > maxDate) {
        this.value = new Date(this.max).toISOString();
      }
      else {
        this.value = nextYearDate.toISOString();
      }
    };
    this.handleButtonClick = (event) => {
      console.log("handleButtonClick: start");
      event.stopPropagation();
      if (!event.target.disabled) {
        console.log("handleButtonClick: inside if statement - start");
        this.localDate.setDate(Number(event.target.text));
        this.localDate.setHours(0, 0, 0, 0);
        this.value = this.localDate.toISOString();
        this.dateFieldUpdate.emit(this.value);
        this.datefieldupdate.emit(this.value);
        // usage 1: close date table after click if button is not disabled
        // usage 2: when used inside dropdown, keep focus so dropdown container will not close
        this.el.focus({ preventScroll: true });
        console.log("handleButtonClick: inside if statement - end");
      }
      console.log("handleButtonClick: end");
    };
  }
  // handle value set by js
  watchHandler() {
    this.updateLocalDate(this.value);
    this.resetErrorStatus();
    this.dateFieldUpdateEventUsedInternal.emit();
  }
  // handle pre set value
  componentWillLoad() {
    this.updateLocalDate(this.value);
    this.resetErrorStatus();
  }
  inputUpdateHandler(event) {
    if (event.detail) {
      this.value = new Date(event.detail + 'T00:00:00.000').toISOString();
    }
    else {
      this.value = '';
    }
    this.dateFieldUpdate.emit(this.value);
    this.datefieldupdate.emit(this.value);
  }
  /**
   * Trigger datefieldupdate event programmatically
   */
  async update() {
    const value = this.value;
    this.dateFieldUpdate.emit(value);
    this.datefieldupdate.emit(value);
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async validate() {
    let valueMissing = false;
    if (this.required && !Boolean(this.value)) {
      valueMissing = true;
    }
    let rangeUnderflow = false;
    if (Boolean(this.min) && Boolean(this.value)) {
      const inputDate = new Date(this.value);
      const minDate = new Date(`${this.min}T00:00:00`);
      if (inputDate < minDate) {
        rangeUnderflow = true;
      }
    }
    let rangeOverflow = false;
    if (Boolean(this.max) && Boolean(this.value)) {
      const inputDate = new Date(this.value);
      const maxDate = new Date(`${this.max}T23:59:59`);
      if (inputDate > maxDate) {
        rangeOverflow = true;
      }
    }
    this.validity = {
      valid: !valueMissing && !rangeUnderflow && !rangeOverflow,
      valueMissing: valueMissing,
      rangeUnderflow: rangeUnderflow,
      rangeOverflow: rangeOverflow,
    };
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    await this.validate();
    const formInputTemplate = this.el.shadowRoot.querySelector('sas-form-input-template');
    if (formInputTemplate !== null) {
      formInputTemplate.error = !this.validity.valid;
    }
    if (!this.validity.valid) {
      this.message = this.message !== "" ? this.message : this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    const formInputTemplate = this.el.shadowRoot.querySelector('sas-form-input-template');
    if (formInputTemplate !== null) {
      formInputTemplate.error = false;
    }
    this.message = '';
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      return this.valueMissingMessage !== '' ? this.valueMissingMessage : `${this.label} is required`;
    }
    if (validity.rangeOverflow) {
      return this.rangeOverflowMessage !== '' ? this.rangeOverflowMessage : `${this.label} must be on or before ${this.max}`;
    }
    if (validity.rangeUnderflow) {
      return this.rangeUnderflowMessage !== '' ? this.rangeUnderflowMessage : `${this.label} must be on or after ${this.min}`;
    }
  }
  updateLocalDate(value) {
    if (value) {
      const result = this.validateValue(value);
      if (result === 'invalid format') {
        const currentDate = new Date();
        let finalDate = currentDate;
        if (Boolean(this.min)) {
          const minDate = new Date(this.min);
          if (currentDate < minDate) {
            finalDate = minDate;
          }
        }
        if (Boolean(this.max)) {
          const maxDate = new Date(this.max);
          if (currentDate > maxDate) {
            finalDate = maxDate;
          }
        }
        this.value = '';
        this.localDate = finalDate;
      }
      else if (result === 'less than min') {
        this.localDate = new Date(this.min);
      }
      else if (result === 'larger than max') {
        this.localDate = new Date(this.max);
      }
      else {
        this.localDate = new Date(this.value);
      }
    }
    else {
      this.localDate = new Date();
    }
  }
  validateValue(value) {
    const dateRegex = /^[0-9]{4}[\-](0[1-9]|1[012])[\-](0[1-9]|[12][0-9]|3[01])T([01][0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])[\.]([0-9]{3})Z$/;
    if (!dateRegex.test(value)) {
      console.warn('The specified value does not conform to the required format, "YYYY-MM-DDTHH:mm:ss.sssZ"');
      return 'invalid format';
    }
    else {
      const inputDate = new Date(value);
      if (Boolean(this.min)) {
        const minDate = new Date(this.min);
        if (inputDate < minDate) {
          return 'less than min';
        }
      }
      if (Boolean(this.max)) {
        const maxDate = new Date(this.max);
        if (inputDate > maxDate) {
          return 'larger than max';
        }
      }
      return 'valid';
    }
  }
  checkIsDateLimit(dateLimit) {
    if (dateLimit) {
      const date = new Date(dateLimit);
      if (date.getFullYear() === this.localDate.getFullYear()
        && date.getMonth() === this.localDate.getMonth()
        && date.getDate() === this.localDate.getDate()) {
        return true;
      }
    }
    return false;
  }
  getLocalDateString(utcDate) {
    if (utcDate) {
      const dateObj = new Date(utcDate);
      if (dateObj.toString() === 'Invalid Date') {
        return '';
      }
      else {
        const year = dateObj.getFullYear();
        let formatYear = `${year}`;
        if (formatYear.length < 4) {
          formatYear = '0'.repeat(4 - formatYear.length) + formatYear;
        }
        const month = dateObj.getMonth() + 1;
        let formatMonth = `${month}`;
        if (month < 10) {
          formatMonth = `0${month}`;
        }
        const date = dateObj.getDate();
        let formatDate = `${date}`;
        if (date < 10) {
          formatDate = `0${date}`;
        }
        return `${formatYear}-${formatMonth}-${formatDate}`;
      }
    }
    else {
      return '';
    }
  }
  getTitleText() {
    const monthName = this.localDate.toLocaleString('default', { month: 'long' });
    const year = this.localDate.getFullYear();
    return `${monthName} ${year}`;
  }
  generateDateContent() {
    const year = this.localDate.getFullYear();
    const month = this.localDate.getMonth();
    const startDayInCurrentMonth = new Date(year, month, 1).getDay();
    const daysInCurrentMonth = new Date(year, month + 1, 0).getDate();
    const dateContent = [];
    let blankDaysFront = startDayInCurrentMonth - 1;
    if (startDayInCurrentMonth === 0) {
      blankDaysFront = 6;
    }
    let row = {};
    const isInMinDate = this.checkIsInMinMonth();
    const isInMaxDate = this.checkIsInMaxMonth();
    for (let i = 0; i < blankDaysFront + daysInCurrentMonth; i += 1) {
      let isDisabled = false;
      if (isInMinDate) {
        const minDate = new Date(this.min);
        if (i - blankDaysFront + 1 < minDate.getDate()) {
          isDisabled = true;
        }
      }
      if (isInMaxDate) {
        const maxDate = new Date(this.max);
        if (i - blankDaysFront + 1 > maxDate.getDate()) {
          isDisabled = true;
        }
      }
      if (i < blankDaysFront) {
        row[this.titles[i % 7].text] = '';
      }
      else {
        row[this.titles[i % 7].text] = h("sas-button", { class: `date-button ${this.getButtonClass(i - blankDaysFront + 1)}`, type: "text", text: `${i - blankDaysFront + 1}`, size: "small", disabled: isDisabled, onClick: this.handleButtonClick });
      }
      if ((i + 1) % 7 === 0 || i === blankDaysFront + daysInCurrentMonth - 1) {
        dateContent.push(row);
        row = {};
      }
    }
    return dateContent;
  }
  checkIsInMinMonth() {
    if (Boolean(this.min)) {
      const minDate = new Date(this.min);
      if (minDate.getFullYear() === this.localDate.getFullYear() && minDate.getMonth() === this.localDate.getMonth()) {
        return true;
      }
    }
    return false;
  }
  checkIsInMaxMonth() {
    if (Boolean(this.max)) {
      const maxDate = new Date(this.max);
      if (maxDate.getFullYear() === this.localDate.getFullYear() && maxDate.getMonth() === this.localDate.getMonth()) {
        return true;
      }
    }
    return false;
  }
  getButtonClass(date) {
    if (date === this.localDate.getDate()) {
      return 'date-button-active';
    }
    return '';
  }
  render() {
    console.log("sas-date-field: render()");
    return (h(Host, { tabindex: "0" }, h("sas-form-input-template", { label: this.label, outside: this.outside, required: this.required, value: this.getLocalDateString(this.value), iconRight: this.icon, type: "date", disabled: this.disabled, clear: this.clear, "data-message": this.message, position: this.position, "position-h": this.positionH, minimumPixelsAboveComponentRequired: 400, minimumPixelsRightOfComponentRequired: 300 }, h("div", { slot: "container", class: "container", tabindex: -1 }, h("div", { class: "title", tabindex: 0 }, h("sas-icon", { class: `prev-year-icon ${this.checkIsDateLimit(this.min) ? 'disabled' : ''}`, tabindex: 0, icon: "far fa-angle-double-left", onClick: this.goToPrevYear, onKeyDown: keydownEvent }), h("sas-icon", { class: `prev-month-icon ${this.checkIsDateLimit(this.min) ? 'disabled' : ''}`, tabindex: 0, icon: "far fa-angle-left", onClick: this.goToPrevMonth, onKeyDown: keydownEvent }), h("span", { class: "title-content" }, this.getTitleText()), h("sas-icon", { class: `next-month-icon ${this.checkIsDateLimit(this.max) ? 'disabled' : ''}`, tabindex: 0, icon: "far fa-angle-right", onClick: this.goToNextMonth, onKeyDown: keydownEvent }), h("sas-icon", { class: `next-year-icon ${this.checkIsDateLimit(this.max) ? 'disabled' : ''}`, tabindex: 0, icon: "far fa-angle-double-right", onClick: this.goToNextYear, onKeyDown: keydownEvent })), h("sas-table", { class: "date-table", headers: this.titles, items: this.generateDateContent(), tabindex: 0 })))));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "value": ["watchHandler"]
  }; }
};
SasDateField.style = sasDateFieldCss;

export { SasDateField as sas_date_field };
