import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-82af0817.js';
import { k as keydownEvent } from './global-8d66eb74.js';

const sasRadioCss = ":host{display:flex}:host .container{display:flex;gap:10px;flex-grow:1;align-items:center;cursor:pointer;position:relative}:host .container:after{content:attr(data-message);position:absolute;top:calc(100% + 5px);color:var(--li-2);font-size:12px;white-space:nowrap}:host .radio{border:1px solid var(--s-300);border-radius:50%;width:19px;height:19px;flex-shrink:0;align-self:flex-start;display:flex;justify-content:center}:host .radio:hover,:host .radio:focus{border-color:var(--s-400);box-shadow:0px 0px 5px #0000001A;outline:none}:host img{width:21px;height:21px;border-radius:50%;margin-left:-1px;margin-top:-1px;-webkit-user-select:none;-ms-user-select:none;user-select:none;}:host sas-icon{--icon-background-width:100%;--icon-background-height:100%;--icon-color:white;--icon-size:90%}:host .text{font-size:14px;color:var(--secondary)}:host([checked]:not([checked=false])) .radio{background-color:var(--primary);border-color:var(--primary)}:host([disabled]:not([disabled=false])) .container{pointer-events:none}:host([disabled]:not([disabled=false])) .radio{background-color:var(--s-200);border:1px solid var(--s-200)}:host([error]:not([error=false])) .text{color:var(--li-2)}:host([text-position=left]) .text{margin-right:10px}:host([text-position=right]) .text{margin-left:10px}:host([align=left]) .container{justify-content:flex-start}:host([align=right]) .container{justify-content:flex-end}:host([align=space-between]) .container{justify-content:space-between}";

let SasRadio = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.radioUpdate = createEvent(this, "radioUpdate", 7);
    this.radioupdate = createEvent(this, "radioupdate", 7);
    /**
     * name
     */
    this.name = null;
    /**
     * Value
     */
    this.value = null;
    /**
     *  Text
     */
    this.text = '';
    /**
     *  Text Position
     */
    this.textPosition = 'right';
    /**
     *  Align
     */
    this.align = 'left';
    /**
     *  Check Status
     */
    this.checked = false;
    /**
     *  Disabled
     */
    this.disabled = false;
    /**
     *  Required
     */
    this.required = false;
    /**
     *  Error
     */
    this.error = false;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
    };
    /**
     * Only shows when prop error is true, auto generated
     */
    this.message = '';
    this.tabindexNumber = 0;
    this.clickEvent = () => {
      if (!this.checked) {
        this.checked = true;
        const radioInfo = {
          value: this.value,
          text: this.text,
        };
        this.radioUpdate.emit(radioInfo);
        this.radioupdate.emit(radioInfo);
      }
    };
  }
  watchHandler() {
    this.resetErrorStatus();
  }
  componentWillRender() {
    this.tabindexNumber = 0;
    if (this.disabled) {
      this.tabindexNumber = -1;
    }
  }
  /**
   * Validate
   */
  async validate() {
    let valueMissing = false;
    if (this.required && !this.checked) {
      valueMissing = true;
    }
    this.validity = {
      valid: !valueMissing,
      valueMissing: valueMissing,
    };
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    await this.validate();
    this.error = !this.validity.valid;
    if (!this.validity.valid) {
      this.message = this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    this.error = false;
    this.message = '';
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      return `Please check this field`;
    }
  }
  render() {
    return (h(Host, null, h("div", { class: "container", "data-message": this.message, onClick: this.clickEvent, onKeyDown: keydownEvent }, this.textPosition === 'left' && Boolean(this.text)
      ? h("div", { class: "text" }, this.text)
      : null, h("div", { class: "radio", tabindex: this.tabindexNumber }, this.checked
      ? h("sas-icon", { icon: "far fa-check" })
      : null), this.textPosition === 'right' && Boolean(this.text)
      ? h("div", { class: "text" }, this.text)
      : null)));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "checked": ["watchHandler"]
  }; }
};
SasRadio.style = sasRadioCss;

export { SasRadio as sas_radio };
