import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-82af0817.js';
import { k as keydownEvent } from './global-8d66eb74.js';

const sasSwitchCss = ":host{display:inline-block}:host .container{position:relative;display:inline-block;width:30px;height:16px;border-radius:8px;background-color:var(--s-200);-webkit-transition:0.4s;transition:0.4s}:host .container:after{content:attr(data-message);position:absolute;top:calc(100% + 5px);color:var(--li-2);font-size:12px;white-space:nowrap}:host .switch{position:absolute;content:\"\";height:14px;width:14px;left:1px;bottom:1px;border-radius:50%;background-color:white;-webkit-transition:0.4s;transition:0.4s}:host .switch:focus,:host .switch:hover{box-shadow:0px 0px 3px 3px var(--switch-background);outline:none}:host([checked]:not([checked=false])) .container{background-color:var(--switch-background)}:host([checked]:not([checked=false])) .switch{-webkit-transform:translateX(14px);-ms-transform:translateX(14px);transform:translateX(14px)}:host([error]:not([error=false])) .container{background-color:var(--li-2)}";

let SasSwitch = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.switchUpdate = createEvent(this, "switchupdate", 7);
    /**
     * name
     */
    this.name = null;
    /**
     *  Check Status
     */
    this.checked = false;
    /**
     *  Required
     */
    this.required = false;
    /**
     *  Error
     */
    this.error = false;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
    };
    /**
     * Only shows when prop error is true, auto generated
     */
    this.message = '';
    this.clickEvent = () => {
      this.checked = !this.checked;
      this.switchUpdate.emit(this.checked);
    };
  }
  watchHandler() {
    this.resetErrorStatus();
  }
  /**
   * Validate
   */
  async validate() {
    let valueMissing = false;
    if (this.required && !this.checked) {
      valueMissing = true;
    }
    this.validity = {
      valid: !valueMissing,
      valueMissing: valueMissing,
    };
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    await this.validate();
    this.error = !this.validity.valid;
    if (!this.validity.valid) {
      this.message = this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    this.error = false;
    this.message = '';
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      return `Please check this field`;
    }
  }
  render() {
    return (h(Host, { onClick: this.clickEvent }, h("div", { class: "container", "data-message": this.message }, h("div", { class: "switch", tabindex: 0, onKeyDown: keydownEvent }))));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "checked": ["watchHandler"]
  }; }
};
SasSwitch.style = sasSwitchCss;

export { SasSwitch as sas_switch };
